/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.hd.d.pg2k.ai.scorer.fixed;

import java.io.IOException;
import java.util.*;

import org.hd.d.pg2k.ai.scorer.*;
import org.hd.d.pg2k.svrCore.datasource.SimpleExhibitPipelineIF;

/**Fixed score given by its single parameter.
 * This "fake" Scorer should represent the highest "goodness"
 * that a fake/dumb implementation (that does not look at its input) can reach;
 * a real Scorer must do better than this to be credible.
 *
 * @author dhd
 */
public final class FixedScore extends AbstractScorer implements BadScorer
    {
    /**Create simple non-parameterised instance. */
    public FixedScore()
        {
        // All parameters get default values.
        resultScoreParam = resultScoreParamBounds;
        }

    /**Create parameterised version. */
    public FixedScore(final String nameAndParameters)
        {
        super(nameAndParameters);

        // Assign parameters from the captured map where possible, using default values otherwise.
        resultScoreParam = (ScorerParamInteger) resultScoreParamBounds.parse(paramValueMap.get(resultScoreParamBounds.name));
        }


    /**Respond with the fixed score based on the parameter; never null. */
    public ScoreAndConf computeScoreAndConfidence(SimpleExhibitPipelineIF dataSource, String exhibitName) throws IOException
        { return(new ScoreAndConf((((2*resultScoreParam.value)-(resultScoreParamBounds.max)) * ScoreAndConf.MAX) / resultScoreParamBounds.max, ScoreAndConf.MAX)); }


    /**Min, default and max result score; never null. */
    private static final ScorerParamInteger resultScoreParamBounds =
        new ScorerParamInteger(0, 50, 100, "resultScore");

    /**Fixed result score for this variant; never null. */
    private final ScorerParamInteger resultScoreParam;

    /**Get parameter definitions and values (immutable) for this Scorer; never null. */
    public Collection<ScorerParam> getParameterDefsAndValues()
        { return(Collections.singleton((ScorerParam) resultScoreParam)); }
    }
