package org.hd.d.pg2k.clApp.uploader;

/**
 * Created by IntelliJ IDEA.
 * User: DHD
 * Date: 05-Oct-2006
 * Time: 19:59:06
 */

import org.hd.d.pg2k.svrCore.props.GenProps;
import org.hd.d.pg2k.svrCore.uploader.UploadInfoBean;

import javax.jnlp.FileContents;
import javax.jnlp.JNLPRandomAccessFile;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;

/**Class to help DnD (drag-and-drop) files into the JWS uploader.
 * Package-visible for access by the GUI classes.
 * <p>
 * TODO: Extend this to accept URLs, eg dragged from a browser.
 */
class FileTransferHandler extends TransferHandler
    {
    private static final long serialVersionUID = 1151591254166507360L;

    /**The business-logic component; never null. */
    private final UploaderLogic logic;

    /**The model for the table so we can force a redraw; never null. */
    final AbstractTableModel uploadTableModel;

    /**The "auto" file-guess mode check box; never null. */
    final JCheckBox autoSuffixCheckBox;

    /**The exhibit description bean; never null. */
    final UploadInfoBean uib;

    /**Create a new instance ready to receive files and drop them into the selected-files database. */
    FileTransferHandler(final JCheckBox autoSuffixCheckBox,
                        final AbstractTableModel uploadTableModel,
                        final UploaderLogic logic,
                        final UploadInfoBean uib)
        {
        if((autoSuffixCheckBox == null) ||
           (uploadTableModel == null) ||
           (logic == null) ||
           (uib == null))
            { throw new IllegalArgumentException(); }
        this.autoSuffixCheckBox = autoSuffixCheckBox;
        this.uploadTableModel = uploadTableModel;
        this.logic = logic;
        this.uib = uib;
        }

    /**Attempt to import file lists, or http and ftp URLs.
     */
    @SuppressWarnings("unchecked")
    public boolean importData(final JComponent c, final Transferable t)
        {
        // If we can't handle this data then return false.
        if(!canImport(c, t.getTransferDataFlavors()))
            { return(false); }

        // FIXME: Load the files in another thread so as not to block the UI.
        try
            {
            if(hasFileFlavor(t.getTransferDataFlavors()))
                {
                final java.util.List<File> files =
                    (java.util.List<File>) t.getTransferData(DataFlavor.javaFileListFlavor);

                // Wrap the files up as FileContents values...
                final FileContents filesFC[];
                // If we appear to be running in JNLP,
                // then do this through the ExtendedService,
                if(logic.exs != null)
                    {
                    final File f[] = new File[files.size()];
                    files.toArray(f);
                    filesFC = logic.exs.openFiles(f);
                    }
                // else do it directly...
                else
                    {
                    filesFC = new FileContents[files.size()];
                    for(int i = files.size(); --i >= 0; )
                        { filesFC[i] = new NormalFileContent(files.get(i)); }
                    }

                // Add to the database...
                logic.addSelectedFiles(autoSuffixCheckBox.isSelected(),
                                       filesFC,
                                       uib);

                // Get the table re-drawn with the new data.
                uploadTableModel.fireTableDataChanged();

                return(true); // We got the data!
                }
            else if(hasStringFlavor(t.getTransferDataFlavors()))
                {
//                System.out.println("importData: got string data flavor");
                final String s = (String) t.getTransferData(DataFlavor.stringFlavor);
                // Assume that the URL is first component;
                // other text may follow after whitespace.
                final URL u = new URL(s.split("[ \\r\\n\\t]")[0]);
                final String protocol = u.getProtocol();
                if("http".equals(protocol) || "ftp".equals(protocol))
                    {
                    System.out.println("importData: got URL: " + u);
                    logic.logger.log("Selected URL: "+u);

                    // Maximum site of object that we can support upload of
                    // if we have to buffer it up first.
                    // Function of how much memory we have and other factors.
                    final int maxLengthToBuffer = (int) Math.min(16*1024*1024, Runtime.getRuntime().totalMemory()/4);

                    // Create single-"file" list from the URL passed to us.
                    final FileContents fileFC[] = new FileContents[]{
                        new FileContents(){
                            /**If we have read and buffered the content, this is non-null. */
                            private byte content[];

                            public boolean canRead() throws IOException
                                { return(true); }

                            public boolean canWrite() throws IOException
                                { return(false); }

                            /**Get stream for the remote URL each time. */
                            public InputStream getInputStream() throws IOException
                                {
                                // If we haven't buffered the data, return a stream.
                                if(content == null) { return(u.openStream()); }
                                // Return the buffered data.
                                return(new ByteArrayInputStream(content));
                                }

                            /**Get length of remote object if possible. */
                            public long getLength() throws IOException
                                {
                                final int declLength = u.openConnection().getContentLength();
                                if(declLength > -1) { return(declLength); }
                                // If we cannot get the Content-Length
                                // then we may actually have to fetch the data!
                                System.out.println("Trying to download item to find its length: "+u);
                                logic.logger.log("Could not get Content-Length, so will try to download item; max length: ~"+maxLengthToBuffer);

                                // Buffer to collect the input.
                                final byte buf[] = new byte[1<<13];
                                final ByteArrayOutputStream baos = new ByteArrayOutputStream(65536);
                                // Get the input.
                                final InputStream is = u.openStream();
                                while(baos.size() < maxLengthToBuffer)
                                    {
                                    final int n = is.read(buf);
                                    if(n < 1)
                                        {
                                        content = baos.toByteArray();
                                        return(content.length);
                                        }
                                    baos.write(buf, 0, n);
                                    }
                                // Could not compute the input length (too large).
                                logic.logger.log("Could not compute item length (too large)...");
                                return(-1);
                                }

                            public long getMaxLength() throws IOException
                                { throw new UnsupportedOperationException(); }

                            /**Return full URL as name. */
                            public String getName() throws IOException
                                { return(s); }

                            public OutputStream getOutputStream(boolean arg0) throws IOException
                                { throw new UnsupportedOperationException("no output possible"); }

                            public JNLPRandomAccessFile getRandomAccessFile(String arg0) throws IOException
                                { throw new UnsupportedOperationException(); }

                            public long setMaxLength(long arg0) throws IOException
                                { throw new UnsupportedOperationException(); }
                            }
                        };

                    // Add name/contents of URL to the database...
                    logic.addSelectedFiles(autoSuffixCheckBox.isSelected(),
                                           fileFC,
                                           uib);

                    // Get the table re-drawn with the new data.
                    uploadTableModel.fireTableDataChanged();

                    System.out.println("importData: done. ");
                    return(true); // Succeeded.
                    }
                return(false); // Cannot import the data...
                }
            }
        catch(final UnsupportedFlavorException ufe)
            {
            System.err.println("importData: unsupported data flavor");
            }
        catch(final IOException ieo)
            {
            System.err.println("importData: I/O exception");
            }
        return(false); // Cannot import the data...
        }

    /**Returns true iff we can import at least one of the specified data flavors.
     * This will be false unless:
     * <ul>
     * <li>The data is presented as a file list or a String (potential URL).
     * <li>Our uib is in a state to make a valid, unique exhibit name.
     * </ul>
     */
    public boolean canImport(final JComponent c, final DataFlavor[] flavors)
        {
        // If the uib is not yet capable of generating us a good exhibit name
        // then return false.
        if(!uib.enoughValidUniqueInfo())
            { return(false); }

        if(!hasFileFlavor(flavors) && !hasStringFlavor(flavors))
            { return(false); }

        return(true); // Seems OK!
        }

    private boolean hasFileFlavor(final DataFlavor[] flavors)
        {
        for(int i = flavors.length; --i >= 0; )
            {
            if(DataFlavor.javaFileListFlavor.equals(flavors[i]))
                { return(true); }
            }
        return(false);
        }

    /**To accept URLs as text strings? */
    private boolean hasStringFlavor(final DataFlavor[] flavors)
        {
        for(int i = flavors.length; --i >= 0; )
            {
            if(flavors[i].isFlavorTextType())
//            if(DataFlavor.stringFlavor.equals(flavors[i]))
                { return(true); }
            }
        return(false);
        }
    }

