/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.hd.d.pg2k.clApp.uploader;

import javax.swing.table.AbstractTableModel;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by IntelliJ IDEA.
 * User: DHD
 * Date: 26-Sep-2006
 * Time: 11:51:09
 */

/**Table model used for JTable displays of files selected for upload.
 * Most access is package-only for access by the GUI classes.
 */
class UploadFileTableModel extends AbstractTableModel
    {
    private static final long serialVersionUID = -2592921016866914042L;

    /**Construct an instance of our table model. */
    UploadFileTableModel(final UploaderLogic l,
               final SelectedFilesDB db,
               final AtomicInteger highlightRow)
        {
        if((l == null) || (db == null))
            { throw new IllegalArgumentException(); }
        this.logic = l;
        this.db = db;
        this.highlightRow = highlightRow;
        }

    /**The "business logic". */
    final UploaderLogic logic;

    /**The backing database. */
    final SelectedFilesDB db;

    /**The holder for the row to highlight, or null. */
    final AtomicInteger highlightRow;

    /**Return the number of columns; fixed for this model. */
    public int getColumnCount() { return(SelectedFileDetails.getColumnCount()); }

    /**Return a column name. */
    public String getColumnName(final int col) { return(SelectedFileDetails.getColumnName(col)); }

    public int getRowCount() { return(db.size()); }

    /**Get specified value; null if none.
     * Uses a lock on the selectedFiles object
     * to ensure that no race can cause an exception.
     */
    public Object getValueAt(final int row, final int col)
        {
        // Is this the the row to highlight?
//            final boolean isHighlightRow = (row == uploadTableEditRow.get());
        final boolean isHighlightRow =
            (highlightRow != null) && (row == highlightRow.get());

        // Hold lock just long enough for race-free access to row.
        final Object valueAt;
        synchronized(db)
            {
            if(row >= db.size()) { return(null); }
            valueAt = db.get(row).getValueAt(logic, col);
            }

        // Format String data for display...
        if(valueAt instanceof String)
            {
            if(isHighlightRow &&
               ((col == SelectedFileDetails.COLNUM_EXHIBIT_NAME) ||
                (col == SelectedFileDetails.COLNUM_DESCRIPTION)))
                {
                // Highlight (simple String) items in the edit row...
                return("<html><strong>" + valueAt + "</strong></html>");
                }
            else if(col == SelectedFileDetails.COLNUM_DESCRIPTION)
                {
                // Always show description as HTML...
                return("<html>" + valueAt + "</html>");
                }
            else if(col == SelectedFileDetails.COLNUM_STATUS)
                {
                // Show any error in red...
                return("<html><font color=red>" + valueAt + "</font></html>");
                }
            }
        return(valueAt); // Return value as-is...
        }
    }
