/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.svrCore;

/**
 * Created by IntelliJ IDEA.
 * User: DHD
 * Date: 24-Oct-2006
 * Time: 12:51:57
 */

/**A ranked series of compression methods/levels used in PG2K.
 * This applies to such things as tunnel-frame content
 * and HTTP transport encodings.
 * <p>
 * Higher numeric levels potentially give greater compression
 * at potentially higher resource cost (especially in the compressor),
 * though YMMV.
 * <p>
 * The order of elements is from lowest to highest compression.
 */
public enum CompressionLevel
    {
    /**No compression, thus free to compress/decompress but possibly expensive to store or transmit. */
    NONE(0),

    /**GZIP/deflate/zlib-style compresssion; widely used and generally cheap and effective. */
    ZIP(1),

    /**BZIP2-style compresssion; more expensive than ZIP (maybe 5x CPU) but more effective on some data sets. */
    BZIP2(2),

    /**LZMA (7-Zip-style) compresssion; more expensive than ZIP and less tested, but much more effective on some data sets. */
    LZMA(3),

    /**PPM compresssion; very expensive in CPU and memory, but about the best compression available. */
    PPM(4);


    private CompressionLevel(final int level) { this.level = level; }

    /**The "level", higher meaning generally better and more expensive; 0 indicates none. */
    private final int level;

    /**Get the "level", higher meaning generally better and more expensive; 0 indicates none. */
    public final int getLevel() { return(level); }
    }
