/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.svrCore;

import javax.naming.InitialContext;

/**This holds some JNDI utilities.
 */
public final class JNDIUtils
    {
    /**Looks up an ejb name with an without a java:comp/env/ prefix.
     * This hides differences in behaviour between (for example),
     * Sun's 1.3 beta RI which uses names with the prefix throughout,
     * and JBoss 2.2.1 WAR and command-line client access which does
     * not, whereas inter-bean access does!
     * <p>
     * This returns a (non-null) object reference suitable for narrowing
     * given a short name such as ejb/MyBean.
     */
    public static Object getObjref(String shortName)
        throws javax.naming.NamingException
        {
        final InitialContext ctx = new InitialContext();
        Object objref = null;
        try { objref = ctx.lookup(shortName); }
        catch(javax.naming.NameNotFoundException e)
            { objref = ctx.lookup(CoreConsts.JNDI_COMPENV_PREFIX + shortName); }
        return(objref);
        }

    }
