/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.svrCore.mediahandler;

import org.hd.d.pg2k.svrCore.MIME.AbstractImageHandler;
import org.hd.d.pg2k.svrCore.MIME.ExhibitMIME;

/**Basic handler for BMP file (MIME type image/bmp).
 * Mainly relies on JAI/IIO support for encode/decode.
 */
public final class bmp extends AbstractImageHandler
    {
    /**Get internal type of ExhibitMIME type; never null. */
    public ExhibitMIME.ExhibitTypeParameters getExhibitType()
        { return(ExhibitMIME.getParamsByType(ExhibitMIME.ET_BMP)); }

    /**Returns true if this can make thumbnail/sample exhibits for this type.
     * Returns true because we should be able to make BMP thumbnails
     * with the built-in imageIO support from JDK1.5 onwards.
     */
    @Override public boolean canMakeThumbnails()
        { return(true); }

    /**Gather together thumbnail parameters. */
    private static final AbstractImageHandler.ThumbnailParams thumbnailParams =
        new ThumbnailParams(4, // Assume 32-bit in-memory RGB representation.
            1,
            100,
            8, // 8 bits per pixel should be fine for a thumbnail.
            8, // We ought to be able to force an 8-bit format at least...
            256);

    /**Get ThumbnailParameters for BMP handler. */
    @Override public AbstractImageHandler.ThumbnailParams getThumbnailParams()
        { return(thumbnailParams); }

    // Inherit javadoc.
    @Override protected int _reduceColoursQualityThreshold() { return(24); }
    }
