/*
 * Created by IntelliJ IDEA.
 * User: d@hd.org
 * Date: 21-May-02
 * Time: 21:35:07

Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */
package org.hd.d.pg2k.svrCore.uploader;

import org.hd.d.pg2k.svrCore.FileTools;

/**Constants pertaining to exhibit upload.
 * Some of these may be set dynamically in future, ie
 * from GenProps or similar.
 */
public final class UploaderConsts
    {
    /**Hopefully-unique (but constant between runs) window name to place search page in. */
    public static final String searchWindow = "searchPaGe4UpLoad";

    /**Attribute (and form field) name under which we store (verified) contributor ID (initials).
     * Prefix with "X-" to use as an HTTP header.
     */
    public static final String USERID_FIELD_NAME = "userID";

    /**Form field name for user password.
     * Prefix with "X-" to use as an HTTP header.
     */
    public static final String PASSWD_FIELD_NAME = "userPass";

    /**Attribute name under which we store UploadInfoBean. */
    public static final String attrUIB = "uib";

    /**Name of parameter containing exhibit file for upload. */
    public static final String exhibitFile = "exhibitFile";

    /**Field name to be present to upload or login pages to force a logout. */
    public static final String LOGOUT_FIELD_NAME = "logout";

    /**Header name on login page to indicate user is logged in.
     * Must start with "X-" to avoid breaking any other code.
     */
    public static final String LOGGED_IN_HEADER = "X-LoggedIn";

    /**Login page root-relative URL (starting with '/'). */
    public static final String LOGIN_URL = "/_upload/login.jsp";

    /**Minimum free space we will contemplate even attempting an upload at.
     * Note that this had probably better be at least a couple of blocks,
     * to represent filesystem consumption for directory and file data,
     * and anyway tiny uploads are not likely to be much use!
     */
    public static final int MIN_UPLOAD_SPACE_BYTES =
        2 * FileTools.FS_EST_BLOCK_SIZE_BYTES;

    /**The prefix to look up the prettyfied/i18ned version of a section title.
     * If present, the title is found with a property name of this prefix
     * on the section directory.
     */
    public static final String SECTION_TITLE_PROPNAME_PREFIX = "org.hd.pg2k.section.title.";

    /**The prefix to look up the section description, if any.
     * If present, the description is found with a property name of this prefix
     * on the section directory.
     */
    public static final String SECTION_DESC_PROPNAME_PREFIX = "org.hd.pg2k.section.description.";
    /**Prefix of i18n-ed attribute word descriptions.
     * The required attribute word can be appended to this and looked up in the common catalogue
     * for a (possibly localised) description: many not be present for all attributes.
     */
    public static final String ATTR_I18N_DESC_PREFIX = "org.hd.pg2k.attribute.word.description.";

    /**Prefix of i18n-ed symbolic filter times.
     * The symbolic name should be appended to this and looked up in the common catalogue
     * for a (possibly localised) description: many not be present for all attributes.
     */
    public static final String SYMTIME_I18N_PREFIX = "common.cat.filtersince.recent.";

    /**If true, then we can pipeline multiple batch uploads in one POST to the non-browser upload servlet. */
    public static final boolean BATCH_UPLOADS_CAN_BE_PIPELINED = true;

    /**Maximum time uploader connection should stay idle before being shut, ms; strictly positive.
     * As connections for batch uploads are non-interactive
     * this need only be long enough to allow for:
     * <ul>
     * <li>Glitches in connectivity and TCP/IP data flow.
     * <li>Client and server turnaround time moving to the next upload.
     * </ul>
     * <p>
     * This period applies while no upload is in progress;
     * a somewhat longer timeout should apply during upload
     * to avoid glitches forcing an upload to be started again later.
     * <p>
     * A few tens of seconds should suffice,
     * and help free unused resources quickly.
     */
    public static final int BATCH_UPLOAD_MAX_IDLE_MS = 30000;
    }
