/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.svrCore.vars;

/**
 * Created by IntelliJ IDEA.
 * User: Damon
 * Date: 30-Dec-2004
 * Time: 18:03:11
 */

/**An enumeration of the available event periods from (very) short to (very) long.
 * The actual event interval in milliseconds is available from the period value,
 * as are some subsidiary methods.
 * <p>
 * Our on-the-wire tunnel (inter-peer) protocol may depend on the value ordinals,
 * ie be very careful when altering this enum in any way.
 */
public enum EventPeriod
    {
    VSHORT(SystemVariables.EVENT_INTERVAL_VSHORT_TERM_MS),
    SHORT(SystemVariables.EVENT_INTERVAL_SHORT_TERM_MS),
    MEDIUM(SystemVariables.EVENT_INTERVAL_MEDIUM_TERM_MS),
    LONG(SystemVariables.EVENT_INTERVAL_LONG_TERM_MS),
    VLONG(SystemVariables.EVENT_INTERVAL_VLONG_TERM_MS);

    private EventPeriod(final int intervalMs) { this.intervalMs = intervalMs; }

    /**The interval for this event period in milliseconds; strictly positive. */
    private final int intervalMs;

    /**Get the interval for this event period in milliseconds; strictly positive. */
    public final int getIntervalMs() { return(intervalMs); }

    /**Compute the interval number given a time in milliseconds within that interval.
     * The time passed must be positive.
     */
    public final long getIntervalNumber(final long time)
        {
        if(time <= 0) { throw new IllegalArgumentException(); }
        return(time / intervalMs);
        }

    /**Get start time (ms) of specified interval number.
     * The interval time passed must be positive,
     * and small enough to avoid overflow in the returned time value.
     */
    public final long getIntervalStartTime(final long intervalNumber)
        {
        if(intervalNumber <= 0) { throw new IllegalArgumentException(); }
        if(Long.MAX_VALUE / intervalMs < intervalNumber) { throw new IllegalArgumentException(); }
        return(intervalMs * intervalNumber);
        }
    }
