/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.svrCore.vars;

import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: Damon
 * Date: 19-Aug-2004
 * Time: 14:11:03
 */

/**Simple interface implemented by several stages in the Web distributed-variable-handling pipeline.
 * The pipeline stages that implement this may well implement other interfaces
 * in parallel to provide richer and more efficient cacheing, etc.
 */
public interface SimpleVariablePipelineIF extends BasicVarMgrInterface
    {
    /**Synchronise variables with upstream values.
     * Pushes updated values upstream to the source,
     * calls sync on the source if called with the "force" argument true,
     * and then retrieves changed values from upstream.
     * <p>
     * When called with force==true, this acts like a full "memory barrier",
     * flushing all write-cached items downstream immediately and afterwards
     * getting the value of all upstream values with getVariables(-1),
     * but may be expensive in terms of CPU or bandwidth, so use sparingly.
     * <p>
     * When called with force==false, this incrementally flushes outstanding
     * writes and will then fetch all, or only new, values from upstream,
     * so is potentally much less resource-intensive.
     * In particular, this does not propagate the sync() upstream.
     * <p>
     * In any case, it is rarely the right thing for a casual user
     * to vall this as it may be very expensive.
     *
     * @param force  if true, this will force a full write flush,
     *     a full sync upstream,
     *     then full read with getVariables(-1),
     *     to get the effect of a full "barrier";
     *     otherwise, in general, a more incremental and non-propagating
     *     mode is used which still does a write flush but may chose
     *     to do a partial read of "new" upstream values
     *
     * @throws java.io.IOException if one is received from upstream
     */
    public void syncVariables(final boolean force)
        throws IOException;
    }
