/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: 28-Dec-02
 * Time: 22:24:51
 * To change template for new class use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package org.hd.d.pg2k.test.dev;

import junit.framework.TestCase;

import org.hd.d.pg2k.ai.scorer.ScorerCacheIF;
import org.hd.d.pg2k.svrCore.*;
import org.hd.d.pg2k.svrCore.datasource.ExhibitDataFileSource;
import org.hd.d.pg2k.svrCore.datasource.SimpleExhibitPipelineIF;
import org.hd.d.pg2k.svrCore.props.GenProps;
import org.hd.d.pg2k.svrCore.uploader.ExhibitHandlerBeanBase;
import org.hd.d.pg2k.svrCore.vars.BasicVarMgr;
import org.hd.d.pg2k.svrCore.vars.BasicVarMgrInterface;
import org.hd.d.pg2k.svrCore.vars.SimpleVariableValue;
import org.hd.d.pg2k.svrCore.vars.SystemVariables;
import org.hd.d.pg2k.webSvr.catalogue.SearchPageJavaBean;

import java.util.*;

/**Test of filtering of exhibits (eg for search page).
 * Tests that the filters behave as expected.
 */
public final class ExhibitFilterTest extends TestCase
    {
    public ExhibitFilterTest(String name)
        {
        super(name);
        }

    // Our test data set; null when not running the tests to save memory...
    private AllExhibitProperties aep;

    // Our private empty but non-zero-timestamp GenProps set.
    private GenProps gp;

    // Names of some notable members of our test set that we look for specifically...
    /**A small single-word-name JPEG by DHD with timestamp of now. */
    private static final String EX_SMALL_DHD_JPEG_NOW = "example/example-DHD.jpg";
    /**A small single-word GIF by DHD a fortnight old. */
    private static final String EX_SMALL_DHD_GIF_2WOLD = "example/2wold-DHD.gif";
    /**A small single-word HTML fragment by ANON from the dawn of timeS (1998). */
    private static final String EX_SMALL_ANON_HTXT_1998 = "example/1998file-blah-blah-ANON.htxt";

    /**Time this class was instantiated. */
    private final static long now = System.currentTimeMillis();

    /**Time representing 19980101. */
    private static final long year1998 = (new GregorianCalendar(1998, 1, 1)).getTime().getTime();

    /**Do any setup needed for the tests. */
    protected void setUp()
        {
        aep = _makeAEP(null);
        gp = new GenProps(new Properties(), System.currentTimeMillis());
        }

    /**Make our standard test AEP value. */
    private static AllExhibitProperties _makeAEP(final AllExhibitProperties oldAEP)
        {
        final Map<String, ExhibitStaticAttr> exhibits = new HashMap<String, ExhibitStaticAttr>();
        exhibits.put(EX_SMALL_DHD_JPEG_NOW, new ExhibitStaticAttr(EX_SMALL_DHD_JPEG_NOW, 100, now));
        exhibits.put(EX_SMALL_DHD_GIF_2WOLD, new ExhibitStaticAttr(EX_SMALL_DHD_GIF_2WOLD, 100, now - 14 * SearchPageJavaBean.DAY_MS));
        exhibits.put(EX_SMALL_ANON_HTXT_1998, new ExhibitStaticAttr(EX_SMALL_ANON_HTXT_1998, 123, year1998));

        final Map<String,ExhibitPropsLoadable> noLoadedProps = Collections.emptyMap();
        final Map<String,ExhibitPropsComputable> noComputedProps = Collections.emptyMap();
        return (new AllExhibitProperties(oldAEP,
                                         new ExhibitPropsGlobalImmutable(),
                                         new AllExhibitImmutableData(exhibits, now),
                                         noLoadedProps, noComputedProps, 0));
        }

    /**Do any clearup needed after the tests. */
    protected void tearDown()
        {
        // cleanup code
        aep = null;
        gp = null;
        }

    /**Do some tests on evaluations of newness, goodness, etc, of exhibits using ExhibitPropsComputableMutable.
     */
    public void testSimpleExhibitPropsComputableMutableExhibitInspection()
        {
        // Get details of new (right-up-to-date) exhibit.
        final ExhibitStaticAttr esaNew = aep.aeid.getStaticAttr(EX_SMALL_DHD_JPEG_NOW);
        final ExhibitPropsComputableMutable epcmNew =
                ExhibitPropsComputableMutable.generateFastApproximation(esaNew, new GenProps());
//        assertTrue("new exhibit must be seen as new", epcmNew.isExhibitNew());
        assertTrue("new-exhibit goodness must be in [-1,+1] range",
                   (epcmNew.getGoodnessAsFloat() >= -1) &&
                   (epcmNew.getGoodnessAsFloat() <= +1));
//        System.out.println("[Example 'new' simple goodness value: "+epcmNew.getGoodnessAsFloat()+" ("+epcmNew.getGoodness()+")]");
        // Get details of old exhibit.
        final ExhibitStaticAttr esaOld = aep.aeid.getStaticAttr(EX_SMALL_ANON_HTXT_1998);
        final ExhibitPropsComputableMutable epcmOld =
                ExhibitPropsComputableMutable.generateFastApproximation(esaOld, new GenProps());
//        assertFalse("old exhibit must be seen as old", epcmOld.isExhibitNew());
        assertTrue("old-exhibit goodness must be in [-1,+1] range",
                   (epcmOld.getGoodnessAsFloat() >= -1) &&
                   (epcmOld.getGoodnessAsFloat() <= +1));
//        System.out.println("[Example 'old' simple goodness value: "+epcmOld.getGoodnessAsFloat()+" ("+epcmOld.getGoodness()+")]");
        }

    /**Test differentials in "goodness" induced via GenProps.
     * Tests that the smallest possible changes in static "goodness" attributes
     * always show up in the goodness score and overwhelm any "random" factors.
     * <p>
     * Tests firstly for author weightings.
     */
    public void testSimpleExhibitPropsComputableMutableGoodnessDifferentials()
        {
        for(final Iterator<ExhibitStaticAttr> it = aep.aeid.getAllStaticAttrs().iterator(); it.hasNext(); )
            {
            final ExhibitStaticAttr esa = it.next();
            // Calculate "base" goodness with empty GenProps.
            final ExhibitPropsComputableMutable epcmBase =
                    ExhibitPropsComputableMutable.generateFastApproximation(esa, new GenProps());
            final int gBase = epcmBase.getGoodness();

            final String fullName = esa.filePath;
            final String auth = ExhibitName.getAuthorComponent(fullName);

            // Compute "down" value with smallest increment down by author.
            final Properties pDownA = new Properties();
            pDownA.setProperty(GenProps.PPREFIX_POPWT_DETAILS + GenProps.PCOMP_POPWR_BYAUTH + auth,
                               "-1");
            final ExhibitPropsComputableMutable epcmDownA =
                    ExhibitPropsComputableMutable.generateFastApproximation(esa, new GenProps(pDownA, 1));
            final int gDownA = epcmDownA.getGoodness();
            assertTrue("negative author weighting must reduce goodness from base value",
                       gDownA < gBase);

            // Compute "up" value with smallest increment up by author.
            final Properties pUpA = new Properties();
            pUpA.setProperty(GenProps.PPREFIX_POPWT_DETAILS + GenProps.PCOMP_POPWR_BYAUTH + auth,
                               "1");
            final ExhibitPropsComputableMutable epcmUpA =
                    ExhibitPropsComputableMutable.generateFastApproximation(esa, new GenProps(pUpA, 1));
            final int gUpA = epcmUpA.getGoodness();
            assertTrue("positive author weighting must increase goodness from base value",
                       gUpA > gBase);
            }

        // TODO: test by attribute...

        // TODO: test by file type/extension...
        }

    /**Test that computable-mutable properties are preserved through serialisation.
     * Note that (de)serialising the whole AEP may choose
     * not to preserve this information as a matter of policy
     * (to save transmission/storage space at the expense of CPU time).
     */
    public void testEPCMSer()
        throws Exception
        {
        // We'll make sure that creating new instances of the AEP
        // produces a new (slightly randomised) goodness each time,
        // for all exhibits.

        // Variable "source".
//        final BasicVarMgrInterface vars = new BasicVarMgr(true);
        final ExhibitDataFileSource edfs = new ExhibitDataFileSource();
        final SimpleExhibitPipelineIF vars = edfs;

        // Put in one vote pro and one con (albeit void) entry.
        // We need to do this to keep vote/score computations happy!
        vars.setVariable(new SimpleVariableValue(SystemVariables.VOTE_CON, null));
        vars.setVariable(new SimpleVariableValue(SystemVariables.VOTE_PRO, null));

        // Repeat test for all exhibits.
        for(final Iterator<ExhibitStaticAttr> it = aep.aeid.getAllStaticAttrs().iterator(); it.hasNext(); )
            {
            final ExhibitStaticAttr esa = it.next();
            final String fullName = esa.filePath;

            // Check that when we ask for a full EPCM we don't get a trivially-stale one.
            assertFalse("Must not get trivially-stale EPCM when told to do full calc",
                        aep.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).isTriviallyStale());

            // Show that retrieving EPCM from same AEP twice gets the same value.
            // (Providing we don't force recomputation.)
            assertEquals("EPCM goodness value must be same when fetched twice",
                         aep.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).getGoodness(),
                         aep.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).getGoodness());

            // Show that we will get a different EPCM from a new identical AEP.
            // (Because of random component.)
            final AllExhibitProperties aep2 = _makeAEP(null);
            assertNotSame("EPCM goodness values expected to be different from different AEP instances",
                          aep.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).getGoodness(),
                          aep2.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).getGoodness());

            // Show that creating new AEP using old one for expensive computable state
            // preserves that state providing that this was not a quick approximation value...
            final AllExhibitProperties aepMakeWithOld = _makeAEP(aep);
            assertEquals("EPCM goodness value must be same when fetched from new instance made using old",
                         aep.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).getGoodness(),
                         aepMakeWithOld.getExhibitPropsComputableMutable(fullName, false, gp, vars, ScorerCacheIF.TRIVIAL).getGoodness());

//            // Show that serialising and deserialising AEP preserves expensive computable state.
//            // Only valid if the epcmMap field is not transient.
//            final AllExhibitProperties aepDeser =
//                    (AllExhibitProperties) SerializationTest.checkSerialisationPreservesEquality(aep);
//            assertEquals("EPCM goodness value must be same when fetched from (de)serialised copy",
//                         aep.getExhibitPropsComputableMutable(fullName, true, gp, vars).getGoodness(),
//                         aepDeser.getExhibitPropsComputableMutable(fullName, true, gp, vars).getGoodness());
            }
        }


    /**Test aspects of filtering by the SearchPageJavaBean.
     * Depends on default data set.
     */
    public void testSearchPageJavaBean()
        {
        final SearchPageJavaBean spjb1 = new SearchPageJavaBean();
        spjb1.setAep(aep);

        // Check that a new search bean returns no filter (null).
        assertTrue("new search search bean does not not return null filter", spjb1.getSearchFilter() == null);

        // Check by-author filtering.
        spjb1.setAuthor("DHD");
        assertTrue("expected author DHD, got: " + spjb1.getAuthor(), spjb1.getAuthor().equals("DHD"));
        // Note that the accept() filter takes exhibit short names (without the dir path).
        // This should work even for exhibits not in our official exhibit set.
        assertTrue(spjb1.getSearchFilter().accept("a-DHD.jpg"));
        assertTrue(!spjb1.getSearchFilter().accept("a-ANON.jpg"));
        assertTrue(spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_JPEG_NOW)));
        spjb1.setAuthor(ExhibitHandlerBeanBase.SETTER_ALL);
        assertTrue("could not clear author, got: " + spjb1.getAuthor(), spjb1.getAuthor().equals(""));

        // Check recent-additions filtering.
        assertTrue("expected no recent-days filtering, got: " + spjb1.getRecentDaysFilter(), spjb1.getRecentDaysFilter() == 0);
        spjb1.setRecentDaysFilter(ExhibitHandlerBeanBase.SETTER_ALL);
        assertTrue("expected no recent-days filtering [2], got: " + spjb1.getRecentDaysFilter(), spjb1.getRecentDaysFilter() == 0);
        // Check that each symbolic time value sets the correct number of days.
        for(final Iterator<String> it = SearchPageJavaBean.symFilterDays.keySet().iterator(); it.hasNext(); )
            {
            final String symName = it.next();
            spjb1.setRecentDaysFilter(symName);
            assertTrue("failed to get right value for symbolic name: " + symName + " got: " + spjb1.getRecentDaysFilter(),
                       spjb1.getRecentDaysFilter() == (SearchPageJavaBean.symFilterDays.get(symName)).intValue());
            }
        // Now test that filtering threshold works correctly,
        // especially trying to look for overflow of milliseconds in an int.
        spjb1.setRecentDaysFilter("d"); // Filter on a week first.
        assertTrue("failed to return new exhibit with d filter", spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_JPEG_NOW)));
        assertTrue("failed to reject 2w-old exhibit with d filter", !spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_GIF_2WOLD)));
        assertTrue("failed to reject 1998 exhibit with d filter", !spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_ANON_HTXT_1998)));
        // especially trying to look for overflow of milliseconds in an int.
        spjb1.setRecentDaysFilter("7"); // Filter on a week first.
        assertTrue("failed to return new exhibit with 7 filter", spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_JPEG_NOW)));
        assertTrue("failed to reject 2w-old exhibit with 7 filter", !spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_GIF_2WOLD)));
        assertTrue("failed to reject 1998 exhibit with 7 filter", !spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_ANON_HTXT_1998)));
        spjb1.setRecentDaysFilter("y"); // Filter on a week first.
        assertTrue("failed to return new exhibit with y filter", spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_JPEG_NOW)));
        assertTrue("failed to return 2w-old exhibit with y filter", spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_DHD_GIF_2WOLD)));
        assertTrue("failed to reject 1998 exhibit with y filter", !spjb1.getSearchFilter().accept(ExhibitName.getFileComponent(EX_SMALL_ANON_HTXT_1998)));
        // Clear recent-days filtering.
        spjb1.setRecentDaysFilter("0");
        assertTrue("expected no recent-days filtering [3], got: " + spjb1.getRecentDaysFilter(), spjb1.getRecentDaysFilter() == 0);





        // Check that a new search bean returns no filter (null).
        assertTrue("used but cleaned-up search bean does not not return null filter", spjb1.getSearchFilter() == null);

        }
    }
