/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: 28-Dec-02
 * Time: 22:24:51
 */
package org.hd.d.pg2k.test.dev;

import ORG.hd.d.jIndexer.server.JIndexBean;
import junit.framework.TestCase;
import org.hd.d.pg2k.svrCore.AllExhibitProperties;
import org.hd.d.pg2k.svrCore.FileTools;
import org.hd.d.pg2k.webSvr.exhibit.DataSourceBean;

import java.io.File;

/**Simple tests of the by-word index, including building the index.
 */
public final class IndexTest extends TestCase
    {
    public IndexTest(final String name)
        {
        super(name);
        }

    /**Check basic building of index on a representative snapshot of our exhibit data.
     */
    public static void testIndexBuilding()
        throws Exception
        {
        // Gzipped, serialised AllExhibitProperties exhibit meta-data.
        // Paths relative to the source root.
        final File f = new File("misc/testdata/20060619._ExProps.dat");

        final Object o = FileTools.deserialiseFromFile(f, true);
        assertNotNull("Deserialised data from "+f+" must not be null", o);
        assertTrue("Deserialised data from "+f+" must have correct type",
                   o instanceof AllExhibitProperties);
        final AllExhibitProperties aep = (AllExhibitProperties) o;

        // Test actually building the index.
        final long startTime = System.currentTimeMillis();
        final JIndexBean jIndexBean = DataSourceBean.computeByWordIndex(aep);
        final long endTime = System.currentTimeMillis();
        Main.getOut().println("[Index construction time: "+(endTime-startTime)+"ms.]");

        assertNotNull("constructed index must not be null", jIndexBean);

        // FIXME: Check that persistence is possible via serialisation.
        //SerializationTest.checkObjectCanBeSerialisedAndDeserialised(jIndexBean);
        }
    }
