/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: 28-Dec-02
 * Time: 22:24:51
 */
package org.hd.d.pg2k.test.dev;

import junit.framework.TestCase;
import org.hd.d.pg2k.svrCore.ExhibitAttrUtils;
import org.hd.d.pg2k.svrCore.ExhibitName;
import org.hd.d.pg2k.svrCore.ExhibitStaticAttr;
import org.hd.d.pg2k.svrCore.TextUtils;
import org.hd.d.pg2k.webSvr.exhibit.SimpleSimilarExhibitFinder;
import org.hd.d.pg2k.webSvr.util.WebUtils;

import java.util.*;

/**Tests of WAR utility routines, mainly from the WebUtils class.
 */
public final class WebUtilsTest extends TestCase
    {
    public WebUtilsTest(String name)
        {
        super(name);
        }

//    /**Do any setup needed for the tests. */
//    protected void setUp()
//        {
//        }
//
//    /**Do any clearup needed after the tests. */
//    protected void tearDown()
//        {
//        // cleanup code
//        }

    /**Tests of SimpleSimilarExhibitFinder class.
     * This class is meant as the fast way to find some simoilar exhibits
     * when the system is too busy for a full search to be practical.
     * <p>
     * The results of this simple search must be sane, if not always wonderful.
     */
    public void testSimpleSimilarExhibitFinder()
        {
        final String THIS_EXNAME = "m/m-MM.jpg";

        final ExhibitAttrUtils.ExhibitAttrWords attrWords = ExhibitAttrUtils.getAttrWords();

        // Test on an empty List; test that we get a benign empty return.
        final List<String> emptyExhibitsList = Collections.emptyList();
        final Set<String> emptyAttrWords = Collections.emptySet();
        final String r1[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
                emptyExhibitsList, String.CASE_INSENSITIVE_ORDER, THIS_EXNAME, emptyAttrWords, Integer.MAX_VALUE);
        assertTrue("result should not be null", r1 != null);
        assertTrue("on empty input list result should be empty", r1.length == 0);

        // Test on an List with only given exhibit; test that we get an empty return.
        final String r2[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            Collections.singletonList(THIS_EXNAME), String.CASE_INSENSITIVE_ORDER, THIS_EXNAME, emptyAttrWords, Integer.MAX_VALUE);
        assertTrue("result should not be null", r2 != null);
        assertTrue("on singleton input list result should be empty", r2.length == 0);

        final long now = System.currentTimeMillis();
        final Map<String,ExhibitStaticAttr> exhibits1 = new HashMap<String, ExhibitStaticAttr>();
        exhibits1.put(THIS_EXNAME, new ExhibitStaticAttr(THIS_EXNAME, 100, now));
//        final AllExhibitProperties aep1 = new AllExhibitProperties(
//                                new ExhibitPropsGlobalImmutable(),
//                                new LocationMap(),
//                                new AllExhibitImmutableData(exhibits1, now),
//                                Collections.EMPTY_MAP /* Map _loadedProps */);
        final String r3[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            Collections.singletonList(THIS_EXNAME), attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), Integer.MAX_VALUE);
        assertTrue("result should not be null", r3 != null);
        assertTrue("on singleton input list result should be empty", r3.length == 0);

        // Add some more more entries with different main words.
        // So this should still give no answers...
        exhibits1.put("a/a-AA.jpg", new ExhibitStaticAttr("a/a-AA.jpg", 100, now));
        exhibits1.put("z/z-ZZ.jpg", new ExhibitStaticAttr("z/z-ZZ.jpg", 100, now));
//        final AllExhibitProperties aep4 = new AllExhibitProperties(
//                                new ExhibitPropsGlobalImmutable(),
//                                new LocationMap(),
//                                new AllExhibitImmutableData(exhibits1, now),
//                                Collections.EMPTY_MAP /* Map _loadedProps */);
        final List<String> l4 = new Vector<String>(exhibits1.keySet());
        Collections.sort(l4, attrWords.SMART_ORDER);
        final String r4[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            l4, attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), Integer.MAX_VALUE);
        assertTrue("result should not be null", r4 != null);
        assertTrue("on input list with only different main words result should be empty", r4.length == 0);

        // Add a variant of the main exhibit name,
        // which should not add any results...
        final String VARIANT1 = "m/m-bg-MM.jpg";
        exhibits1.put(VARIANT1, new ExhibitStaticAttr(VARIANT1, 100, now));
//        final AllExhibitProperties aep5 = new AllExhibitProperties(
//                                new ExhibitPropsGlobalImmutable(),
//                                new LocationMap(),
//                                new AllExhibitImmutableData(exhibits1, now),
//                                Collections.EMPTY_MAP /* Map _loadedProps */);
        final List<String> l5 = new Vector<String>(exhibits1.keySet());
        Collections.sort(l5, attrWords.SMART_ORDER);
        final String r5[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            l5, attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), Integer.MAX_VALUE);
        assertTrue("result should not be null", r5 != null);
        assertTrue("on input list with only variants and different main words result should be empty", r5.length == 0);

        // Add a different exhibit with the same main word,
        // which should now appear in the results...
        final String EXHIBIT2 = "m/m-yyy-MM.jpg";
        exhibits1.put(EXHIBIT2, new ExhibitStaticAttr(EXHIBIT2, 100, now));
//        final AllExhibitProperties aep6 = new AllExhibitProperties(
//                                new ExhibitPropsGlobalImmutable(),
//                                new LocationMap(),
//                                new AllExhibitImmutableData(exhibits1, now),
//                                Collections.EMPTY_MAP /* Map _loadedProps */);
        final List<String> l6 = new Vector<String>(exhibits1.keySet());
        Collections.sort(l6, attrWords.SMART_ORDER);
        final String r6[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            l6, attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), Integer.MAX_VALUE);
        assertTrue("result should not be null", r6 != null);
        assertTrue("on input list with one different exhibit should yield one result", r6.length == 1);
        assertTrue("on input list with one different exhibit should yield that exhibit", ExhibitName.getFileComponent(EXHIBIT2).equals(r6[0]));

        // Add a second exhibit with the same main word,
        // so both should now appear in the results...
        final String EXHIBIT3 = "m/m-www-MM.jpg";
        exhibits1.put(EXHIBIT3, new ExhibitStaticAttr(EXHIBIT3, 100, now));
//        final AllExhibitProperties aep7 = new AllExhibitProperties(
//                                new ExhibitPropsGlobalImmutable(),
//                                new LocationMap(),
//                                new AllExhibitImmutableData(exhibits1, now),
//                                Collections.EMPTY_MAP /* Map _loadedProps */);
        final List<String> l7 = new Vector<String>(exhibits1.keySet());
        Collections.sort(l7, attrWords.SMART_ORDER);
        final String r7[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            l7, attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), Integer.MAX_VALUE);
        assertTrue("result should not be null", r7 != null);
        assertTrue("on input list with two extra exhibits should yield two results", r7.length == 2);
        assertTrue("both new exhibits should be in the results",
                        (ExhibitName.getFileComponent(EXHIBIT3).equals(r7[0])) &&
                        (ExhibitName.getFileComponent(EXHIBIT2).equals(r7[1])));

        // Now limit the output size and check that all outputs are
        // the new exhibits.
        final String r7b[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            l7, attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), 1);
        assertTrue("result should not be null", r7b != null);
        assertTrue("output should be limited to requested length", r7b.length == 1);
        assertTrue("both new exhibits should be in the results",
                        (ExhibitName.getFileComponent(EXHIBIT3).equals(r7b[0])) ||
                        (ExhibitName.getFileComponent(EXHIBIT2).equals(r7b[0])));

        // Add a third exhibit with the same main word (but different in case),
        // so both all should now appear in the results
        // (comparator must be case-insensitive other than to break ties)...
        final String EXHIBIT4 = "m/M-vvv-MM.jpg";
        exhibits1.put(EXHIBIT4, new ExhibitStaticAttr(EXHIBIT4, 100, now));
//        final AllExhibitProperties aep8 = new AllExhibitProperties(
//                                new ExhibitPropsGlobalImmutable(),
//                                new LocationMap(),
//                                new AllExhibitImmutableData(exhibits1, now),
//                                Collections.EMPTY_MAP /* Map _loadedProps */);
        final List<String> l8 = new Vector<String>(exhibits1.keySet());
        Collections.sort(l8, attrWords.SMART_ORDER);
        final String r8[] = SimpleSimilarExhibitFinder.getSimpleSimilarExhibitList(
            l8, attrWords.SMART_ORDER, THIS_EXNAME, attrWords.getAttrWordsSortedSet(), Integer.MAX_VALUE);
        assertTrue("result should not be null", r8 != null);
        assertTrue("expect comparator to be case-insensitive", r8.length == 3);
        assertTrue("all new exhibits should be in the results",
                        (ExhibitName.getFileComponent(EXHIBIT4).equals(r8[0])) &&
                        (ExhibitName.getFileComponent(EXHIBIT3).equals(r8[1])) &&
                        (ExhibitName.getFileComponent(EXHIBIT2).equals(r8[2])));
        }

    /**Test of generation of short exhibit name prefix for titles.
     * The prefixes generated must always be valid non-zero-length
     * whole-word prefixes of the file components of the exhibit names
     * selected, that make sense ti a human reader,
     * but need not be totally perfect.
     */
    public void testMinimalUniqueENTitlePrefix()
        {
        // Test that a zero-length List always returns an empty string,
        // even for an otherwise-invalid index.
        final List<String> emptyExhibitsList = Collections.emptyList();
        assertTrue("On empty list, result should be harmless empty string.",
                   "".equals(WebUtils.minimalUniqueENTitlePrefix(emptyExhibitsList, -1)));

        // Our set of fake input names.
        final List<String> sampleNames = Arrays.asList(new String[]
            {
            "zzz/apple-DHD.jpg",
            "yyy/apple-turnover-ANON.gif",
            "yyy/Apple-turnover-2-ANON.gif",
            "aaa-bbb/cat-1-ANON.gif",
            "ccc_ddd/cat-2-ANON.gif",
            "ccc-ddd/dog-2-ANON.gif",
            "ccc-ddd/dog-in-manger-2-ANON.gif",
            "ccc-ddd/dog-zeal-2-ANON.gif",
            "places-and-sights/_more1999/_more07/Indonesia-Bali-Ubud-Monkey-Forest-market-1-MB.jpg",
            "places-and-sights/_more1999/_more07/Indonesia-Bali-Ubud-Monkey-Forest-Pura-Dalem-ie-Temple-of-The-Dead-1-MB.jpg",
            "places-and-sights/_more1999/_more07/Indonesia-Bali-Ubud-Monkey-Forest-Pura-Dalem-ie-Temple-of-The-Dead-2-MB.jpg",
            "places-and-sights/_more1999/_more07/Indonesia-Bali-Ubud-Monkey-Forest-Pura-Dalem-ie-Temple-of-The-Dead-3-MB.jpg",
            "aaa/zebra-in-zoo-ANON.png"
            });

        // Expected abbreviation for each name.
        final String expectedResponses[] =
            {
            "apple-",
            "apple-turnover-",
            "Apple-turnover-",
            "cat-",
            "cat-",
            "dog-",
            "dog-in-",
            "dog-zeal-",
            "Indonesia-Bali-Ubud-Monkey-Forest-market-",
            "Indonesia-Bali-Ubud-Monkey-Forest-Pura-",
            "Indonesia-Bali-Ubud-Monkey-Forest-Pura-",
            "Indonesia-Bali-Ubud-Monkey-Forest-Pura-",
            "zebra-"
            };

        // Make sure that our test data is self-consistent.
        assertTrue("test set wrong size", expectedResponses.length == sampleNames.size());

        // Test for friendly behaviour on bad inputs.
        assertTrue("On negative (illegal) index, result should be harmless empty string.",
                   "".equals(WebUtils.minimalUniqueENTitlePrefix(sampleNames, -11111)));
        assertTrue("On over-large (illegal) index, result should be harmless empty string.",
                   "".equals(WebUtils.minimalUniqueENTitlePrefix(sampleNames, 3 + 2*sampleNames.size())));


        // Test that we get the expected output for each index.
        for(int i = sampleNames.size(); --i >= 0; )
            {
            final String actualResponse = WebUtils.minimalUniqueENTitlePrefix(sampleNames, i);
            assertTrue("unexpected response `"+actualResponse+"' at index "+i+
                            " instead of `"+(expectedResponses[i])+"'",
                       expectedResponses[i].equals(actualResponse));
            }
        }

    /**Simple tests of WebUtils.sanitiseForXML().
     * This tests for robust behaviour of this routine with and without
     * special handling of i18n marked-up text.
     */
    public static final void testSimple_sanitiseForXML()
        {
        // Maximum length for sanitisation.
        final int maxLen = 16;
        // This data has three columns:
        //  1) The input String.
        //  2) The output String if entity codes are not allowed.
        //  3) The output String if entity codes are allowed.
        final String testData[][] =
            {
                // Empty string; no conversion needed.
                { "", "", "" },

                // Convert null input to empty-string result.
                { null, "", "" },

                // All whitespace; result is empty string.
                { " \r \n \r\n \t\t", "", "" },

                // Simple short pure alphanumeric: no transform needed.
                { "abc1234Z", "abc1234Z", "abc1234Z" },

                // Simple short text with whitespace: whitespace is trimmed.
                { " hot  dog!  ", "hot  dog!", "hot  dog!" },

                // Long string is trimmed then truncated.
                { " a  long long string  ", "a  long long ...", "a  long long ..." },

                // Test handling of a single entity.
                { "Cow &amp; Gate", "Cow +amp; Gate", "Cow &amp; Gate" },

                // Test handling of a single entity that seems to make the string too long.
                { "&longbutvalidentitycode;", "+longbutvalid...", "&longbutvalidentitycode;" },

                // Test handling of non-ASCII characters...
                { "Mer\u0127ba ghal", "Mer ba ghal", "Mer&#295;ba ghal" },
            };

        for(int i = testData.length; --i >= 0; )
            {
            final String input = testData[i][0];
            final String expectedNEOutput = testData[i][1];
            final String expectedEOutput = testData[i][2];

            final String nEOutput = TextUtils.sanitiseForXML(input, maxLen, false);
            assertTrue("no-entity-code output should be strictly length bounded",
                       nEOutput.length() <= maxLen);
            assertTrue("unexpected no-entity-code result at index "+i+
                       " got `"+nEOutput+"', expected `"+expectedNEOutput+"'",
               expectedNEOutput.equals(nEOutput));

            final String eOutput = TextUtils.sanitiseForXML(input, maxLen, true);
            assertTrue("unexpected entity-code result at index "+i+
                       " got `"+eOutput+"', expected `"+expectedEOutput+"'",
               expectedEOutput.equals(eOutput));
            }
        }

    /**Test bot/spider matching from UA name. */
    public static void testUABotNameMatching()
        {
        if(WebUtils.UA_REGEX == null)
            {
            Main.getOut().println("UA_REGEX is null; skipping test...");
            return;
            }

        assertFalse("Must not class vanilla browser (eg FF) as a bot", WebUtils.UA_REGEX.matcher("Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.7) Gecko/20060909 Firefox/1.5.0.7").matches());
        assertTrue("Must catch common bots (eg IA)",  WebUtils.UA_REGEX.matcher("ia_archiver").matches());
        assertTrue("Should match newly-created bot name",  WebUtils.UA_REGEX.matcher("MyNewBadBot").matches());
        }
    }
