package SevenZip.Compression.LZ;

public class InWindow {
    public byte[] _bufferBase = null; // pointer to buffer with data
    java.io.InputStream _stream;
    int _posLimit;  // offset (from _buffer) of first b_yte when new block reading must be done
    boolean _streamEndWasReached; // if (true) then _streamPos shows real end of stream
    
    int _pointerToLastSafePosition;
    
    public int _bufferOffset;
    
    public int _blockSize;  // Size of Allocated memory block
    public int _pos;             // offset (from _buffer) of curent b_yte
    int _keepSizeBefore;  // how many BYTEs must be kept in buffer before _pos
    int _keepSizeAfter;   // how many BYTEs must be kept buffer after _pos
    int _keepSizeReserv;  // how many BYTEs must be kept as reserv
    public int _streamPos;   // offset (from _buffer) of first not read b_yte from Stream
    
    public void MoveBlock() {
        int offset = _bufferOffset + _pos - _keepSizeBefore;
        int numBytes = _bufferOffset + _streamPos - offset;
        
        // check negative offset ????
        for (int i = 0; i < numBytes; i++) _bufferBase[i] = _bufferBase[offset + i];
        _bufferOffset -= offset;
    }
    
    public void ReadBlock() throws java.io.IOException {
        if (_streamEndWasReached)
            return;
        while (true) {
            int size = _blockSize - _streamPos - _bufferOffset;
            if (size == 0)
                return;
            int numReadBytes = _stream.read(_bufferBase, _bufferOffset + _streamPos, size);
            if (numReadBytes == -1) {
                _posLimit = _streamPos;
                int pointerToPostion = _bufferOffset + _posLimit;
                if (pointerToPostion > _pointerToLastSafePosition)
                    _posLimit = (_pointerToLastSafePosition - _bufferOffset);
                _streamEndWasReached = true;
                return;
            }
            _streamPos += numReadBytes;
            if (_streamPos >= _pos + _keepSizeAfter)
                _posLimit = _streamPos - _keepSizeAfter;
        }
    }
    
    
    public void Create(int keepSizeBefore, int keepSizeAfter, int keepSizeReserv) {
        _keepSizeBefore = keepSizeBefore;
        _keepSizeAfter = keepSizeAfter;
        _keepSizeReserv = keepSizeReserv;
        int blockSize = keepSizeBefore + keepSizeAfter + keepSizeReserv;
        if (_bufferBase == null || _blockSize != blockSize) {
            _blockSize = blockSize;
            _bufferBase = new byte[_blockSize];
        }
        _pointerToLastSafePosition = _blockSize - keepSizeAfter;
    }
    
    public void Init(java.io.InputStream stream) throws java.io.IOException {
        _stream = stream;
        _bufferOffset = 0;
        _pos = 0;
        _streamPos = 0;
        _streamEndWasReached = false;
        ReadBlock();
    }
    
    public void ReleaseStream() { _stream = null; }
    
    public void MovePos() throws java.io.IOException {
        _pos++;
        if (_pos > _posLimit) {
            int pointerToPostion = _bufferOffset + _pos;
            if (pointerToPostion > _pointerToLastSafePosition)
                MoveBlock();
            ReadBlock();
        }
    }
    
    public int GetIndexByte(int index) { return _bufferBase[_bufferOffset + _pos + index]&0xFF; }
    
    // index + limit have not to exceed _keepSizeAfter;
    public int GetMatchLen(int index, int distance, int limit) {
        if (_streamEndWasReached)
            if ((_pos + index) + limit > _streamPos)
                limit = _streamPos - (_pos + index);
        distance++;
        int pby = _bufferOffset + _pos + index;
        
        int i;
        for (i = 0; i < limit && _bufferBase[pby + i] == _bufferBase[pby + i - distance]; i++) ;
        return i;
    }
    
    public int GetNumAvailableBytes() { return _streamPos - _pos; }
    
    public void ReduceOffsets(int subValue) {
        _bufferOffset += subValue;
        _posLimit -= subValue;
        _pos -= subValue;
        _streamPos -= subValue;
    }
}

