/*
 * Created by IntelliJ IDEA.
 * User: d@hd.org
 * Date: 22-Jul-02
 * Time: 18:13:44

Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */
package org.hd.d.pg2k.webSvr.ads;

import ORG.hd.d.IsDebug;
import org.hd.d.pg2k.webSvr.util.WebConsts;

import java.util.Random;

/**This implements the AdBrite bean.
 */
public final class AdBriteBean implements AdBeanInterface
    {
    /**Parameter value to select vertical (tower) format (default). */
    public static final String AD_FORMAT_AB_VERTICAL = "ABVertical";

    /**Parameter value to select horizontal (banner) format. */
    public static final String AD_FORMAT_AB_HORIZONTAL = "ABHorizontal";

    /**Requested ad format or null; volatile to allow lock-less thread-safe access. */
    private volatile String selectedFormat;

    /**Can be used to set general parameters for the bean, or can be null.
     */
    public AdBeanInterface setParams(final String rawParams)
        {
        selectedFormat = rawParams;
        return(this);
        }

    /**Can make an HTML body-text insert in the given context.
     * Always true for AdBrite.
     */
    public boolean canMakeHTMLBodyInsert(final Context ctxt)
        {
        return(true);
        }

    /**Make HTML body text in the given context, or null if none can be made.
     * @param ctxt  not used
     * @param source  not used
     */
    public String getHTMLBodyInsert(final Context ctxt,
                                    final Random source)
        {
        if(!canMakeHTMLBodyInsert(ctxt)) { return(""); } // Definitely cannot.

        final int capacity = 599;
        final StringBuilder sb = new StringBuilder(capacity);
        if(AD_FORMAT_AB_HORIZONTAL.equals(selectedFormat))
            {
            // Horizontal banner requested...
            //  Variable number of ads shown.
            sb.append("<!-- Begin: AdBrite -->").append(WebConsts.WEB_LINE_END);
            sb.append("<style type=\"text/css\">").append(WebConsts.WEB_LINE_END);
            sb.append(".adHeadline {font: bold 10pt Arial; text-decoration: underline; color: blue; background-color: white;}").append(WebConsts.WEB_LINE_END);
            sb.append(".adText {font: normal 10pt Arial; text-decoration: none; color: black; background-color: white;}").append(WebConsts.WEB_LINE_END);
            sb.append("</style>").append(WebConsts.WEB_LINE_END);
            sb.append("<script type=\"text/javascript\" src=\"http://3.adbrite.com/mb/text_group.php?sid=44471&amp;col=").append(source.nextBoolean() ? 2 : 4).append("&amp;br=1\"></script>").append(WebConsts.WEB_LINE_END);
            sb.append("<p />").append(WebConsts.WEB_LINE_END);
            sb.append("<div><a class=\"adHeadline\" target=\"_top\" href=\"http://www.adbrite.com/mb/commerce/purchase_form.php?opid=44471&amp;afsid=1\">Your Ad Here</a></div>").append(WebConsts.WEB_LINE_END);
            sb.append("<!-- End: AdBrite -->").append(WebConsts.WEB_LINE_END);
            }
        else
            {
            // Vertical tower (default)...
            sb.append("<!-- Begin: AdBrite -->").append(WebConsts.WEB_LINE_END);
            sb.append("<style type=\"text/css\">").append(WebConsts.WEB_LINE_END);
            sb.append(".adHeadline {font: bold 10pt Arial; text-decoration: underline; color: blue; background-color: white;}").append(WebConsts.WEB_LINE_END);
            sb.append(".adText {font: normal 10pt Arial; text-decoration: none; color: black; background-color: white;}").append(WebConsts.WEB_LINE_END);
            sb.append("</style>").append(WebConsts.WEB_LINE_END);
            sb.append("<script type=\"text/javascript\" src=\"http://3.adbrite.com/mb/text_group.php?sid=44471&amp;br=1\"></script>").append(WebConsts.WEB_LINE_END);
            sb.append("<p />").append(WebConsts.WEB_LINE_END);
            sb.append("<div><a class=\"adHeadline\" target=\"_top\" href=\"http://www.adbrite.com/mb/commerce/purchase_form.php?opid=44471&amp;afsid=1\">Your Ad Here</a></div>").append(WebConsts.WEB_LINE_END);
            sb.append("<!-- End: AdBrite -->").append(WebConsts.WEB_LINE_END);
            }

        // We can see if the initial capacity guess needs adjusting...
        if(IsDebug.isDebug && (sb.length() > capacity))
            { System.err.println("WARNING: AB BEAN OUTPUT LEN = " + sb.length() + " vs initial capacity guess " + capacity); }

        return(sb.toString());
        }
    }
