/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.hd.d.pg2k.webSvr.ads;



/**Advertising-related constants.
 */
public final class AdConsts
    {
    /**Prevent construction of an instance. */
    private AdConsts() { }

    /**If true, support Web advertising in the Gallery.
     * This includes Google AdSense and TribalFusion for example.
     * <p>
     * This may include things such as in-line banners,
     * adjusted page layout,
     * click-through monitoring, etc.
     * <p>
     * With this false, such special support is removed,
     * and ads are limited to whatever is included in general
     * banner rotations, etc.
     */
    public static final boolean AD_SUPPORT = true;

    /**Drop more ads on poorly-performing pages.
     * This means that we might show alternative interesting content
     * rather than even trying to show an ad.
     * <p>
     * This also needs a run-time property to be set to be effective.
     * <p>
     * We may do this to help reduce ad blindness
     * and to increase CTR for advertisers
     * (and thus eCPM for us long-term possibly).
     */
    public static final boolean DROP_MORE_ADS_FOR_ALT_CONTENT = true;

    /**If true then show no or fewer ads on first page that a user sees, whichever it is.
     * This may improve page-load speed
     * and the user's general perception of the site,
     * but may reduce ad revenue such as from single-page visitors.
     */
    public static final boolean DROP_ADS_ON_FIRST_PAGE_VIEW = false;

    /**If true, enable eShirt link/ads. */
    public static final boolean ENABLE_ESHIRT = true;

    /**If true then enable conversion tracking of ads for the Gallery. */
    public static final boolean ENABLE_CONV_TRACKING = true;

    /**If true, enable AdBrite ad slots. */
    public static final boolean ENABLE_ADBRITE = true;

    /**If true, enable TribalFusion ad slots. */
    public static final boolean ENABLE_TRIBALFUSION = true;

    /**If true, enable Casale Media ad slots. */
    public static final boolean ENABLE_CASALEMEDIA = true;

    /**Maximum number of Google AdSense AdUnits (not including AdLinks) allowed on one page. */
    public static final int ADSENSE_MAX_ADUNITS = 3;

    /**Prefix in global event set for layoutStyle parameter to capture name of click-through in given page layout. */
    public static final String LAYOUT_STYLE_EVENT_PREFIX = "ctLayoutStyle=";

    /**Prefix in global event set of ad/page layout page-view name. */
    public static final String PAGE_LAYOUT_KHITS_EVENT_PREFIX = "kPLV=";

    /**Request property name that holds String name of (last) internal tower layout, if any, for current page. */
    public static final String RQPNAME_TOWERINTLAYOUT = "TowerIntLayout";

    /**Prefix in global event set for click-through when in given internal tower layout. */
    public static final String TOWERINTLAYOUT_EVENT_PREFIX = "ct" + RQPNAME_TOWERINTLAYOUT + '=';

    /**Prefix in global event set of tower-ad internal layout page-view name. */
    public static final String TOWERINTLAYOUT_KHITS_EVENT_PREFIX = "kPTILV=";

    /**Request property name that holds String name of (last) internal mid-page layout, if any, for current page. */
    public static final String RQPNAME_MPINTLAYOUT = "MPIntLayout";

    /**Prefix in global event set for click-through when in given internal mid-page layout. */
    public static final String MPINTLAYOUT_EVENT_PREFIX = "ct" + RQPNAME_MPINTLAYOUT + '=';

    /**Prefix in global event set of mid-page-ad internal layout page-view name. */
    public static final String MPINTLAYOUT_KHITS_EVENT_PREFIX = "kPMPILV=";

    /**Name of request-level attribute to indicate that we've voluntarily dropped a G AdLink unit if set non-null.
     * This means that the unit is available to another position on the page.
     * Whatever code mops up and displays the available link-unit should unset the attribute
     */
    public static final String ASLU_AVAILABLE_REQ_ATTR_NAME = "org.hd.d.pg2k.webSvr.ads.ALUNITAVAIL";
    }
