/*
 * Created by IntelliJ IDEA.
 * User: d@hd.org
 * Date: 22-Jul-02
 * Time: 18:13:44

Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */
package org.hd.d.pg2k.webSvr.ads;

import ORG.hd.d.IsDebug;
import org.hd.d.pg2k.webSvr.util.WebConsts;

import java.util.Random;

/**This implements the Tribal Fusion bean.
 * Lightweight.
 * <p>
 * Thread-safe.
 */
public final class TribalFusionBean implements AdBeanInterface
    {
    /**Parameter value to select wide (up to 160x600) variable-width tower format (default). */
    public static final String AD_FORMAT_TF_TOWER_WIDEVAR = "TFWideVAR";

    /**Parameter value to select narrow (120x600) fixed-width tower format. */
    public static final String AD_FORMAT_TF_TOWER_NARROW = "TFNarrowVAR";

    /**Parameter value to select wide (up to 160x600) iframe tower format; fastest page loading. */
    public static final String AD_FORMAT_TF_TOWER_WIDE_IF = "TFWideIF";

    /**Parameter value to select narrow (120x600) iframe tower format; fastest page loading. */
    public static final String AD_FORMAT_TF_TOWER_NARROW_IF = "TFNarrowIF";

    /**Requested ad format or null; volatile to allow lock-less thread-safe access. */
    private volatile String selectedFormat;

    /**Can be used to set general parameters for the bean, or can be null.
     */
    public AdBeanInterface setParams(final String rawParams)
        {
        selectedFormat = rawParams;
        return(this);
        }

    /**Can make an HTML body-text insert in the given context.
     * Always true for Tribal Fusion towers.
     */
    public boolean canMakeHTMLBodyInsert(final Context ctxt)
        {
        return(true);
        }


    /**If true, sometimes use NoAd (collapsing) code in tower slots. */
    private static final boolean ALLOW_NOAD_CODE = false;

    /**Make HTML body text in the given context, or null if none can be made.
     * @param ctxt  not used
     * @param source  must not be null
     */
    public String getHTMLBodyInsert(final Context ctxt,
                                    final Random source)
        {
        if(!canMakeHTMLBodyInsert(ctxt)) { return(""); } // Definitely cannot.

        // Cache-buster value...
        final int rndID = (source.nextInt() >>> 1);

        final int capacity = 1300;
        final StringBuilder sb = new StringBuilder(capacity);
        if(AD_FORMAT_TF_TOWER_NARROW_IF.equals(selectedFormat))
            {
            sb.append("<!-- TF 120x600 IFrame code -->").append(WebConsts.WEB_LINE_END);
            sb.append(" <iframe src=\"http://a.tribalfusion.com/f.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=1009387044\" width=120 height=600 marginwidth=0 marginheight=0 hspace=0 vspace=0 frameborder=0 scrolling=no allowTransparency=true>").append(WebConsts.WEB_LINE_END);
            sb.append("         <script language=javascript><!--").append(WebConsts.WEB_LINE_END);
            sb.append("           randNum = ((new Date()).getTime() % 2147483648) + Math.random();").append(WebConsts.WEB_LINE_END);
            sb.append("           document.write(").append(WebConsts.WEB_LINE_END);
            sb.append("             \"<a href='http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=\" + randNum + \"' target=_blank>\" +").append(WebConsts.WEB_LINE_END);
            sb.append("             \"<img src='http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=\" + randNum + \"'\" +").append(WebConsts.WEB_LINE_END);
            sb.append("               \" width=120 height=600 border=0 alt='Click Here'></a>\");").append(WebConsts.WEB_LINE_END);
            sb.append("         // --></script>").append(WebConsts.WEB_LINE_END);
            sb.append("         <noscript>").append(WebConsts.WEB_LINE_END);
            sb.append("             <a href=\"http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=1009387044\" target=_blank>").append(WebConsts.WEB_LINE_END);
            sb.append("             <img src=\"http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=1009387044\"").append(WebConsts.WEB_LINE_END);
            sb.append("                  width=120 height=600 border=0 alt=\"Click Here\"></a>").append(WebConsts.WEB_LINE_END);
            sb.append("         </noscript>").append(WebConsts.WEB_LINE_END);
            sb.append(" </iframe>").append(WebConsts.WEB_LINE_END);
            sb.append("<!-- TF 120x600 IFrame code -->").append(WebConsts.WEB_LINE_END);
            }
        else if(AD_FORMAT_TF_TOWER_WIDE_IF.equals(selectedFormat))
            {
            sb.append("<!-- TF 160x600 IFrame VAR code -->").append(WebConsts.WEB_LINE_END);
            sb.append(" <iframe src=\"http://a.tribalfusion.com/f.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&type=var&requestID=265901636\" width=160 height=600 marginwidth=0 marginheight=0 hspace=0 vspace=0 frameborder=0 scrolling=no allowTransparency=true>").append(WebConsts.WEB_LINE_END);
            sb.append("         <script language=javascript><!--").append(WebConsts.WEB_LINE_END);
            sb.append("           randNum = ((new Date()).getTime() % 2147483648) + Math.random();").append(WebConsts.WEB_LINE_END);
            sb.append("           document.write(").append(WebConsts.WEB_LINE_END);
            sb.append("             \"<a href='http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=\" + randNum + \"' target=_blank>\" +").append(WebConsts.WEB_LINE_END);
            sb.append("             \"<img src='http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=\" + randNum + \"'\" +").append(WebConsts.WEB_LINE_END);
            sb.append("               \" width=160 height=600 border=0 alt='Click Here'></a>\");").append(WebConsts.WEB_LINE_END);
            sb.append("         // --></script>").append(WebConsts.WEB_LINE_END);
            sb.append("         <noscript>").append(WebConsts.WEB_LINE_END);
            sb.append("             <a href=\"http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=265901636\" target=_blank>").append(WebConsts.WEB_LINE_END);
            sb.append("             <img src=\"http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=265901636\"").append(WebConsts.WEB_LINE_END);
            sb.append("                  width=160 height=600 border=0 alt=\"Click Here\"></a>").append(WebConsts.WEB_LINE_END);
            sb.append("         </noscript>").append(WebConsts.WEB_LINE_END);
            sb.append(" </iframe>").append(WebConsts.WEB_LINE_END);
            sb.append("<!-- TF 160x600 IFrame VAR code -->").append(WebConsts.WEB_LINE_END);
            }
        else if(AD_FORMAT_TF_TOWER_NARROW.equals(selectedFormat))
            {
            // Narrow tower requested...
            if(ALLOW_NOAD_CODE && source.nextBoolean())
                {
                sb.append("<!-- TF 120x600 JScript NoAD code -->").append(WebConsts.WEB_LINE_END);
                sb.append("<script language=javascript><!--").append(WebConsts.WEB_LINE_END);
                sb.append("document.write('<scr'+'ipt language=javascript src=\"http://a.tribalfusion.com/j.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&noAd=1&requestID='+((new Date()).getTime() % 2147483648) + Math.random()+'\"></scr'+'ipt>');").append(WebConsts.WEB_LINE_END);
                sb.append("//-->").append(WebConsts.WEB_LINE_END);
                sb.append("</script>").append(WebConsts.WEB_LINE_END);
                sb.append("<noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("   <a href=\"http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=").append(rndID).append("\" target=_blank>").append(WebConsts.WEB_LINE_END);
                sb.append("   <img src=\"http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=").append(rndID).append('"').append(WebConsts.WEB_LINE_END);
                sb.append("                  width=120 height=600 border=0 alt=\"Click Here\"></a>").append(WebConsts.WEB_LINE_END);
                sb.append("</noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("<!-- TF 120x600 JScript NoAD code -->").append(WebConsts.WEB_LINE_END);
                }
            else
                {
                sb.append("<!-- TF 120x600 JScript code -->").append(WebConsts.WEB_LINE_END);
                sb.append("<script language=javascript><!--").append(WebConsts.WEB_LINE_END);
                sb.append("document.write('<scr'+'ipt language=javascript src=\"http://a.tribalfusion.com/j.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID='+((new Date()).getTime() % 2147483648) + Math.random()+'\"></scr'+'ipt>');").append(WebConsts.WEB_LINE_END);
                sb.append("//-->").append(WebConsts.WEB_LINE_END);
                sb.append("</script>").append(WebConsts.WEB_LINE_END);
                sb.append("<noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("   <a href=\"http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=").append(rndID).append("\" target=_blank>").append(WebConsts.WEB_LINE_END);
                sb.append("   <img src=\"http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=120x600&requestID=").append(rndID).append('"').append(WebConsts.WEB_LINE_END);
                sb.append("                  width=120 height=600 border=0 alt=\"Click Here\"></a>").append(WebConsts.WEB_LINE_END);
                sb.append("</noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("<!-- TF 120x600 JScript code -->").append(WebConsts.WEB_LINE_END);
                }
            }
        else
            {
            // Wide/variable tower (default)...
            if(ALLOW_NOAD_CODE && source.nextBoolean())
                {
                // Show nothing if no ad available...
                sb.append("<!-- TF 160x600 JScript NoAD code -->").append(WebConsts.WEB_LINE_END);
                sb.append("<script language=javascript><!--").append(WebConsts.WEB_LINE_END);
                sb.append("document.write('<scr'+'ipt language=javascript src=\"http://a.tribalfusion.com/j.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&noAd=1&requestID='+((new Date()).getTime() % 2147483648) + Math.random()+'\"></scr'+'ipt>');").append(WebConsts.WEB_LINE_END);
                sb.append("//-->").append(WebConsts.WEB_LINE_END);
                sb.append("</script>").append(WebConsts.WEB_LINE_END);
                sb.append("<noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("   <a href=\"http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=").append(rndID).append("\" target=_blank>").append(WebConsts.WEB_LINE_END);
                sb.append("   <img src=\"http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=").append(rndID).append('"').append(WebConsts.WEB_LINE_END);
                sb.append("                  width=160 height=600 border=0 alt=\"Click Here\"></a>").append(WebConsts.WEB_LINE_END);
                sb.append("</noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("<!-- TF 160x600 JScript NoAD code -->").append(WebConsts.WEB_LINE_END);
                }
            else
                {
                // Always show an ad of some sort, ie PSA or POST if need be.
                sb.append("<!-- TF 160x600 JScript VAR code -->").append(WebConsts.WEB_LINE_END);
                sb.append("<script language=javascript><!--").append(WebConsts.WEB_LINE_END);
                sb.append("document.write('<scr'+'ipt language=javascript src=\"http://a.tribalfusion.com/j.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&type=var&requestID='+((new Date()).getTime() % 2147483648) + Math.random()+'\"></scr'+'ipt>');").append(WebConsts.WEB_LINE_END);
                sb.append("//-->").append(WebConsts.WEB_LINE_END);
                sb.append("</script>").append(WebConsts.WEB_LINE_END);
                sb.append("<noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("   <a href=\"http://a.tribalfusion.com/i.click?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=").append(rndID).append("\" target=_blank>").append(WebConsts.WEB_LINE_END);
                sb.append("   <img src=\"http://a.tribalfusion.com/i.ad?site=DHDMultimediaGallery&adSpace=ROS&size=160x600&requestID=").append(rndID).append('"').append(WebConsts.WEB_LINE_END);
                sb.append("                  width=160 height=600 border=0 alt=\"Click Here\"></a>").append(WebConsts.WEB_LINE_END);
                sb.append("</noscript>").append(WebConsts.WEB_LINE_END);
                sb.append("<!-- TF 160x600 JScript VAR code -->").append(WebConsts.WEB_LINE_END);
                }
            }

        // We can see if the initial capacity guess needs adjusting...
        if(IsDebug.isDebug && (sb.length() > capacity))
            { System.err.println("WARNING: TF BEAN OUTPUT LEN = " + sb.length() + " vs initial capacity guess " + capacity); }

        return(sb.toString());
        }
    }
