/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.hd.d.pg2k.webSvr.catalogue;

import org.hd.d.pg2k.webSvr.exhibit.BuiltInFilters;
import org.hd.d.pg2k.webSvr.exhibit.Expr;
import org.hd.d.pg2k.webSvr.exhibit.FilterExpr;

/**
 * Created by IntelliJ IDEA.
 * User: Damon Hart-Davis
 * Date: 31-May-2003
 * Time: 12:28:58
 */

/**Servlet to do filtering and selection of one tree of exhibits for display.
 * This class gathers all exhibits for display in a tree.
 * <p>
 * This will forward all hits one way or another to a JSP (or another servlet) for display.
 */
public final class TreeFilterServletAllExhibits
    extends TreeFilterServletBase
    {
    /**Get the filter expression to apply.
     * Defaults to accept everything.
     * <p>
     * Override in derived classes.
     */
    protected Expr getFilter()
        {
        return(new FilterExpr(null, new BuiltInFilters.filtAll(null)));
        }

    /**Get (short) title text common catalogue name; null if none.
     * The expanded text should be 7-bit printable ASCII,
     * possibly with embedded entities and mark-up.
     * <p>
     * Override in derived classes.
     */
    protected String getTitleCommonCatalogueName()
        { return("common.virtCol.allExhibits.title"); }

    /**Get descriptive text (i18n-ed if possible); null if none.
     * The expanded text should be 7-bit printable ASCII,
     * possibly with embedded entities and mark-up.
     * <p>
     * Defaults to null (no descriptive text).
     * <p>
     * Override as needed in derived classes.
     */
    protected String getDescriptionCommonCatalogueName()
        { return("common.virtCol.allExhibits.description"); }

    /**Unique Serialisation class ID generated by http://random&#46;hd&#46;org/. */
    private static final long serialVersionUID = 5283378272843090137L;
    }
