/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.hd.d.pg2k.webSvr.catalogue;

import org.hd.d.pg2k.svrCore.AllExhibitProperties;
import org.hd.d.pg2k.svrCore.ExhibitName;
import org.hd.d.pg2k.svrCore.ExhibitPropsComputable;
import org.hd.d.pg2k.svrCore.ImageUtils;
import org.hd.d.pg2k.svrCore.MIME.ExhibitMIME;
import org.hd.d.pg2k.webSvr.exhibit.Expr;
import org.hd.d.pg2k.webSvr.exhibit.FilterExpr;
import org.hd.d.pg2k.webSvr.exhibit.FilterIF;
import org.hd.d.pg2k.webSvr.util.WebUtils;

import java.awt.*;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: Damon Hart-Davis
 * Date: 04-Jul-2003
 * Time: 20:19:11
 */

/**Generates the "all icons" tree.
 * This consists of all HTML-inlineable images of side no more than 64 pixels.
 */
public final class TreeFilterServletAllIcons extends TreeFilterServletBase
    {
    /**Get (short) title text common catalogue name; null if none.
     * The expanded text should be 7-bit printable ASCII,
     * possibly with embedded entities and mark-up.
     */
    protected String getTitleCommonCatalogueName()
        { return("common.virtCol.allIcons.title"); }

    /**Get descriptive text (i18n-ed if possible); null if none.
     * The expanded text should be 7-bit printable ASCII,
     * possibly with embedded entities and mark-up.
     * <p>
     * Defaults to null (no descriptive text).
     * <p>
     * Override as needed in derived classes.
     */
    protected String getDescriptionCommonCatalogueName()
        { return("common.virtCol.allIcons.description"); }

    /**Get the filter expression to apply.
     * Defaults to accept as icons as per the class description.
     * <p>
     * Should only be called once,
     * so need not implement equals() and hashCode() in the class.
     */
    protected Expr getFilter()
        {
        return(new FilterExpr(null, (new FilterIF(){
            public boolean accept(final AllExhibitProperties aep, final String exhibitName)
                {
                // If the exhibit is not appropriate to inline in an HTML page
                // then stop immediately.
                final String extension =
                    ExhibitName.getExtensionComponent(exhibitName);
                ExhibitMIME.ExhibitTypeParameters exhibitType =
                    ExhibitMIME.getExhibitType(extension);
                if(!ImageUtils.canInlineInHTMLPageSimple(exhibitType))
                    { return(false); }

                // Now get the image dimensions...
                final ExhibitPropsComputable epc =
                    aep.getExhibitPropsComputable(exhibitName);
                if(epc == null) { return(false); }
                final Dimension xyDim = epc.getXyDimensions();

                // Not valid if no dimensions or item not square...
                if((xyDim == null) ||
                   (xyDim.width > 64) ||
                   (xyDim.height > 64))
                    { return(false); }

//                // Right, now check if edge size is good for us...
//                final int l = xyDim.width;
//                return((l == 16) || (l == 32) || (l == 64));

                return(true); // Seems OK.
                }

            /**Unique Serialisation class ID generated by http://random&#46;hd&#46;org/. */
            private static final long serialVersionUID = 4157995688833159375L;
            })));
        }

    /**Unique Serialisation class ID generated by http://random&#46;hd&#46;org/. */
    private static final long serialVersionUID = -3815214610456191652L;
    }
