/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.webSvr.exhibit;

import org.hd.d.pg2k.svrCore.AllExhibitProperties;

/**Interface for a class to filter exhibits (for a FilterBean).
 * Filters must be Serializable so that filter expressions can
 * be persisted.  They must not attempt to cache any reference,
 * direct or indirect, to the DataSourceBean.
 * <p>
 * Filter objects must be immutable and the accept() method must be
 * idempotent and not depend on any state held within the object.
 * <p>
 * The output ordering is not necessarily stable with respect to the input.
 * <p>
 * Implementing classes should implement equals() and hashCode() properly.
 */
public interface FilterIF extends ExprIF, AllExhibitProperties.AEPFilter
    {
    /**Returns true if the exhibit (the full name is passed) should be accepted.
     */
    public boolean accept(AllExhibitProperties aep, String exhibitName);
    }
