/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.webSvr.exhibit;

import org.hd.d.pg2k.svrCore.AllExhibitProperties;

/**Interface for a class to sort exhibits (for a FilterBean).
 * Sorters must be Serializable so that expressions can
 * be persisted.  They must not attempt to cache any reference,
 * direct or indirect, to a DataSourceBean.
 * <p>
 * Sorter objects must be immutable and the sort() method must be
 * idempotent.
 * <p>
 * This interface may be used by any operation that needs to
 * operate on the whole (remaining) set of exhibits at once.
 * <p>
 * A sort routine may return its input array re-arranged, or may
 * return a new array.
 * <p>
 * In no case must it leave nulls or duplicates in the result,
 * nor may the result be null.
 * <p>
 * Implementing classes should implement equals() and hashCode() properly.
 */
public interface SortIF extends ExprIF
    {
    /**Sorts the given exhibits (full names are passed) in situ.
     */
    public String[] sort(AllExhibitProperties aep, String exhibitNames[]);
    }
