/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.hd.d.pg2k.webSvr.threeD;

import ORG.hd.d.IsDebug;
import com.sun.j3d.utils.image.TextureLoader;
import org.hd.d.pg2k.svrCore.*;
import org.hd.d.pg2k.svrCore.MIME.ExhibitMIME;
import org.hd.d.pg2k.svrCore.datasource.ExhibitDataFileSource;

import javax.jnlp.*;
import javax.media.j3d.Texture;
import java.awt.image.BufferedImage;
import java.io.*;
import java.lang.ref.SoftReference;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;


/**3D Walkthrough "business-logic" holder.
 * Is designed to be GUI-free and just contain the logic.
 * <p>
 * Package visible since need be seen only by the main GUI class.
 */
final class ThreeDLogic implements LightweightMetaDataFetchInterface
    {
    /**Package-visible constructor since need be seen only by the main GUI class.
     * @param logger  reference to central logger; never null
     */
    ThreeDLogic(final SimpleLoggerIF logger)
        {
        assert(logger != null);
        this.logger = logger;

        // Get access to basic services.
        BasicService basicService = null;
        try { basicService = (BasicService) ServiceManager.lookup("javax.jnlp.BasicService"); }
        catch(final UnavailableServiceException e) { }
        bs = basicService;

        // Get access to persistence management.
        PersistenceService persistenceService = null;
        try { persistenceService = (PersistenceService) ServiceManager.lookup("javax.jnlp.PersistenceService"); }
        catch(final UnavailableServiceException e) { }
        ps = persistenceService;

//        // Get access to file open services.
//        FileOpenService fileOpenService = null;
//        try { fileOpenService = (FileOpenService) ServiceManager.lookup("javax.jnlp.FileOpenService"); }
//        catch(final UnavailableServiceException e) { }
//        fos = fileOpenService;

        // Get access to single-instance services.
        SingleInstanceService singleInstanceService = null;
        try { singleInstanceService = (SingleInstanceService) ServiceManager.lookup("javax.jnlp.SingleInstanceService"); }
        catch(final UnavailableServiceException e) { }
        sis = singleInstanceService;
//
//        // Get access to extended services.
//        ExtendedService extendedService = null;
//        try { extendedService = (ExtendedService) ServiceManager.lookup("javax.jnlp.ExtendedService"); }
//        catch(final UnavailableServiceException e) { }
//        exs = extendedService;

        if((ps != null) && (bs != null))
            {
            // Try to reload the properties/preferences...
            // This should be quick,
            // and doing it here should avoid races with the UI.
//            try
//                {
//                final FileContents fc = ps.get(makePropsURL(bs));
//                final InputStream inputStream = fc.getInputStream();
//                try { props.loadFromPersistentData(inputStream); }
//                finally { inputStream.close(); }
//                }
//            catch(final IOException e)
//                {
//                logger.log("No saved properties/preferences to reload (yet).");
//                }
            }
        else
            {
            // If there is no persistence...
            }


        if(ALLOW_FALLBACK_FS_FILE_ACCESS)
            {
            logger.log("Allowing fallback filesystem access...");

            // If we are not running in JWS
            // then try to pre-load a test exhibit set from the filesystem.
            if(bs == null)
                {
                final ExhibitDataFileSource edfs = new ExhibitDataFileSource();
                try
                    {
                    final AllExhibitImmutableData aeid = edfs.getAllExhibitImmutableData(-1);
                    _cache_getExhibitName = aeid.getAllExhibitNamesSorted();
                    final int nExhibits = _cache_getExhibitName.length;
                    cachedBasicMetaData = new GalleryBasicMetaData(aeid.timestamp, nExhibits, 1);
                    }
                catch(final IOException e)
                    { e.printStackTrace(); }
                }
            }
        }

    /**If true then we can try the local filesystem as a fallback if not in JWS.
     * This can be an aid to development, especially if working off-line.
     */
    private static final boolean ALLOW_FALLBACK_FS_FILE_ACCESS = false && IsDebug.isDebug;


    /**Reference to central logger; never null. */
    private final SimpleLoggerIF logger;


    /**Handle on JWS basic service; null if none.
     * Package-visible so as to be directly usable by GUI classes.
     */
    final BasicService bs;

    /**Handle on JWS persistence service; null if none.
     * Package-visible so as to be directly usable by GUI classes.
     */
    final PersistenceService ps;

//    /**Handle on JWS file-open service; null if none.
//     * Package-visible so as to be directly usable by GUI classes.
//     */
//    final FileOpenService fos;

    /**Handle on JWS singleton service; null if none.
     * Package-visible so as to be directly usable by GUI classes.
     */
    final SingleInstanceService sis;

//    /**Handle on JWS extended service; null if none.
//     * Package-visible so as to be directly usable by GUI classes.
//     */
//    final ExtendedService exs;


    /**Last time the user was active in this run, eg in the UI; initially object construction time.
     * Volatile for thread-safe access without a lock.
     */
    private volatile long userLastActive = System.currentTimeMillis();

    /**Get the last time the user was active in this run, eg in the UI; initially zero.
     * No harm in letting everyone see this, so is public.
     * <p>
     * When the user has not been active for a long time,
     * some activities, such as polling the server, may halt or slow down
     * to conserve resources.
     */
    public long getUserLastActive() { return(userLastActive); }

    /**Note user as active.
     * Made package-visible (ie not public) for security.
     * <p>
     * This may be triggered/called by a number of events
     * that indicate that the user is active.
     * <p>
     * When the user has not been active for a long time,
     * some activities, such as polling the server, may halt or slow down
     * to conserve resources.
     */
    void setUserLastActive(final String doingWhat)
        {
        userLastActive = System.currentTimeMillis();
//logger.log("UI active: " + doingWhat + ": " + (new Date())); }
        }

    /**Returns true iff the user is considered inactive.
     * This is used to reduce load on the server (and the user's machine)
     * if they have not used the interace for circa several minutes.
     */
    public boolean userInactive()
        {
        return(System.currentTimeMillis() - userLastActive > 10 * 60 * 1000); // 10 minute limit.
        }

    /**Cached copy of the basic Gallery meta-data; never null.
     * Initially EMPTY.
     * <p>
     * Is volatile for thread-safe lock-free access.
     */
    private volatile LightweightMetaDataFetchInterface.GalleryBasicMetaData cachedBasicMetaData = GalleryBasicMetaData.EMPTY;

    /**Time after which we should check metadata with server.
     * Is volatile for thread-safe lock-free access.
     */
    private volatile transient long _metaDataCheckTime;

    /**Return cached copy (or EMPTY if none); never null.
     * This value is always returned from cache,
     * ie we never block or go over the Net to get it,
     * so this call is always fast.
     */
    public GalleryBasicMetaData getGalleryBasicMetaData()
        {
        return(cachedBasicMetaData);
        }

    /**Cache for getExhibitName(); never null.
     * Mapping from index to name.
     * <p>
     * Length should be exhibit count.
     * <p>
     * This may be cleared (replaced by a new empty array of the right length)
     * when a new exhibit set is detected by a changed hash code.
     * <p>
     * Is volatile for safe lock-free access.
     */
    private volatile String[] _cache_getExhibitName = new String[0];

    /**Set of requested exhibit names by ordinal; never null.
     * A lock must be held on this instance during any access to it,
     * since BitSet is not inherently thread-safe.
     * <p>
     * We use our poller thread to fetch names maked as needed.
     */
    private final BitSet _namesNeeded = new BitSet();

    /**Fetch the full name of the given numbered exhibit; null if no such exhibit or not currently available.
     * The range is zero to numberOfExhibits-1.
     * <p>
     * This assumes that there is a sensible stable ordering of exhibits,
     * probably in "smart-sorted" order or similar.
     * <p>
     * This ordering is fixed from one exhibit set to the next,
     * ie is stable while the exhibit set hash remains unchanged.
     * <p>
     * This may go across the Net to fetch its values, so may block for a while.
     * (We aim to cache responses, however,
     * so that repeated retrieval of the same value should be quick,
     * at least until the exhibit set changes.)
     */
    public String getExhibitName(final int ordinal)
        {
        // Capture a snapshot of the cache reference.
        final String[] names = _cache_getExhibitName;

        // Return null if input is out of range.
        if((ordinal < 0) || (ordinal >= names.length)) { return(null); }

        // If the value is cached then return it immediately...
        final String cached = names[ordinal];
        if(cached != null) { return(cached); }

        // Queue this name to be fetched asynchronously.
        // Signal any worker thread that a new item is waiting...
        synchronized(_namesNeeded)
            {
            _namesNeeded.set(ordinal);
            _namesNeeded.notifyAll(); // Signal any waiting worker(s)...
            }

        // Don't have the name ready yet.
        return(null);
        }

    /** Fetch the full name of the given numbered exhibits; each entry null if no such exhibit or not currently available, overall result never null.
     * The range for each item is 0 to numberOfExhibits-1.
     * <p>
     * This assumes that there is a sensible stable ordering of exhibits,
     * probably in "smart-sorted" order or similar.
     * <p>
     * This ordering is fixed from one exhibit set to the next,
     * ie is stable while the exhibit set hash remains unchanged.
     */
    public String[] getExhibitNames(final int ordinals[])
        {
        if((ordinals == null) || (ordinals.length > MAX_NAMES_REQUEST))
            { throw new IllegalArgumentException(); }

        // Potentially rather inefficient implementation.
        final String result[] = new String[ordinals.length];
        for(int i = ordinals.length; --i >= 0; )
            { result[i] = getExhibitName(ordinals[i]); }
        return(result);
        }

    /**Thumbnail (standard) rendered image width and height, positive power-of-two; fixed maximum value for all thumbnails. */
    static final int TN_STD_IMAGE_DIM = ExhibitThumbnails.STD_STATIC_IMAGE_TN_LDIM_PX;

    /**Thumbnail (small) rendered image width and height, positive power-of-two smaller than TN_STD_IMAGE_DIM. */
    static final int TN_SML_IMAGE_DIM = ExhibitThumbnails.SML_STATIC_IMAGE_TN_LDIM_PX;

    /**Thumbnail image format (from Java3D point of view). */
    static final int TN_IMAGE_FORMAT = javax.media.j3d.ImageComponent.FORMAT_RGB;

    /**Power-of-two edge size of default/absent thumbnail stand-in. */
    private static final int defaultTNImage_Edge = 16;

    /**Default thumbnail place-holder image; never null. */
    private static final BufferedImage defaultTNImage = new BufferedImage(defaultTNImage_Edge, defaultTNImage_Edge, BufferedImage.TYPE_INT_RGB);

    /**Initialise defaultTNImage. */
    static
        {
        // Solid grey.
        for(int i = defaultTNImage_Edge; --i >= 0; )
            {
            for(int j = defaultTNImage_Edge; --j >= 0; )
                {
                defaultTNImage.setRGB(i, j, 0xFFCCCCCC);
                }
            }

        // Red diagonal cross to indicate nothing present...
        for(int i = defaultTNImage_Edge; --i >= 0; )
            {
            defaultTNImage.setRGB(i, i, 0xFFFF0000);
            defaultTNImage.setRGB(i, defaultTNImage_Edge - 1 - i, 0xFFFF0000);
            }
        }

    /**Default flags for loading textures.
     * Using BY_REFERENCE may reduce memory usage
     * (and make the texture cache work better)
     * at the cost of some rendering performance.
     */
    private static final int TEXTURE_LOADER_FLAGS = TextureLoader.BY_REFERENCE;

    /**Default thumbnail place-holder image as a texture; never null. */
    private static final Texture defaultTNTexture = (new TextureLoader(defaultTNImage, TextureLoader.BY_REFERENCE)).getTexture();

    /**Thumbnail image for items not yet loaded, minimal power-of-two-edge size; never null. */
    private static final BufferedImage notLoadedTNImage = new BufferedImage(1, 1, BufferedImage.TYPE_INT_RGB);

    /**Initialise notLoadedTNImage. */
    static
        {
        // Solid light red (1 pixel).
        notLoadedTNImage.setRGB(0, 0, 0xFFFF3333);
        }

    /**Thumbnail not-loaded image as a texture; never null. */
    private static final Texture notLoadedTNTexture = (new TextureLoader(notLoadedTNImage, TextureLoader.BY_REFERENCE)).getTexture();


    /**Maximum number of small thumbnail binaries to cache in memory to limit resource use; strictly positive.
     * Each small thumbnail image binary may take up to about 4kBytes.
     * <p>
     * We cap this to a fraction (~20%) of estimated total JVM memory available,
     * though ensure that a we always set a minimum useful size.
     * (If maxMemory() is not limited/set then we use totalMemory().)
     */
    private final static int MAX_SML_TN_CACHE_IN_MEMORY = Math.max(4001,
        (int) (((Runtime.getRuntime().maxMemory() != Long.MAX_VALUE) ? Runtime.getRuntime().maxMemory() : Runtime.getRuntime().totalMemory()) /
            (5 * ExhibitThumbnails.SML_ABS_MAX_BYTES)));
//    static { if(IsDebug.isDebug) { System.out.println("MAX_SML_TN_CACHE_IN_MEMORY="+MAX_SML_TN_CACHE_IN_MEMORY); } }

    /**LRU cache of small thumbnail images for getThumbnailImageAsTexture() in binary form; never null.
     * We hold the binary (compressed) format of the thumbnails,
     * hoping that this does not exhaust memory,
     * so that we can more-or-less guarantee to show something quickly
     * once it has been loaded.
     * <p>
     * A zero-length entry indicates a permanent failure to load the thumbnail.
     */
    private final MemoryTools.SimpleLRUMap<String, ROByteArray> _cache_getThumbnailImage_sml_binary = new MemoryTools.SimpleLRUMap<String, ROByteArray>(MAX_SML_TN_CACHE_IN_MEMORY);

    /**Cache of small thumbnail images for getThumbnailImageAsTexture(); never null.
     * Mapping from full exhibit name to SoftReference to the standard (larger)
     * thumbnail/texture.
     * <p>
     * We use a non-strong reference because the memory requirements may be
     * large and unpredictable.
     * <p>
     * TODO: This should be purged of stale entries periodically
     * and/or when a new exhibit set is detected by a changed hash code
     * in case those entries refer to exhibits that no longer exist.
     * <p>
     * Thread-safe.
     */
    private final Map<String, SoftReference<Texture>> _cache_getThumbnailImage_sml = new Hashtable<String, SoftReference<Texture>>();

    /**Cache of standard (larger) thumbnail images for getThumbnailImageAsTexture(); never null.
     * Mapping from full exhibit name to SoftReference to the standard (larger)
     * thumbnail/texture.
     * <p>
     * We use a non-strong reference because the memory requirements may be
     * large and unpredictable.
     * <p>
     * TODO: This should be purged of stale entries periodically
     * and/or when a new exhibit set is detected by a changed hash code
     * in case those entries refer to exhibits that no longer exist.
     * <p>
     * Thread-safe.
     */
    private final Map<String, SoftReference<Texture>> _cache_getThumbnailImage_std = new Hashtable<String, SoftReference<Texture>>();

    /**Maximum number of thumbnails to fetch in parallel; strictly positive.
     * Too large a number may overwhelm (or be rejected by) the server.
     * <p>
     * Set to more than the number of available processors,
     * to help make use of available resources and overcome I/O latency,
     * though capped to protect our resources and those of the server.
     */
    private static final int MAX_TN_CONC_FETCHES = Math.min(5, 1 + Runtime.getRuntime().availableProcessors());

    /**Map of names of exhibits whose thumbnails currently being fetched to the Threads fetching them; never null.
     * Thread-safe.
     * <p>
     * This may be shared with other per-exhibit threads.
     * <p>
     * The size of this should never be larger than MAX_TN_CONC_FETCHES.
     */
    private final Map<String, Thread> thumbnailsBeingFetched = new Hashtable<String, Thread>();

    /**Minimum time in milliseconds before retrying a thumbnail fetch after any previous failure to fetch it; strictly positive. */
    private static final int MIN_TN_RETRY_MS = 60000; // 1 minute.

    /**Map of thumbnail name to time before we may attempt to refetch following a failure; never null.
     * Thread-safe.
     */
    private final Map<String, Long> tnNoRetryBefore = new Hashtable<String, Long>();

    /**Get thumbnail image as a power-of-two-side square Texture for given image index.
     * Returns a logically-read-only RGB-format image for the given exhibit
     * in a fixed-size (maximum dimension for a standard thumbnail) rect.
     * <p>
     * This routine assumes that the caller will never alter the result,
     * so instances can be safely cached/shared.
     * <p>
     * If the exhibit does not exist or a thumbnail is not available
     * then this returns a standard "grey"/not-present image.
     * <p>
     * An out-of-range request returns a different "not-loaded" image.
     * <p>
     * The underlying image is (re)fetched from the server and cached as need be
     * which should be largely invisible to the Java3D rendering engine.
     * <p>
     * This may release image data that is not used for a long time.
     * <p>
     * This fetches low-priority requests only when not fetching any others.
     * <p>
     * Kept package-visible-only for now.
     *
     * @param ordinal  index of thumbnail texture to retrieve,
     *     else -1 to get standard "not-yet-fetched" thumbnail texture
     * @param std  if true then fetch standard thumbnail, else small
     * @param lowPriority  fetch the texture only if the system is quiet
     *
     * @return null if currently fetching image or too busy to do so,
     *     else the thumbnail as a texture if available
     *     else a standard "no-image" texture
     */
    Texture getThumbnailImageAsTexture(final int ordinal,
                                       final boolean std,
                                       final boolean lowPriority)
        {
        // For negative (always invalid) request return "not-loaded" texture.
        if(ordinal < 0)
            { return(notLoadedTNTexture); /* Should be read-only or a copy... */ }

        // If user is not currently active
        // then refuse to fetch thumbnails to save load here and on the server.
        if(userInactive()) { return(null); }

        // Return null texture if we cannot get the exhibit's name.
        // This may be because the name is not yet in cache
        // and we do not want to block...
        final String exhibitName = getExhibitName(ordinal);
        if(!ExhibitName.validNameSyntax(exhibitName))
            { return(null); /* Should be read-only or a copy... */ }

        // If any other fetch threads are running
        // then refuse to handle a low-priority request right now.
        // We don't even need to lookup the exhibit name, etc.
        // (We were prepared, however, to get the name fetched above.)
        if(lowPriority && !thumbnailsBeingFetched.isEmpty())
            {
//if(IsDebug.isDebug) { logger.log("[Deferring low-priority thumbnail fetch for #"+ordinal+".]"); }
            return(null);
            }

        // Get a handle on the appropriate thumbnail texture cache...
        final Map<String, SoftReference<Texture>> cache =
            std ? _cache_getThumbnailImage_std : _cache_getThumbnailImage_sml;

        // If we have something already cached then return it immediately...
        final SoftReference<Texture> sr = cache.get(exhibitName);
        if(sr != null)
            {
            final Texture texture = sr.get();
            if(texture != null) { return(texture); }
            }

        // Return default texture for unsupported type
        // (ie if not a still-image type).
        final ExhibitMIME.ExhibitTypeParameters exhibitType = ExhibitMIME.getInputFileType(exhibitName);
        if(!Utils.supportedTypeFor3DWT(exhibitType))
            { return(defaultTNTexture); /* Should be read-only or a copy... */ }

        // Check if we have a block on fetching this thumbnail.
        final Long l = tnNoRetryBefore.get(exhibitName);
        if((l != null) && (l.longValue() > System.currentTimeMillis()))
            { return(null); }

        // Avoid races when starting new threads...
        synchronized(thumbnailsBeingFetched)
            {
            // If this is already in the process of being fetched,
            // or we are already concurrently fetching as many as is allowed,
            // then return null now.
            Thread th = thumbnailsBeingFetched.get(exhibitName);
            if((th != null) && !th.isAlive())
                {
                thumbnailsBeingFetched.remove(exhibitName);
                logger.log("WARNING: removed dead thread for fetching " + exhibitName);
                th = null;
                }
            // Give hi-res/high-priority fetches priority:
            // don't let lo-res/low-priority fetches take the last slot/thread.
            final int inProgress = thumbnailsBeingFetched.size();
            if((th != null) ||
               (inProgress >= MAX_TN_CONC_FETCHES) ||
               (lowPriority && !std && (inProgress > 0) && (inProgress >= MAX_TN_CONC_FETCHES-1)))
                { return(null); }

            // Just show the short name to reduce "noise" for the user...
            final String shortName = ExhibitName.getFileComponent(exhibitName);

            // Create new thread to fetch texture not in the cache...
            // We also regulate system load (client and server) a little here.
            final Thread fetchThread = new Thread("fetching thumbnail for " + shortName){
                @Override public final void run()
                    {
                    final long startTime = System.currentTimeMillis();

                    try
                        {
                        // Since the exhibit name may live for a long while
                        // and may be used in a number of places
                        // we intern() it to save memory.
                        final String name = MemoryTools.intern(exhibitName);

                        // Put a block on retrying any refetch after this...
                        // And avoid any concurrent/duplicate fetch.
                        tnNoRetryBefore.put(name, System.currentTimeMillis() + MIN_TN_RETRY_MS);

                        // Note fetch/refetch in status area...
                        logger.log(((cache.get(name) != null) ? "Refetching" : "Fetching")+
                                   " thumbnail for "+shortName+"...");

                        // Clear any stale value from the cache.
                        cache.remove(name);

                        // Attempt to fetch the texture synchronously...
                        fetchTexture(name, cache, std);

                        if(cache.get(name) != null)
                            {
                            // Assumed to be a success
                            // (or a permanent failure).
                            // Allow an immediate refetch in future if need be.
                            tnNoRetryBefore.remove(name);
                            }
                        else
                            {
                            // Assume that we failed: postpone any retry.
                            // Add in random component to help avoid collisions.
                            tnNoRetryBefore.put(name, System.currentTimeMillis() + MIN_TN_RETRY_MS +
                                                      Rnd.fastRnd.nextInt(MIN_TN_RETRY_MS));
                            }
                        }
                    finally
                        {
                        if(thumbnailsBeingFetched.size() == 1)
                            { logger.log("Finished fetching thumbnail for "+shortName+"."); }

                        // Regulate load, at least for low-priority fetches.
                        if(lowPriority)
                            {
                            try
                                {
                                // Try to make sure that we do not consume all CPU
                                // on this client system nor at the server
                                // by sleeping a multiple of fetch wall-clock time.
                                // Put an upper bound of a few seconds on this.
                                Thread.sleep(1 + Math.min(10101, Math.max(1,
                                    1 * (System.currentTimeMillis() - startTime))));
                                }
                            catch(final InterruptedException e)
                                {
                                e.printStackTrace(); /* Absorb error, though whinge and finish immediately... */
                                }
                            }

                        assert(this == thumbnailsBeingFetched.get(exhibitName));
                        thumbnailsBeingFetched.remove(exhibitName);

                        if(thumbnailsBeingFetched.isEmpty())
                            { logger.log("Idle."); }
                        }
                    }
                };
            fetchThread.setDaemon(true);
            // Attempt to actually run low-priority fetches
            // on lower-than-normal (lower-than-caller) priority threads...
            fetchThread.setPriority(Math.max(Thread.currentThread().getPriority()-(lowPriority?4:2), Thread.MIN_PRIORITY));
            thumbnailsBeingFetched.put(exhibitName, fetchThread);
            fetchThread.start();
            }

        // Do not wait for result...
        return(null); // In process of fetching texture...
        }

    /**Root-relatve absolute URL prefix for standard thumbnails; starts and ends with '/'. */
    private static final String TN_STD_RRURL_PREFIX = "/_tn/std/";

    /**Root-relatve absolute URL prefix for small thumbnails; starts and ends with '/'. */
    private static final String TN_SML_RRURL_PREFIX = "/_tn/sml/";

    /**If true then allow fetchTexture() to return a small thumbnail where the standard one is unavailable. */
    private static final boolean ALLOW_TEXTURE_FALLBACK = true;

    /**If true then allow local persistance of small thumbnails in their binary form using JWS.
     * As of 200603 there seems to be a problem storing more than 255 muffins total!
     */
    private static final boolean CACHE_JWS_TN_SML = false;

    /**If true then allow local cacheing of small thumbnails in their binary form in memory.
     */
    private static final boolean CACHE_MEM_TN_SML = true;

    /**Adjust URLs to be suitable for JWS muffins.
     * JWS does not seem to like directory components,
     * so we flatten the URL at some slight risk of ambiguity.
     */
    private static URL _adjustURLForMuffin(final URL full)
        throws MalformedURLException
        {
        return(new URL(full.getProtocol(),
                       full.getHost(),
                       full.getPort(),
//                       (full.getFile().startsWith("/") ? full.getFile().substring(1) : full.getFile())));
                       full.getFile().replace('/', '-')));
        }

    /**Private cache value for fetchTexture(). */
    private transient volatile Object _c_fT;

    /**Fetch the image/texture synchronously from the server.
     * Caches the image if successful,
     * else leaves the cache slot alone.
     * <p>
     * This never deletes cache entries.
     * <p>
     * This may try to fall back to the small thumbnail
     * if the large one cannot be fetched...
     *
     * @param exhibitName  the full valid name of the exhibit
     * @param cache  the cache to save the texture in
     * @param std  if true then fetch standard thumbnail, else small
     */
    private void fetchTexture(final String exhibitName,
                              final Map<String, SoftReference<Texture>> cache,
                              final boolean std)
        {
        assert(ExhibitName.validNameSyntax(exhibitName));
        assert(cache != null);

if(IsDebug.isDebug) { System.out.println("Request for thumbnail: " + ExhibitName.getFileComponent(exhibitName)); }

        // Note if raw thumbnails image binary read from local cache;
        // if so, don't write it back to the same cache!
        boolean readFromMemory = false;
        boolean readFromPersistentStore = false;

        // Get thumbnail as (encoded) input stream...
        InputStream is = null;
        try
            {
            // Try to fetch thumbnail from memory if possible.
            if(CACHE_MEM_TN_SML && !std)
                {
                final ROByteArray binaryData = _cache_getThumbnailImage_sml_binary.get(exhibitName);
                if((binaryData != null) && (binaryData.length() > 0))
                    {
                    is = new ByteArrayInputStream(binaryData.getData());
                    readFromMemory = true;
if(IsDebug.isDebug) { System.out.println("Retrieved thumbnail from memory store: " + exhibitName); }
                    }
                }

            // Try to fetch image relative to code base by preference.
            // Construct URL to fetch thumbnail.
//        final String tnRRL = WebUtils.makeThumbnailRRURL(exhibitName, true);
            final String tnRRL = (std ? TN_STD_RRURL_PREFIX : TN_SML_RRURL_PREFIX)+ exhibitName;
            URL tnURL = null;
            if(bs != null)
                {
                try { tnURL = new URL(bs.getCodeBase(), tnRRL); }
                catch(final Exception e) { if(IsDebug.isDebug) { e.printStackTrace(); } }
                }

            // Allow read from local filesystem exhibit store if enabled...
            // (And if we haven't already got the data we need...)
            if(ALLOW_FALLBACK_FS_FILE_ACCESS && (tnURL == null) && (is == null))
                {
                try
                    {
                    // Create file source on first use...
                    ExhibitDataFileSource edfs = (ExhibitDataFileSource) _c_fT;
                    if(edfs == null) { _c_fT = edfs = new ExhibitDataFileSource(); }
                    assert(edfs != null);
                    final AllExhibitProperties aep = edfs.getAllExhibitProperties(-1);
                    final ExhibitStaticAttr esa = aep.aeid.getStaticAttr(exhibitName);
                    if(esa == null)
                        {
                        System.out.println("fetchTexture(): cannot find local exhibit: " + exhibitName);
                        cache.put(exhibitName, new SoftReference<Texture>(defaultTNTexture));
                        return;
                        }
                    else
                        {
                        // OK, attempt to generate thumbnails here, not under any lock.
                        final ExhibitMIME.ExhibitTypeParameters exhibitType = (ExhibitMIME.getInputFileType(esa.filePath));
                        final ExhibitThumbnails result = exhibitType.handler.makeThumbnails(
                                    esa,
                                    edfs.makeExhibitDataSource(),
                                    aep,
                                    true);

                        if(result == null)
                            { return; /* Not available yet; don't update cache. */ }
                        final ExhibitThumbnails.Thumbnail thumbnail;
                        if((thumbnail = (std ? result.getStandard() : result.getSmall())) != null)
                            {
                            System.out.println("fetchTexture(): generated thumbnail for local exhibit: " + exhibitName);
                            is = new ByteArrayInputStream(thumbnail.toByteArrray());
                            }
                        else if(ALLOW_TEXTURE_FALLBACK && (std == true))
                            {
                            fetchTexture(exhibitName, cache, false);
                            return;
                            }
                        else
                            {
if(IsDebug.isDebug) { System.out.println("Request for thumbnail failed from local filesystem."); }
                            cache.put(exhibitName, new SoftReference<Texture>(defaultTNTexture));
                            return;
                            }
                        }
                    }
                catch(final Throwable t)
                    {
                    t.printStackTrace();
                    // If we fail to create from local filesystem
                    // then don't try again...
if(IsDebug.isDebug) { System.out.println("Request for thumbnail failed from local filesystem with an IOException."); }
                    cache.put(exhibitName, new SoftReference<Texture>(defaultTNTexture));
                    return;
                    }
                }

            // Try to fetch tn URL as a stream from local persistent store.
            // Try this for all URLs as it should be quick...
            if((CACHE_JWS_TN_SML) && (ps != null) && (is == null) && (tnURL != null))
                {
                try
                    {
                    final FileContents fileContents = ps.get(_adjustURLForMuffin(tnURL));
                    is = fileContents.getInputStream();
                    readFromPersistentStore = true;
if(IsDebug.isDebug) { System.out.println("Read thumbnail from persistent store: " + tnURL); }
                    }
                catch(final FileNotFoundException e) { /* e.printStackTrace(); */ }
                catch(final Throwable t) { t.printStackTrace(); }
                }

            // Fallback...
            // If JWS codebase is not available then try the main data host.
            if((tnURL == null) && (is == null))
                {
                try { tnURL = new URL("http", CoreConsts.MAIN_DATA_HOST, tnRRL); }
                catch(final Exception e) { if(IsDebug.isDebug) { e.printStackTrace(); } }
                }

            // Try to fetch tn URL as a stream...
            if((is == null) && (tnURL != null))
                {
if(IsDebug.isDebug) { System.out.println("Will try to fetch thumbnail from: " + tnURL); }
                final URLConnection urlConnection = tnURL.openConnection();
                // Adjust the connection properties to still behave OK
                // given a slow/lossy connection.
                urlConnection.setConnectTimeout(Utils.RPC_HTTP_CONNECT_TIMEOUT_MS); // Cap the connect time in case of slow/lossy network.
                urlConnection.setReadTimeout(Utils.RPC_HTTP_READ_TIMEOUT_MS); // Avoid getting stuck in a read too.
                urlConnection.setUseCaches(true); // Take advantage of any cacheing available!
                urlConnection.setAllowUserInteraction(false);
                urlConnection.setRequestProperty("Referer", "http://" + CoreConsts.MAIN_DATA_HOST + Utils.SERVLET_MOUNT_POINT);
                if(urlConnection instanceof HttpURLConnection)
                    {
                    final HttpURLConnection huc = (HttpURLConnection) urlConnection;
                    final int responseCode = huc.getResponseCode();
                    final int topDigit = responseCode / 100;
                    if(topDigit == 4)
                        {
                        // For a permanent failure use the default texture.
                        // We should not try again.
                        // Note that the strong reference to the default texture
                        // should keep this SoftReference live indefinitely.
if(IsDebug.isDebug) { System.out.println("Request for thumbnail failed with code "+responseCode+"."); }

                        if(ALLOW_TEXTURE_FALLBACK && (std == true))
                            { fetchTexture(exhibitName, cache, false); }
                        else
                            { cache.put(exhibitName, new SoftReference<Texture>(defaultTNTexture)); }
                        return;
                        }
                    }
                is = urlConnection.getInputStream();
                }

            // If we have the input stream, then try to:
            //   * copy/cache the data if applicable
            //   * convert to a texture (and cache the texture)!
            if(is != null)
                {
                // If we are allowed to persist locally
                // then keep a copy of the input stream.
                // Don't (re)cache in persistent store if fetched from memory.
                final boolean canCacheInMemory = (CACHE_MEM_TN_SML && !std) && !readFromMemory;
                final boolean canPersistData = !readFromMemory && (CACHE_JWS_TN_SML && !std) && !readFromPersistentStore && (ps != null);
                final boolean captureDataCopy = canPersistData || canCacheInMemory;
                if(captureDataCopy)
                    {
                    final int maxSize = std ? ExhibitThumbnails.STD_ABS_MAX_BYTES : ExhibitThumbnails.SML_ABS_MAX_BYTES;

                    final InputStream oldIs = is;

                    final ByteArrayOutputStream dataCopy = new ByteArrayOutputStream(maxSize);
                    final byte buf[] = new byte[1024];
                    for( ; ; )
                        {
                        final int n = is.read(buf);
                        if(n < 1) { break; /* EOF */ }
                        dataCopy.write(buf, 0, n);

                        if(dataCopy.size() > maxSize)
                            { throw new IOException("corrupt thumbnail data (too large): "+dataCopy.size()+" vs "+maxSize); }
                        }

                    // Close the original input stream...
                    oldIs.close();

                    // Replace with new buffered input.
                    is = (new ByteArrayInputStream(dataCopy.toByteArray()));

                    // Persist the tn data that we have just collected...
                    // Give up quietly if this fails for any reason
                    // (eg it already exists, because we just read from it).
                    try
                        {
                        if(canCacheInMemory)
                            {
                            // Cache in memory...
                            _cache_getThumbnailImage_sml_binary.put(exhibitName, new ROByteArray(dataCopy.toByteArray()));
if(IsDebug.isDebug) { System.out.println("Wrote thumbnail to memory store: " + exhibitName); }
                            }
                        if(canPersistData)
                            {
                            final URL flattenedURL = _adjustURLForMuffin(tnURL);
                            if(ps.create(flattenedURL, dataCopy.size()) >= dataCopy.size())
                                {
                                // Mark file as cached copy, ie discardable at will...
                                ps.setTag(flattenedURL, PersistenceService.CACHED);
                                final FileContents fileContents = ps.get(flattenedURL);
                                final OutputStream os = fileContents.getOutputStream(true); // Replace any extant entry...
                                os.write(dataCopy.toByteArray());
                                os.close();
if(IsDebug.isDebug) { System.out.println("Wrote thumbnail to persistent store: " + tnURL); }
                                }
                            else { ps.delete(flattenedURL); } // Not enough space, so zap muffin.
                            }
                        }
                    catch(final Throwable t) { t.printStackTrace(); /* Whinge but ignore. */ }
                    }

                try
                    {
                    // Synchronous thumbnail image loading...
//                    final long imgPrepStart = System.currentTimeMillis();
                    final BufferedImage bufferedImageRawTN = javax.imageio.ImageIO.read(is);
                    if(bufferedImageRawTN == null)
                        { throw new IOException("could not decode image"); }
                    final BufferedImage bufferedImageTrueCol = ImageUtils.convertToTrueColourARGB(bufferedImageRawTN, false);
                    final int height = bufferedImageTrueCol.getHeight();
                    final int width = bufferedImageTrueCol.getWidth();
                    final int targetDim = std ? TN_STD_IMAGE_DIM : TN_SML_IMAGE_DIM;
                    final BufferedImage thImage = new BufferedImage(targetDim, targetDim, BufferedImage.TYPE_INT_ARGB);
                    final int bgRGB = std ? 0x80333333 : 0x80CC9999;
                    // TODO: make this all more efficient!
//                    final long imgPrep0 = System.currentTimeMillis();
                    for(int x = targetDim; --x >= 0; )
                        {
                        for(int y = targetDim; --y >= 0; )
                            {
                            // Possible partial transparency...
                            thImage.setRGB(x, y, bgRGB);
                            }
                        }
//                    final long imgPrep1 = System.currentTimeMillis();
                    final int left = (targetDim-width)/2;
                    final int top = (targetDim-height)/2;
//                compositeImage.getSubimage(left, top, width, height).getRaster().
//                    setRect(bufferedImageTrueCol.getRaster());
                    for(int x = width; --x >= 0; )
                        {
                        final int xo = x + left;
                        if((xo < 0) || (xo >= targetDim)) { continue; }
                        for(int y = height; --y >= 0; )
                            {
                            final int yo = y + top;
                            if((yo < 0) || (yo >= targetDim)) { continue; }
                            thImage.setRGB(xo, yo, bufferedImageTrueCol.getRGB(x, y));
                            }
                        }
//                    final long imgPrep2 = System.currentTimeMillis();
                    final TextureLoader textureLoader = new TextureLoader(thImage, TEXTURE_LOADER_FLAGS);
                    final Texture texture = textureLoader.getTexture();
//                    final long imgPrep3 = System.currentTimeMillis();
//if(IsDebug.isDebug)
//    {
//    System.out.println("  imgPrep "+(std?"std":"sml")+" (ms) 0/1/2/3: " + (imgPrep0-imgPrepStart) + "/" + (imgPrep1-imgPrep0)+ "/" + (imgPrep2-imgPrep1) + "/" + (imgPrep3-imgPrep2));
//    }
                    if(texture != null) // Success!
                        {
if(IsDebug.isDebug) { System.out.println("Request for thumbnail succeeded: " + exhibitName); }
                        cache.put(exhibitName, new SoftReference<Texture>(texture));
                        return; // Done!
                        }
                    }
                // Possibly want to defer any further attempt
                // to fetch this thumbnail for a while...
                catch(final Throwable t) { t.printStackTrace(); }
                }
            }
        catch(final IOException e)
            {
//            e.printStackTrace(); // Failed...
            logger.log("[Problem fetching thumbnail "+exhibitName+": " + e.getMessage());
            }
        finally
            {
            // Release resources ASAP.
            if(is != null)
                {
                try { is.close(); /* Close any open stream... */ }
                catch(final IOException e) { e.printStackTrace(); }
                }
            }

        // Failed; leave cache untouched.
if(IsDebug.isDebug) { System.out.println("Request for thumbnail failed: " + exhibitName); }
        }


    /**Load any (large) persisted data from a previous execution and start a background worker thread.
     * Do this as early as possible after construction,
     * and preferably before allowing (much) user interaction.
     * <p>
     * We spin this off into a separate thread to avoid blocking startup.
     * <p>
     * This should be called at most once.
     * <p>
     * Package-visible so as to be directly usable by GUI classes.
     */
    void startup()
        {
        final Thread th = new Thread("3D logic startup()"){
            /**Do load work as background thread. */
            public final void run()
                {
//                if((ps != null) && (bs != null))
//                    {
//                    // ...
//                    }

                // Run the first poll manually/synchronously.
                poll();

                // Once we have finished reloading any persisted data,
                // start a (daemon) poller thread for non-UI async activity
                // such as fetching any updated AEP over the tunnel.
                // After a short delay, run approximately every second or so.
                // This is created/started after initial UI object construction
                // is complete to avoid any unpleasant races.
                final Timer timer = new Timer(true);
                timer.schedule(new TimerTask(){
                    /**The action to be performed by this task. */
                    public final void run()
                        {
                        if(shuttingDown) { cancel(); }
                        else { poll(); }
                        }
                    }, 500, 901 + Rnd.fastRnd.nextInt(1003));
                }
            };
        th.setDaemon(true); // Don't prevent the application from exiting...
        th.start();

        // Try to fetch names with high priority.
        try { nameFetchWorker.setPriority(Thread.MAX_PRIORITY); }
        catch(final Throwable t) { }
        nameFetchWorker.setDaemon(true); // Don't prevent the application from exiting...
        nameFetchWorker.start();

        // Wait a short while for first poll to complete before returning
        // in the hope that this might save doing some UI work twice...
        try { th.join(CoreConsts.MAX_INTERACTIVE_DELAY_MS); }
        catch(final InterruptedException e) { }
        }

//    /**Make the props persistence URL; only viable when in JWS/JNLP. */
//    private static URL makePropsURL(final BasicService bs) throws IOException
//        {
//        final URL codebase = bs.getCodeBase();
//        final URL urlProps = new URL(codebase, FNAME_MAIN_PROPS);
//        return(urlProps);
//        }


    /**Called (by a daemon thread) to perform async activity.
     * This is not Swing-safe, and does not block Swing if blocked.
     * <p>
     * This is not called until construction is complete.
     * <p>
     * Package-visible so as to be directly usable by the main GUI class.
     */
    private void poll()
        {
        final long pollStart = System.currentTimeMillis();
        try
            {
            // If user is not currently active
            // then refuse to do the polling to save load here and on server.
            if(userInactive()) { return; }

            // Where we have picked up a local exhibit set,
            // don't confuse things by trying to get metadata from a server!
            final boolean useLocalExhibitSet =
                ALLOW_FALLBACK_FS_FILE_ACCESS && (bs == null) && (cachedBasicMetaData.exhibitCount != 0);

            if(!useLocalExhibitSet && (pollStart > _metaDataCheckTime) &&
               Rnd.fastRnd.nextBoolean() /* Sometimes skip this read attempt. */ )
                {
                try
                    {
if(IsDebug.isDebug) { logger.log("[Fetching metadata from server...]"); }

                    // Time to check the exhibit metadata with the server,
                    // and clear caches as appropriate
                    // if the exhibit set has changed.
                    final GalleryBasicMetaData md =
                        Utils.getMetaData((bs == null) ? null : bs.getCodeBase());
if(IsDebug.isDebug) { logger.log("[Fetched metadata: "+md+".]"); }
                    if(cachedBasicMetaData.hashCode() != md.hashCode())
                        {
if(IsDebug.isDebug) { logger.log("[Metadata has changed: "+md+".]"); }
                        // Data has changed; update our caches!
                        _cache_getExhibitName = new String[md.exhibitCount];
                        cachedBasicMetaData = md;
                        synchronized(_namesNeeded) { _namesNeeded.clear(); }
                        // TODO: start cleaner thread for texture/image cache...
                        }
                    // Put off the next metadata fetch as suggested by the server.
                    // Coerce the suggested wait to reasonable bounds.
                    _metaDataCheckTime = pollStart + Math.max(60000, // Avoid pestering server too often.
                        Math.min(CoreConsts.DEFAULT_TEMPORAL_SLACKNESS_S * 750,
                                 md.metaDataMaxCacheLifeMs)) +
                        // Throw in a random component to help unsync requests.
                        Rnd.fastRnd.nextInt(10201);
                    }
                catch(final IOException e)
                    {
                    e.printStackTrace(); /* Absorb error but report it. */
                    logger.log("Problem fetching gallery metadata, please check your network connection: " + e.getMessage());
                    }
                }

            // Pitch in with the dedicated worker thread for a short while
            // to fetch any names that have been requested (by index).
            final long nameFetchStart = System.currentTimeMillis();
            final long stopBy = nameFetchStart + 1003;
            _fetchNames(stopBy, _cache_getExhibitName);

            // Note count of names still queued for fetching...
            synchronized(_namesNeeded)
                {
                if(!_namesNeeded.isEmpty())
                    { logger.log("[Names still to fetch: "+_namesNeeded.cardinality()+"...]"); }
                }

//if(IsDebug.isDebug) { System.out.println("[Memory use free/total = "+Runtime.getRuntime().freeMemory()+"/"+Runtime.getRuntime().totalMemory()+".]"); }
            }
        catch(final Throwable t)
            {
            t.printStackTrace();  /* Absorb error, but whinge. */
            }
        }

    /**Worker thread that does nothing but get names in the background.
     * Exits when this is shut down.
     */
    private final Thread nameFetchWorker = new Thread("name fetch worker"){
        @Override public final void run()
            {
            while(!shuttingDown)
                {
                try
                    {
                    // Wait until we see a pending name request.
                    synchronized(_namesNeeded)
                        {
                        while(_namesNeeded.isEmpty())
                            { _namesNeeded.wait(1111 + Rnd.fastRnd.nextInt(1001)); }
                        }

                    // Spend a little while getting requested names.
                    _fetchNames(System.currentTimeMillis() + 120000, _cache_getExhibitName);
                    }
                catch(final Throwable t)
                    {
                    t.printStackTrace();  /* Absorb error, but whinge. */
                    }
                }
            }
        };

    /**Fetch all outstanding names that we can, up to the specified time limit.
     * It is safe to run two or more of these concurrently,
     * though there is a small risk of races causing duplicate fetches.
     * Note really efficient at high levels of concurrency.
     */
    private void _fetchNames(final long stopBy, final String[] names)
        {
        final List<Integer> needed = new ArrayList<Integer>(LightweightMetaDataFetchInterface.MAX_NAMES_REQUEST);
        for(int nextNameNeeded = -1; System.currentTimeMillis() < stopBy; )
            {
            // Find next needed names index under the lock,
            // else quit if none needed.
            // Collect an efficient-size block,
            // and clear them as we go to prevent another thread
            // trying to collect the same ones.
            needed.clear();
            synchronized(_namesNeeded)
                {
                do
                    {
                    // Look for next name.
                    nextNameNeeded = _namesNeeded.nextSetBit(++nextNameNeeded);
                    if(nextNameNeeded < 0)
                        { break; /* Finished; no more names to fetch. */ }
                    else if(nextNameNeeded >= names.length)
                        {
                        _namesNeeded.clear(); /* Spurious values presumably after exhibit set shrank. */
                        break; /* Done. */
                        }
                    else if(names[nextNameNeeded] != null)
                        {
                        _namesNeeded.clear(nextNameNeeded); /* Already fetched. */
                        continue;
                        }

                    needed.add(Integer.valueOf(nextNameNeeded));
                    _namesNeeded.clear(nextNameNeeded); // Stop any other thread from picking this up...
                    } while((nextNameNeeded != -1) && (needed.size() < LightweightMetaDataFetchInterface.MAX_NAMES_REQUEST));
                }

            // If no requests found then quit the loop...
            if(needed.isEmpty()) { break; }

            // OK, we actually do need to fetch these names...
            // Do it outside of the lock scope.
            try
                {
if(IsDebug.isDebug) { logger.log("[NAMES: Fetching "+(needed.size())+" names " + needed + "...]"); }
                final int ordinals[] = new int[needed.size()];
                for(int i = ordinals.length; --i >= 0; )
                    { ordinals[i] = needed.get(i).intValue(); }
                final String results[] = Utils.getExhibitNames((bs == null) ? null : bs.getCodeBase(), ordinals);
                for(int i = ordinals.length; --i >= 0; )
                    {
                    final int ordinal = ordinals[i];
                    names[ordinal] = MemoryTools.intern(results[i]); // Name likely to be long-lived.
                    // Clear flag given that name has been fetched successfully...
                    assert(results[i] != null);
                    synchronized(_namesNeeded) { _namesNeeded.clear(ordinal); }
                    }
                }
            catch(final Exception e) /* Absorb error, but whinge. */
                {
                if(IsDebug.isDebug) { e.printStackTrace(); }
                logger.log("Problem fetching exhibit names, please check your network connection: " + e.getMessage());

                // Pause a moment in case of a persistent problem
                // to avoid eating all the CPU in a high-priority thread.
                try { Thread.sleep(1001 + Rnd.fastRnd.nextInt(1001)); }
                catch(final InterruptedException ie) { }

                break; /* After any error, stop trying to fetch more on this round. */
                }
            }
        }


//    /**Save persistent properties (if possible).
//     * If we have PersistenceService then
//     * try to create/allocate space to save (ignoring errors if already there)
//     * and save the data.
//     */
//    private void saveProperties()
//        {
//        if((ps != null) && (bs != null))
//            {
//            try
//                {
//                final URL propsURL = makePropsURL(bs);
//
//                // Try to create a space for persisting the data
//                // if not already present (ignoring error if it is).
//                try { ps.create(propsURL, UploaderProps.MAX_PERS_BYTES); }
//                catch(final IOException e) { }
//
//                // Now try to save our properties data.
//                final FileContents fc = ps.get(propsURL);
//                final OutputStream os = fc.getOutputStream(true);
//                try { os.write(props.getPersistentData()); }
//                finally { os.close(); }
//
//if(IsDebug.isDebug) { logger.log("Saved properties..."); }
//                }
//            catch(final IOException e)
//                {
//                e.printStackTrace();
//                logger.log("Cannot save properties: " + e.getMessage());
//                }
//            }
//        }

    /**Set true when we are shutting down (and never set false again). */
    private volatile boolean shuttingDown;

    /**Perform any activity required to shut down cleanly, eg save state.
     * This should try to avoid taking a long time.
     * <p>
     * Package-visible so as to be directly callable by the main GUI class.
     */
    void shutdown()
        {
        shuttingDown = true;

//        // Save any state...
//        saveProperties();
        }
    }
