/* DERIVED FROM APACHE CODE, modifications (c) Damon Hart-Davis 2001-2006, under same terms:
 *
 * CompressionServletResponseWrapper.java
 * $Header: /rw/CVS-repository.s0.l/general/PG2Ksrc/javasrc/webSvr/org/hd/d/pg2k/webSvr/util/CompressionServletResponseWrapper.java,v 1.3 2004/11/06 16:27:58 dhd Exp $
 * $Revision: 1.3 $
 * $Date: 2004/11/06 16:27:58 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.hd.d.pg2k.webSvr.util;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import ORG.hd.d.IsDebug;

import java.io.IOException;
import java.io.PrintWriter;

/**
 * Implementation of <b>HttpServletResponseWrapper</b> that works with
 * the CompressionServletResponseStream implementation.
 *
 * @author Amy Roh
 * @version $Revision: 1.3 $, $Date: 2004/11/06 16:27:58 $
 */

public class CompressionServletResponseWrapper extends HttpServletResponseWrapper {

    // ----------------------------------------------------- Constructor

    /**
     * Calls the parent constructor which creates a ServletResponse adaptor
     * wrapping the given response object.
     */

    public CompressionServletResponseWrapper(HttpServletResponse response) {
      super(response);
      origResponse = response;
      //System.out.println("CompressionServletResponseWrapper constructor gets called");
    }


    // ----------------------------------------------------- Instance Variables

    /**
     * Original response
     */

    protected HttpServletResponse origResponse = null;

    /**
     * Descriptive information about this Response implementation.
     */

    protected static final String info = "CompressionServletResponseWrapper";

    /**
     * The ServletOutputStream that has been returned by
     * <code>getOutputStream()</code>, if any.
     */

    protected ServletOutputStream stream = null;


    /**
     * The PrintWriter that has been returned by
     * <code>getWriter()</code>, if any.
     */

    protected PrintWriter writer = null;

    /**
     * The threshold number to compress
     */
    protected int threshold = 0;


    // --------------------------------------------------------- Public Methods


    /**
     * Set threshold number
     */
    public void setCompressionThreshold(int threshold) {
        //System.out.println("setCompressionThreshold @ CompressionServletResponseWrapper");
        this.threshold = threshold;
    }

    /**
     * Create and return a ServletOutputStream to write the content
     * associated with this Response.
     *
     * @exception IOException if an input/output error occurs
     */
    public ServletOutputStream createOutputStream() throws IOException {

        //System.out.println("createOutputStream gets called");
        return(new CompressionResponseStream(this, origResponse));

    }


    /**
     * Finish a response.
     */
    public void finishResponse() {
        try {
            if (writer != null) {
                writer.close();
            } else {
                if (stream != null)
                    stream.close();
            }
        } catch (IOException e) {
        }
    }


    // ------------------------------------------------ ServletResponse Methods


    /**
     * Flush the buffer and commit this response.
     *
     * @exception IOException if an input/output error occurs
     */
    public void flushBuffer() throws IOException {

        //  System.out.println("flush buffer @ CompressionServletResponseWrapper");
        stream.flush();

    }

    /**
     * Return the servlet output stream associated with this Response.
     *
     * @exception IllegalStateException if <code>getWriter</code> has
     *  already been called for this response
     * @exception IOException if an input/output error occurs
     */
    public ServletOutputStream getOutputStream() throws IOException {

        if (writer != null)
            throw new IllegalStateException("getWriter() has already been called for this response");

        if (stream == null)
            stream = createOutputStream();
        //System.out.println("stream is set to "+stream+" in getOutputStream");
        ((CompressionResponseStream) stream).setBuffer(threshold);
	    return (stream);

    }

    /**
     * Return the writer associated with this Response.
     *
     * @exception IllegalStateException if <code>getOutputStream</code> has
     *  already been called for this response
     * @exception IOException if an input/output error occurs
     */
    public PrintWriter getWriter() throws IOException {

        if (writer != null)
            return (writer);

        if (stream != null)
            throw new IllegalStateException("getOutputStream() has already been called for this response");

        stream = createOutputStream();
        //System.out.println("strean is set to "+stream+" in getWriter");
        ((CompressionResponseStream) stream).setBuffer(threshold);
        writer = new PrintWriter(stream);
        return (writer);

    }

    /**The value set for the Content-Length header, or zero if not (yet) set. */
    private int contentLength;

    /**Get the value set for the Content-Length header, or zero if not (yet) set.
     * This is only made package-visible.
     */
    int getContentLength()
        { return(contentLength); }

    /**Prevent the content length being set directly.
     * We do this because if we do apply compression
     * then the value in this header will be wrong (too large).
     * <p>
     * But if we don't apply compression
     * then we may wish to re-apply this value.
     */
    @Override public void setContentLength(final int length)
        { contentLength = length; }

    /**The value set for the Content-MD5 header, or null if not (yet) set. */
    private String contentMD5;

    /**Get the value set for the Content-MD5 header, or null if not (yet) set.
     * This is only made package-visible.
     */
    String getContentMD5()
        { return(contentMD5); }


    /**Intercept any headers that interact with compression and Content-Encoding.
     * We do this because if we do apply compression/encoding
     * then the user-supplied value in some of these headers would be wrong.
     * <p>
     * But if we don't apply compression
     * then we may wish to re-apply some of these values.
     */
    @Override public void setHeader(final String name, final String value)
        {
        if("Content-MD5".equalsIgnoreCase(name))
            {
if(IsDebug.isDebug) { System.out.println("Intercepted set of Content-MD5 header"); }
            contentMD5 = value;
            return;
            }

        // By default pass all sets/changes on.
        super.setHeader(name, value);
        }

}
