/*
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
package org.hd.d.pg2k.webSvr.util;

/* FIXME:
 * 
 * java.util.ConcurrentModificationException
        at java.util.Hashtable$Enumerator.next(Hashtable.java:1020)
        at java.util.AbstractCollection.addAll(AbstractCollection.java:318)
        at java.util.HashSet.<init>(HashSet.java:100)
        at org.hd.d.tmf.ThroughputMonitorFilterBase._updateCurrentUniqueStickyVisitors(Unknown Source)
        at org.hd.d.tmf.ThroughputMonitorFilterBase.poll(Unknown Source)
        at org.hd.d.tmf.ThroughputMonitorFilterBase.access$000(Unknown Source)
        at org.hd.d.tmf.ThroughputMonitorFilterBase$1.run(Unknown Source)
        at java.util.TimerThread.mainLoop(Timer.java:512)
        at java.util.TimerThread.run(Timer.java:462)

   FIXME:

   IPv6 support.

 */

import ORG.hd.d.IsDebug;

import org.hd.d.pg2k.svrCore.*;
import org.hd.d.pg2k.svrCore.MIME.ExhibitMIME;
import org.hd.d.pg2k.svrCore.location.GeoProximity;
import org.hd.d.pg2k.svrCore.location.GeoUtils;
import org.hd.d.pg2k.svrCore.location.LoadBalancingUtils;
import org.hd.d.pg2k.svrCore.location.GeoUtils.CCTLD;
import org.hd.d.pg2k.svrCore.props.GenProps;
import org.hd.d.pg2k.svrCore.props.LocalProps;
import org.hd.d.pg2k.svrCore.stats.StatsLogger;
import org.hd.d.pg2k.svrCore.vars.SimpleVariableDefinition;
import org.hd.d.pg2k.svrCore.vars.SimpleVariablePipelineIF;
import org.hd.d.pg2k.svrCore.vars.SimpleVariableValue;
import org.hd.d.pg2k.svrCore.vars.SystemVariables;
import org.hd.d.pg2k.webSvr.ads.AdUtils;
import org.hd.d.pg2k.webSvr.exhibit.DataSourceBean;
import org.hd.d.pg2k.webSvr.virtualHosts.VirtualHosts;
import org.hd.d.tmf.ThroughputMonitorFilterBase;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.management.ThreadInfo;
import java.lang.management.ThreadMXBean;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

import static java.lang.Thread.getAllStackTraces;

/**Filter to monitor throughput (output rate) to help regulate flow.
 * Can be used in association with the compression filter to
 * enable compression when our output pipe is likely to get congested.
 * <p>
 * (This filter has had a few miscellaneous bits of functionality thrown in for now:
 * <ul>
 * <li>This captures changes to the standard set of session attributes.
 * <li>This redirects users to a canonical hostname
 *     that have arrived at a non-standard or deprecated one.
 * </ul>)
 * <p>
 * This also makes a rough estimate of the number of users
 * (without forcing a session) by counting the number of IP addresses
 * used to visit the site through <em>this servlet container instance</em>
 * (probably a slight overestimate).  This ignores clients that are
 * probably spiders (ie that never send a referring URL),
 * and that appear to be one-off hotlinks from external sites.
 * <p>
 * A truely distributed front end will need a more sophisticated communication
 * mechanism to share data efficiently between front-end instances.
 * <p>
 * TODO: This filter is not yet suitable for multi-JVM front-ends (clustering)
 *     or otherwise where the
 *     WAR container creates multiple instances of this filter
 *     for what is logically one front-end.
 */
public final class ThroughputMonitorFilterPG2K extends ThroughputMonitorFilterBase
    {
    /**Request attribute (property name) in which we save the incoming requestURI.
     * This is useful when we do a redirect internally.
     */
    public static final String PNAME_SAVED_REQ_URI = "tmf.savedReqURI";

    /**If true, then we may try to force a pre-emptive GC when the system is idle. */
    private static final boolean DO_PREEMPTIVE_GC = true;

    /**Conversion factor from byte/second to GB/30day. */
    private static final float BPS_TO_GBP30D = (3600 * 24 * 30) / 1.0e9f;

    /**Any GB-per-30-days traffic limit; strictly positive if set. */
    private final int maxGBytesPer30D = Integer.getInteger("org.hd.d.pg2k.maxGBytesPer30D", 0);

    // Compute derived values for speed.
    private final int maxBpsFor30D = Math.max(0, Math.round(maxGBytesPer30D / BPS_TO_GBP30D));
    /**Unique prefix for exhibits. */
    private static final String EX_PREFIX = '/' + WebConsts.BASE_PATH_EXHIBITS + '/';
//    private static final String TN_PREFIX = '/' + WebConsts.BASE_PATH_TN + '/';
//    private static final String STATIC_PREFIX = '/' + WebConsts.BASE_PATH_STATIC + '/';
    /**Unique prefix for catalogue pages. */
    private static final String CATPAGE_PREFIX = '/' + WebConsts.BASE_PATH_CATPAGE + '/';
    /**Common (possibly ambiguous) prefix for catalogue page and search page. */
    private static final String CATAREA_PREFIX = "/_c";

    /**System event to which profiling/busy events are recorded. */
    private static final SimpleVariableDefinition PROFILING_EVENT = SystemVariables.PERFMON_STRING_GLOBAL_EVENT;


//    /**Time that this instance was constructed. */
//    private final long consTime = System.currentTimeMillis();


    /**Called when the system is idle according to the throughput monitor.
     * We can use this to request a pre-emptive GC.
     */
    protected void onIdle()
        {
        if(DO_PREEMPTIVE_GC)
            {
            final ServletContext ctxt = getConfig().getServletContext();
            final DataSourceBean dsb = DataSourceBean.getApplicationInstance(ctxt);
            if(MemoryTools.preemptiveGC())
                {
                try
                    {
                    dsb.setVariable(new SimpleVariableValue(
                        SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                        "PreemptiveGC=" + LocalProps.getMirrorTag()));
                    }
                catch(final Exception e)
                    {
                    e.printStackTrace(); // Absorb error, but whinge...
                    }
                }
            }
        }

    /**Unhook from servlet logger. */
    public void destroy()
        {
        logger.setContext(null); // Stop using servlet logger.
        super.destroy();
        }

    /**Hook into servlet logger. */
    public void init(final FilterConfig filterConfig)
        {
        logger.setContext(filterConfig.getServletContext()); // Use servlet.log().
        super.init(filterConfig);

        if(maxGBytesPer30D > 0)
            {
            final String message = "[Using per-30-day bandwidth limit of "+maxGBytesPer30D+"GB ("+(maxGBytesPer30D / BPS_TO_GBP30D)+"Bps).]";
            System.out.println(message);
            logger.log(message);
            }
        }


    /**Our logger which falls back to System.out if servlet log not available; never null. */
    private final WebUtils.ServletLoggerWithFallback logger = new WebUtils.ServletLoggerWithFallback();

    /**The stats set to which we log general throughput events.
     * The unique codes are the constants THRFGNAME_XXX.
     */
    private final StatsLogger.StatsConfig statsIDTHRFGEN =
        new StatsLogger.StatsConfig("THROUGHPUT-GENERAL",
                                    logger, // Use servlet log if poss.
                                    false, // Only dump summaries...
                                    12 * 3600, // About every 12 hours.
                                    true); // Adaptive.

    /**General stats event name prefix: client region by IP address. */
    public static final String THRFNAMEPR_CLIENT_REGION = "client-region-";

    /**General stats event name prefix: "sticky" client region by IP address. */
    public static final String THRFNAMEPR_STICKY_CLIENT_REGION = "sticky-client-region-";

    /**General stats event name prefix: client estimated proximity from IP address. */
    public static final String THRFNAMEPR_CLIENT_PROXIMITY = "client-proximity-";

    /**General stats event name prefix: client SPAMmer/compromised detected by given DNSRBL. */
    public static final String THRFNAMEPR_DNSRBL_DETECTION = "client-in-DNSRBL-";

    /**General stats event name prefix: coda processing time log(ms). */
    public static final String THRFNAMEPR_CODATIME = "coda-time-";

    /**General stats event name prefix: virtual host name used to fetch page. */
    public static final String THRFNAMEPR_VHOST = "vhost-";

    /**General stats event name: redirect issued to client because of unknown name. */
    public static final String THRFNAME_REDIR_UNKNOWN = "redirect-unknown";

    /**General stats event name: redirect issued to client to get them to local mirror. */
    public static final String THRFNAME_REDIR_TO_LOCAL = "redirect-to-local";

    /**General stats event name: redirect issued to client because of deprecated alias. */
    public static final String THRFNAME_REDIR_ALIAS = "redirect-alias";

    /**General stats event name: 1000 hits (GETs, POSTs, redirects, etc). */
    public static final String THRFNAME_kHIT = "kHit";

    /**General stats event name: 1000 catalogue page hits (GETs). */
    public static final String THRFNAME_kHIT_CAT_PAGE = "kHit-page-GET-catalogue";

    /**General stats event name: 1000 "sticky client" page hits (GETs). */
    public static final String THRFNAME_kHIT_STICKY_CLIENT_PAGE = "kHit-page-GET-byStickyClient";


    /**If false, be prepared to consume extra resources working out geographical location of client.
     * Else (if true), just do quick lookup with internal/cached data.
     */
    private static final boolean DO_QUICK_GEO_LOOKUP = true;


    /**If true, report any unexpected host name used to access the site. */
    private static final boolean REPORT_UNEXPECTED_HOSTNAME = true;

    /**If true, redirect users arriving with a deprecated host name.
     * FIXME: possibly this functionality might be better moved elsewhere.
     */
    private static final boolean REDIRECT_DEPRECATED_HOSTNAME = true;


    /**Returns expected approximate bytes-per-second per sticky client; strictly positive.
     * Picks up our hard-wired value.
     */
    protected int expectedBpsPerStickyClient()
        {
        return(WebConsts.DEFAULT_EXPECTED_USER_BW_BYTESPERSEC);
        }

    /**Time when we will next report the slow-changing parameters, initially 0 to force immediate send.
     * Marked volatile for safe lock-free access,
     * though in fact this is probably unnecessary.
     */
    private volatile long nextSendAllParams;

    /**Called on every tick, and passed the most important metrics.
     *
     * @param availableBpsPerClientBasic  estimated available bytes/sec
     *     available for one new client (or an existing client); non-negative
     * @param allVisitorCount  approximate count of "live" visitors;
     *     non-negative
     * @param stickyVisitorCount  approximate count of "live" "sticky" visitors;
     *     non-negative
     * @param ltbpss  long-term bytes/sec smoothed throughput
     * @param dailyUniqueVisitors  estimated unique vistors per day;
     *     non-negative
     * @param busyFractionSmoothed  smoothed system "busy-ness" value
     *     ranging from 0.0 (idle) to 1.0 (busy)
     * @param tooManyVisitors  true if well over capacity by user count
     * @param connectionTooBusy  true if well over capacity by bandwidth
     * @param systemTooBusy  true if system busy fraction dangerously high
     */
    protected void tick(final int availableBpsPerClientBasic,
                        final int allVisitorCount,
                        final int stickyVisitorCount,
                        final float ltbpss,
                        final int dailyUniqueVisitors,
                        final float busyFractionSmoothed,
                        final boolean tooManyVisitors,
                        final boolean connectionTooBusy,
                        final boolean systemTooBusy)
        {
        final ServletContext ctxt = getConfig().getServletContext();
        final DataSourceBean dsb = DataSourceBean.getApplicationInstance(ctxt);

        // Set the light-load flag directly.
        // We aim to be slightly conservative in this.
        // In particular we want to see none of the danger flags set,
        // and the smoothed "busy" value well under 50%.
        // The overload flag is set elsewhere.
        final boolean isLightlyLoaded = (!systemTooBusy) &&
                                        (!connectionTooBusy) &&
                                        (!tooManyVisitors) &&
                                        (availableBpsPerClientBasic > 0) &&
                                        (busyFractionSmoothed < 0.3f);
        ctxt.setAttribute(WebConsts.BANDWIDTH_LIGHTLOAD_ATTR_NAME, isLightlyLoaded ? Boolean.TRUE : Boolean.FALSE);

        // Compute actual Bps since server start;
        // we use the long-term smoothed value if this is not available.
        final Object lifetimeBpsOutO = getValue(ATTR_NAME_SUFFIX_SERVER_LIFETIME_BYTES_PER_SEC_OUTBOUND);
        final long lifetimeBpsOut = (!(lifetimeBpsOutO instanceof Number)) ? Math.round(ltbpss) :
            (((Number)lifetimeBpsOutO).longValue());

        // If this host is a master
        // then report only a fraction of actual available bandwidth
        // as we expect a heavy load dealing with slaves, etc.
        // Note that we apply a similar/further "slowdown" factor for
        // slaves with expensive bandwidth approaching their limits, etc,
        // and this can be cumulative if required.
        final boolean isMaster = !Boolean.TRUE.equals(dsb.isSlave());
        final int serverSlowdownFactor = LocalProps.getServerSlowdownFactor();
        assert(serverSlowdownFactor > 0);
        // If lifetime bandwidth is close to (or above) any 30D limit
        // then strictly limit what we report as available to what is left.
        // Note that this threshold probably needs to be significantly below
        // that imposed by any upstream traffic shaping
        // if both are to be effective and behave as expected.
        final boolean approachingPCMLimit = ((maxGBytesPer30D > 0) && (lifetimeBpsOut*1.2 > maxBpsFor30D));
        final int availableBpsPerClient;
        if(!isMaster && (serverSlowdownFactor < 2) && !approachingPCMLimit)
            {
             // Usual case: full-speed slave nowhere near PCM limits.
            availableBpsPerClient = availableBpsPerClientBasic;
            }
        else
            {
            // Master or slave with high-cost or scarce bandwidth...
            availableBpsPerClient = availableBpsPerClientBasic /
                (isMaster ? (4 * serverSlowdownFactor) : serverSlowdownFactor);
            }

        // If there is a per-30-day data-transfer limit for this server,
        // and if we are anywhere near it,
        // then report a value capped by it;
        // scale the per-30-day limit down to reflect long-term load.
        // Conservative value so as to avoid attracting unsupportable traffic.
        int reportedAvailableBpsPerClient = !approachingPCMLimit ?
                availableBpsPerClient /* No "monthly" transfer limit need be enforced. */ :
                (Math.min(availableBpsPerClient,
                      Math.max(0, (int) Math.round(maxBpsFor30D - lifetimeBpsOut*1.1))));

        // If it looks as if we don't have enough bandwidth per client
        // then clamp our reported available bandwidth to zero to deter new clients.
        if(reportedAvailableBpsPerClient < expectedBpsPerStickyClient()/2)
            { reportedAvailableBpsPerClient = 0; }

        // Post updates to system variables.
        // We need not necessarily set every one of these on each tick,
        // especially those that only meaningfully change slowly,
        // though the cost of doing so is probably not excessive.
        //
        // IOExceptions may veto setting later values,
        // so put more important or fast-changing ones first.
        try
            {
            // Predicted bandwidth available to a new client.
            // Used to help distributed load-balancing algorithm.
            // Conservative to avoid overload.
            dsb.setVariable(new SimpleVariableValue(
                SystemVariables.ThroughputMonitorFilter_AVAIL_BPS_PER_CLIENT,
                Integer.valueOf(reportedAvailableBpsPerClient)));
            // For now our "sticky" count is our normal user count.
            dsb.setVariable(new SimpleVariableValue(
                SystemVariables.ThroughputMonitorFilter_STICKY_CLIENT_COUNT,
                Integer.valueOf(stickyVisitorCount)));

            // Send slow-changing values less often.
            final long now = System.currentTimeMillis();
            if(now >= nextSendAllParams)
                {
                // All-users count, including spiders, hotlinkers, etc.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ThroughputMonitorFilter_CLIENT_COUNT,
                    Integer.valueOf(allVisitorCount)));
                // Long-term bandwidth.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ThroughputMonitorFilter_ltBps,
                    new Integer((int) ltbpss))); // Integer is compact and simple.
                // Smoothed busy-fraction value.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ThroughputMonitorFilter_BUSY_FRACTION,
                    new Float(busyFractionSmoothed))); // Float is compact and has enough precision.
                // Estimated yearly unique "sticky" human users.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ThroughputMonitorFilter_YEARLY_UNIQUE_VISITORS,
                    new Float(dailyUniqueVisitors * 365.242f))); // No more than ~6SF of precision sensible!
                // Note the version of the AEP that we are currently using locally.
                // This can be used to decide which mirrors are in sync.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ThroughputMonitorFilter_AEP_LONGHASH,
                    (new Long(dsb.getAllExhibitProperties(-1).longHash))));
                // Set our mirror tag if we have one (else null)...
                // Note that we may be a mirror and master simultaneously.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ThroughputMonitorFilter_ACTIVE_MIRROR_NAME,
                    LocalProps.getMirrorTag()));

                // Put off next full send a while,
                // but before the previous values actually expire...
                nextSendAllParams = now + SystemVariables.MAX_VALUE_DISTRIBUTION_LATENCY_MS/3;
                }
            }
        catch(final IllegalArgumentException e)
            { System.err.println("ThroughputMonitorFilter: IllegalArgumentException: failed to set system variable: " + e.getMessage()); }
        catch(final IOException e)
            { System.err.println("ThroughputMonitorFilter: IOException: failed to set system variable: " + e.getMessage()); }


        // Attempt slow/continuous system CPU profile if so requested.
        try
            {
            final GenProps gp = dsb.getGenProps(-1);
            if(slowProfileEnabled(gp))
                { _logThreads(-1, true); }
            }
        catch(final Exception e)
            {
            e.printStackTrace();
            }

        // Attempt to incrementally purge expired entries from the inRBL cache.
        // Try freeing whatever we get first with an iterator
        // (hopefully the LRU item, though not really clear from JDK1.5 docs),
        // and keep going until we hit a non-expired item
        // so as to be able to clear down garbage from a load spike effectively.
        while(inRBL.size() > 0)
            {
            // Grab the lock for a as short a time as possible each time,
            // and thus start iterating all over again each time,
            // but only deal with the first item on each loop.
            // This allows other activity to be interleaved with clearup.
            synchronized(inRBL)
                {
                final Iterator<Map.Entry<InetAddress,Tuple.Pair<Long, String>>> it = inRBL.entrySet().iterator();
                if(it.hasNext())
                    {
                    final Map.Entry<InetAddress,Tuple.Pair<Long, String>> entry = it.next();
                    if((entry.getValue() == null) || (entry.getValue().first == null) || (entry.getValue().first.longValue() < System.currentTimeMillis()))
                        {
if(IsDebug.isDebug) { logger.log("[TMF: inRBL.size()="+inRBL.size()+"; removing item: "+entry.getKey()+"->"+entry.getValue()+"...]"); }
                        it.remove();
                        continue;
                        }

                    break; /* Give up when we hit a live entry. */
                    }
                }
            }
        }

    /**Key in generic props of slow-profile flag. */
    private static final String KEY_debugFlag = CoreConsts.GEN_PREFIX_debugFlag + "slowProfile";

    /**If true then slow profiling is enabled. */
    private static boolean slowProfileEnabled(final GenProps gp)
        {
        return(gp.getGen().get(KEY_debugFlag) != null);
        }

    /**Don't dump busy threads again before this time; initially zero so first dump is immediately as required.
     * Volatile for thread-safety and atomicity of access.
     */
    private long dontDumpThreadsBefore;

    /**Make a note of the thread state in a (busy) system.
     * Log the "busy" methods to the distributed logging store,
     * and extra detail to the System.err.
     *
     * @param connCount  current active-connection count; non-negative
     */
    protected void logBusyThreads(final int connCount)
        {
        final long dumpStart = System.currentTimeMillis();

        // Don't print if too soon (but do always record events).
        final boolean dontPrint = (System.currentTimeMillis() < dontDumpThreadsBefore);

        try
            {
            final ServletContext servletContext = getConfig().getServletContext();
            final ServletContext ctxt = servletContext;
            final DataSourceBean dsb = DataSourceBean.getApplicationInstance(ctxt);
            final SimpleVariablePipelineIF vars = dsb;
            final GenProps gp = dsb.getGenProps(-1);

            // Note that this host system is busy.
            vars.setVariable(new SimpleVariableValue(PROFILING_EVENT,
                                                    "SysBusy=" + LocalProps.getMirrorTag()));

            if(!dontPrint)
                { servletContext.log("WARNING: SYSTEM TOO BUSY: ACTIVE CONNECTION COUNT: " + connCount); }

            // Don't log threads again if slow profiling ss already enabled.
            if(!slowProfileEnabled(gp))
                { _logThreads(connCount, dontPrint); }
            }
        catch(final Exception e)
            {
            e.printStackTrace(); // Absorb error, but whinge...
            }
        finally
            {
            // If we just printed a full dump
            // then postpone the next one a while.
            if(!dontPrint)
                {
                final long dumpEnd = System.currentTimeMillis();
                final long dumpTime = dumpEnd - dumpStart;
                // Limit time spent dumping to no more than about 1%,
                // but dump at least once per minute in any case.
                dontDumpThreadsBefore = System.currentTimeMillis() +
                    10000 + Math.min(50000, 100*dumpTime);
                }
            }
        }

    /**Log running threads to the event record and optionally to the console.
     *
     * @param connCount  number of open inbound connections if non-negative
     * @param dontPrint  if false then log to the system console too
     */
    private void _logThreads(final int connCount,
                             final boolean dontPrint)
        throws IOException
        {
        final ServletContext servletContext = getConfig().getServletContext();
        final ServletContext ctxt = servletContext;
        final DataSourceBean dsb = DataSourceBean.getApplicationInstance(ctxt);
        final SimpleVariablePipelineIF vars = dsb;

        // Now note the state of the threads.
        final Map<Thread, StackTraceElement[]> threads = getAllStackTraces();
        if(!dontPrint && (connCount >= 0)) { servletContext.log("System activity: ACTIVE THREAD COUNT: " + threads.size()); }
        for(final Thread t : threads.keySet())
            {
            final Thread.State state = t.getState();
            if(!dontPrint) { servletContext.log("System activity: ACTIVE THREAD: " + t.getName() + ": " + state); }
            // Omit this current thread's detail as uninteresting...
            if(t == Thread.currentThread())
                { continue; }
            // Omit detail for threads in probably-uninteresting
            // (non-CPU/IO-using) states...
            if((state == Thread.State.NEW) ||
               (state == Thread.State.TIMED_WAITING) ||
               (state == Thread.State.WAITING) || /* WAITING might be indefinitely blocked or deadlocked. */
               (state == Thread.State.TERMINATED))
                { continue; }
            // Log stack entries for PG2K code only.
            boolean loggedEvent = false;
            for(final StackTraceElement ste : threads.get(t))
                {
                if(ste == null)
                    { continue; }
                if(loggedEvent && dontPrint)
                    { break; /* Nothing more to do... */ }
                final String cn = ste.getClassName();
                final boolean ourCode = cn.startsWith("org.hd.d.pg2k.");
                // Don't show 3rd-party methods above the first instance
                // of our code in the stack trace.
                if(!ourCode && loggedEvent)
                    { continue; }
                final String mn = ste.getMethodName();
                final int ln = ste.getLineNumber();
                final String methodDetail = ("Busy=" + cn + "." + mn + ((ln >= 0) ? ("(line:" + ln + ")") : "()"));
                if(!dontPrint) { servletContext.log("System activity: " + methodDetail); }
                if(ourCode && !loggedEvent)
                    {
                    // Note the state of the Thread running our method.
                    vars.setVariable(new SimpleVariableValue(PROFILING_EVENT,
                                                             "BusyThreadState=" + state));

                    // Log the event...
                    // Do so after the thread state
                    // as this is a slightly more interesting "last value"
                    // to see in the stats.
                    vars.setVariable(new SimpleVariableValue(PROFILING_EVENT,
                                                             methodDetail));

                    // Print any lock on which we are blocked, and by whom...
                    if(!dontPrint)
                        {
                        final long tid = t.getId();
                        final ThreadMXBean tMXB =
                            java.lang.management.ManagementFactory.getThreadMXBean();
                        final ThreadInfo tInfo = tMXB.getThreadInfo(tid);
                        if(tInfo != null)
                            {
                            final String lockName = tInfo.getLockName();
                            if(lockName != null)
                                { servletContext.log("System activity: MONITOR: " + lockName); }
                            final String lockerName = tInfo.getLockOwnerName();
                            if(lockerName != null)
                                { servletContext.log("System activity: BLOCKER THREAD: " + lockerName); }
                            }
                        }

                    loggedEvent = true;
                    }
                }
            }
        }

    /**Allow start-up fast execution sampling for performance tuning. */
    private static final boolean STARTUP_EXECUTION_SAMPLING_ENABLED = true;

    /**Start-up sampling time (ms); strictly positive. */
    private static final int STARTUP_EXECUTION_SAMPLING_MS = 180000;

    /**Maximum number of (top) sampled sites to dump; strictly positive. */
    private static final int STARTUP_EXECUTION_SAMPLING_MAX_DUMP = 100;

    /**Start sampling ASAP, if enabled and requested at run-time. */
    static
        {
        if(STARTUP_EXECUTION_SAMPLING_ENABLED &&
           Boolean.getBoolean("org.hd.d.pg2k.profileStartup"))
            {
            final long sampleStartTime = System.currentTimeMillis();
            final long stopBy = sampleStartTime + STARTUP_EXECUTION_SAMPLING_MS;
            final Thread t = new Thread("startup sampling thread"){
                @Override public final void run()
                    {
                    final ConcurrentHashMap<StackTraceElement,AtomicInteger> counts =
                        new ConcurrentHashMap<StackTraceElement, AtomicInteger>(10001);
                    System.out.println("[Sampling started...]");
                    do
                        {
                        // Sample loop.
                        _perfSampleThreadsQuick(counts);
                        // Approx 100Hz sample rate should be OK.
                        try { Thread.sleep(10); }
                        catch(final InterruptedException e)
                            {
                            e.printStackTrace();
                            break; // Stop sampling...
                            }
                        } while(System.currentTimeMillis() < stopBy);
                    
                    GenUtils.dumpPerfSamples("START-UP ACTIVITY SAMPLES (10ms)", counts, STARTUP_EXECUTION_SAMPLING_MAX_DUMP, GenUtils.systemOutLogger);
                    }
                };
            t.setDaemon(true);
            t.setPriority(Thread.MAX_PRIORITY); // Highest-possible priority.
            t.start();
            }
        }

    /**Quickly sample running threads for performance logging, eg at start-up.
     * This is likely to be called very frequently,
     * so should be very fast and efficient.
     * <p>
     * This logs the top-most (current) activation record for every live thread
     * (non-NEW, non-TERMINATED) in the system as a count in the table.
     * <p>
     * The calling thread is omitted.
     * <p>
     * It is safe for other threads to read the counts map while it is
     * being updated in here: the map is always kept in a consistent state.
     *
     * @param counts  caller-supplied map to be updated with counts of samples
     *     at each execution site; never null
     */
    private static void _perfSampleThreadsQuick(final ConcurrentHashMap<StackTraceElement,AtomicInteger> counts)
        {
        // Now note the state of the threads.
        final Map<Thread, StackTraceElement[]> threads = getAllStackTraces();
        for(final Thread t : threads.keySet())
            {
            final Thread.State state = t.getState();
            // Omit this current thread's sample as uninteresting...
            if(t == Thread.currentThread())
                { continue; }
            // Omit sample for threads in probably-uninteresting states...
            if((state == Thread.State.NEW) ||
               (state == Thread.State.TERMINATED))
                { continue; }
            // Get stack trace.
            final StackTraceElement[] stackTraceElements = threads.get(t);
            // Skip threads with an empty or strange stack.
            final StackTraceElement top;
            if((stackTraceElements == null) ||
               (stackTraceElements.length < 1) ||
               (null == (top = stackTraceElements[0])))
                { continue; }

            // Get existing counter for this execution site,
            // or (atomically) create a new zero one.
            AtomicInteger count;
            while(null == (count = counts.get(top)))
                { counts.putIfAbsent(top, new AtomicInteger()); }

            // Increment counter.
            count.incrementAndGet();
            }
        }

//    /**Private session attribute name which we use to avoid recording multiple new hits/conversions for one client.
//     * We DO NOT create a session to store this,
//     * but if a session is available then we check with this.
//     */
//    private static final String SESSION_STICKY_LOGGED_ATTRNAME = "org.hd.d.pg2k.webSvr.util.ThroughputMonitorFilterPG2K.ALREADY_NOTED_AS_STICKY";

    /**Returns true if we deem this a hit to be from a "sticky" client.
     * "Sticky" means:
     * <ul>
     * <li>A GET request.
     * <li>Not a spider/bot or a hot-link or a prefetch
     *     (ie a human seems to have clicked from another of our pages).
     * <li>Not a SPAMmer or compromised machine.
     * <li>A request for a text page (not an exhibit/graphic) not top-level,
     *     eg a catalogue page (as the real meat of the site).
     * </ul>
     * <p>
     * This means that we should capture hits by a user on their
     * second and subsequent page views.
     *
     * @param isGET  true if the method is GET
     * @param requestH  the HTTP request; never null
     * @param clientIP  the client's IP address; never null
     *
     * @return true  iff this hit is to be treated as from a "sticky" client
     */
    @Override
    protected boolean isStickyClient(final boolean isGET,
                                     final HttpServletRequest requestH,
                                     final InetAddress clientIP)
        {
        assert(requestH != null);
        assert(clientIP != null);

        // Reject non-catalogue-area hits
        // (disallow top-level pages, allow cat/search pages).
        // We do this check first because it should be relatively fast.
        final String reqURI = requestH.getRequestURI();
        if((reqURI == null) ||
//           (reqURI.lastIndexOf('/') <= 0) ||
           !reqURI.startsWith(CATAREA_PREFIX))
            { return(false); }

        // Reject non-human visitors or new visitors or precache requests...
        if(!isGET ||
           WebUtils.requestProbablyFromSpider(requestH) ||
           WebUtils.isPrecacheRequest(requestH) ||
           (null != WebUtils.requestProbablyReferredFromExternalSite(requestH)))
            { return(false); }

        // Reject visitors from compromised machines
        // and those from bad neighbourhoods listed in DNS BLs.
        // This may be slow, so we do it as the last check.
        if(null != AddrTools.isAddrBlackholed(getDNSBLs(), clientIP, false))
            { return(false); }

        // Note the "sticky client" page hit.
        stickyClientPageHit(clientIP);
        return(true); // Yes, is sticky.
        }

    /**Counter of "sticky" hits on site pages (rather than graphics).
     * These are batched up and submitted to stats in bundles of 1000 (kHits).
     * This value should remain between 0 and around 1000.
     * <p>
     * We use an AtomicInteger to avoid having to take a lock.
     */
    private final AtomicInteger stickyClientHitCounter = new AtomicInteger();

    /**Note a page (eg HTML) hit from a sticky client.
     * @param clientIP  client's IP address, never null.
     */
    protected void stickyClientPageHit(final InetAddress clientIP)
        {
        // Record another 1000 hits...
        if(stickyClientHitCounter.incrementAndGet() >= 1000)
            {
            stickyClientHitCounter.addAndGet(-1000);
            StatsLogger.captureDataPoint(statsIDTHRFGEN,
                                         THRFNAME_kHIT_STICKY_CLIENT_PAGE);
            final DataSourceBean dsb = DataSourceBean.getApplicationInstance(getConfig().getServletContext());
            try
                {
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                        THRFNAME_kHIT_STICKY_CLIENT_PAGE));
                }
            catch(final IOException e)
                {
                e.printStackTrace(); // Absorb and report errors.
                }
            }

        // Record the hit from a "sticky" client by location.
        // Use a quick lookup to avoid delaying the user unduly.
//                    dsb.setVariable(new SimpleVariableValue(
//                        SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
//                        MemoryTools.intern("StickyHitFrom=" + GeoUtils.getRegionByAddress(clientIP, true))));
        final String regionByAddress = GeoUtils.getRegionByAddress(clientIP, true);
        StatsLogger.captureDataPoint(statsIDTHRFGEN,
                                     THRFNAMEPR_STICKY_CLIENT_REGION + regionByAddress);
        }

    /**Counter of all (successful) hits.
     * These are batched up and submitted to stats in bundles of 1000 (kHits).
     * This value should remain between 0 and around 1000.
     * <p>
     * We use an AtomicInteger to avoid having to take a lock.
     */
    private final AtomicInteger hitCounter = new AtomicInteger();

    /**Counter of all catalogue-page GET hits.
     * These are batched up and submitted to stats in bundles of 1000 (kHits).
     * This value should remain between 0 and around 1000.
     * <p>
     * We use an AtomicInteger to avoid having to take a lock.
     */
    private final AtomicInteger catPageHitCounter = new AtomicInteger();

    /**If true, check clients in DNS BL <em>before</em> each access if status is not known.
     * This can severely impact performance as seen by the user
     * if used for (for example) all HTML pages
     * so a "lazy" approach is generally preferred,
     * but if before downloads of expensive items (eg exhibits) then an eager approach may be better.
     */
    private final static boolean CHECK_DNSRBL_EARLY = true;

    /**Name of attribute that we use to avoid duplicate application of this filter on one filter chain. */
    private static final String ANTI_DUP_ATTR_NAME = "org.hd.d.pg2k.webSvr.util.ThroughputMonitorFilterPG2K.DUPFLAG";

    /**Called to handle each (HTTP) request.
     * Hooks into the requests to measure throughput and other stats.
     * <p>
     * We may also redirect a client if a non-preferred (or unrecognised)
     * hostname is used to contact the server.
     * <p>
     * We call super.doFilter() to get all the basic throughput measurement
     * tasks done.
     */
    public void doFilter(final ServletRequest request,
                         final ServletResponse response,
                         final FilterChain chain)
        throws IOException, ServletException
        {
        // Avoid duplicate application of this filter,
        // eg during servlet redirection.
        if(null != request.getAttribute(ANTI_DUP_ATTR_NAME))
            {
            // Avoid counting the same traffic more than once.
if(IsDebug.isDebug) { System.err.println("WARNING: avoiding duplicate TMF application to " + request); }
            chain.doFilter(request, response);
            return;
            }
        request.setAttribute(ANTI_DUP_ATTR_NAME, Boolean.TRUE);

        if(!(response instanceof HttpServletResponse) ||
           !(request instanceof HttpServletRequest))
            {
            // Cannot handle non-HTTP traffic, so don't even wrap it.
            chain.doFilter(request, response);
            return;
            }

        final HttpServletRequest httpServletRequest = (HttpServletRequest) request;
        final HttpServletResponse httpServletResponse = (HttpServletResponse) response;        

        // Record another hit...
        if(hitCounter.incrementAndGet() >= 1000)
            {
            hitCounter.addAndGet(-1000);
            StatsLogger.captureDataPoint(statsIDTHRFGEN, THRFNAME_kHIT);
            }

        // We can monitor this if it is an HTTP request
        // and, possibly, only if it is a GET request.
        if(!ThroughputMonitorFilterPG2K.MONITOR_GET_TRAFFIC_ONLY ||
           "GET".equalsIgnoreCase(httpServletRequest.getMethod()))
            {   
            final InetAddress clientIP =
                InetAddress.getByName(request.getRemoteAddr());

            // Save the original incoming request URI
            // for components that need to see it.
            final String reqURI = httpServletRequest.getRequestURI();
            httpServletRequest.setAttribute(PNAME_SAVED_REQ_URI, reqURI);

            // We may block accesses from "bad" clients
            // (SPAMmers' spiders or compromised machines)
            // to all but top-level (including error) pages.
            // More than one '/' in the URI means a non-top-level page.
            // We may allow some content through anyway for efficiency
            // and/or allow the top-level page(s) to show properly.
            final Tuple.Pair<Long, String> statusInRBL = inRBL.get(clientIP);
            final FilterConfig config = getConfig();
            assert(config != null);

//            final boolean isNotTopLevel = (reqURI != null) &&
//                                       (reqURI.indexOf('/') != reqURI.lastIndexOf('/'));
//            final boolean checkBL = isNotTopLevel && /* Allow any top-level content. */
//                           !reqURI.startsWith(TN_PREFIX) && /* Allow thumbnails. */
//                           !reqURI.startsWith(STATIC_PREFIX); /* Allow static content. */

            // Check client's bona fides before attempts to download exhibits.
            final boolean checkBL = (reqURI != null) && reqURI.startsWith(EX_PREFIX);
            if(checkBL)
                {
                // Does this look like a "bad" (SPAMmer/compromised) client?
                // We decide so if we have any entry in cache of this client address
                // or a positive result from a quick BL check.
                // We ignore the cache-entry expiry time for this purpose,
                // relying on coda processing to expunge or recheck old records.
                // This may only happen when a non-blocked resource is accessed
                // AFTER the BL entry for this address has been dropped.
                String BL = null;
                if(statusInRBL != null)
                    { BL = statusInRBL.second; }
                else if(CHECK_DNSRBL_EARLY)
                    {
                    // If no cache entry then do a QUICK check here
                    // and cache any untoward result found.
                    BL = AddrTools.isAddrBlackholed(getDNSBLs(), clientIP, true);
                    if(BL != null)
                        {
                        inRBL.put(clientIP, new Tuple.Pair<Long, String>(
                            new Long(System.currentTimeMillis() + DNSBL_RESULT_CACHE_MS), BL));
                        }
                    }

                // Looks like a bad 'un!
                if(BL != null)
                    {
                    StatsLogger.captureDataPoint(statsIDTHRFGEN,
                        ThroughputMonitorFilterPG2K.THRFNAMEPR_DNSRBL_DETECTION + BL);
                    // This is interesting enough to log centrally...
                    final DataSourceBean dsb = DataSourceBean.getApplicationInstance(config.getServletContext());
                    dsb.setVariable(new SimpleVariableValue(
                        SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                        ThroughputMonitorFilterPG2K.THRFNAMEPR_DNSRBL_DETECTION + BL));
                    // Note a few more details of this request...
                    logger.log("WARNING: client "+AddrTools.doReverseLookup(clientIP, true)+'/'+clientIP+" is in DNSRBL "+BL+" and requested "+TextUtils.sanitiseForXML(httpServletRequest.getRequestURL().toString(), 512, true));
                    // TODO: maybe redirect to a "your machine has been pwnd" page.
                    httpServletResponse.sendError(HttpServletResponse.SC_FORBIDDEN,
                        "WARNING: get help from your ISP or administrator: your machine may be INFECTED with a VIRUS or TROJAN, or your IP address may have been used to send SPAM: your IP address is listed at DNS BL: " + BL);
                    return;
                    }
                }


            // Note the hit by (normalised) host name.
            // Optionally also redirect away from deprecated host names...
            // Possibly allow bounce to "local" mirror.
            final String serverName = request.getServerName();
            // Our preferred form of the hostname is all-lower-case.
            final String serverNameLC = serverName.toLowerCase();
            final String normalisedServerName =
                HostUtils.normaliseVirtualHostName(serverNameLC);
            final VirtualHosts.VirtualHost vHost =
                VirtualHosts.getVirtualHostDetails(serverNameLC, null);
            StatsLogger.captureDataPoint(statsIDTHRFGEN,
                ThroughputMonitorFilterPG2K.THRFNAMEPR_VHOST +
                    ((vHost == null) ? "OTHER" : normalisedServerName));
            // Report unexpected/unrecognised host name used to access the site.
            if(vHost == null)
                {
                if(ThroughputMonitorFilterPG2K.REPORT_UNEXPECTED_HOSTNAME)
                    { logger.log("Unexpected hostname used to access this site: "+serverName+", clientIP="+clientIP); }
                // Regard these requests as a bit suspect, and thus low-priority.
                request.setAttribute("LowPriConn", Boolean.TRUE);
                }

            final String mirrorTag = LocalProps.getMirrorTag();

            // Optionally redirect user from deprecated/non-canonical hostname.
            // Usually we redirect to the canonical name,
            // but if we are a mirror and the inbound if for a mirror in the same country,
            // then redirect to our mirror URL,
            // all so as to try to keep mirror juggling from hurting SEs and end users.
            // Never redirect a robots.txt request so we can handle it elsewhere.
            if(ThroughputMonitorFilterPG2K.REDIRECT_DEPRECATED_HOSTNAME &&
            		!WebConsts.ROBOTS_PAGE.equals(reqURI))
                {
                // If we don't recognise the host name at all,
                // then PERMANENTLY redirect to our main/default name,
                // preserving the URI (but not any non-standard port).
                if(vHost == null)
                    {
                    StatsLogger.captureDataPoint(statsIDTHRFGEN,
                                                 ThroughputMonitorFilterPG2K.THRFNAME_REDIR_UNKNOWN);
                    final String canonURI = "http://" +
                                            VirtualHosts.VIRTUAL_HOST_DEFAULT.normalisedName +
                                            ((reqURI == null) ? "/" : reqURI);
                    // Permanent (301, SE-friendly) redirect.
                    (httpServletResponse).setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                    (httpServletResponse).setHeader("Location", canonURI);
                    return;
                    }
                // As an optimisation when fielding a request for an old/deprecated mirror name,
                // if this host is mirror,
                // and a (valid) mirror name is being used that isn't ours,
                // and the request mirror name is for the same country as our mirror name,
                // then PERMANENTLY redirect to our own mirror name, preserving the URI.
                // We expect to end up servicing this redirected request sooner or later.
                else if((null != mirrorTag) &&
                		CoreConsts.MAIN_DATA_HOST.equals(vHost.normalisedName) &&
                		HostUtils.isMirrorName(serverNameLC) &&
                		mirrorTag.substring(0, 2).equals(HostUtils.getMirrorCC(serverNameLC).code) &&
                		!serverNameLC.equals(HostUtils.makeMirrorNameGeneric(mirrorTag)))
	                {
                	final String mirrorName = HostUtils.makeMirrorNameGeneric(mirrorTag);
                    StatsLogger.captureDataPoint(statsIDTHRFGEN,
                            ThroughputMonitorFilterPG2K.THRFNAME_REDIR_ALIAS);
					final String requestURI = reqURI;
					final String canonURI = "http://" +
					                           mirrorName +
					                           ((requestURI == null) ? "/" : requestURI);
					// Permanent (301, SE-friendly) redirect.
					(httpServletResponse).setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
					(httpServletResponse).setHeader("Location", canonURI);
					logger.log("WARNING: redirected (perm:301) from inbound server name "+serverName+" to preferred same-CC mirror "+mirrorName+" for client="+clientIP);
					return;
	                }
                // If a deprecated alias is being used,
                // or the alias/name is not lower-case,
                // and the name is not the master (special) name,
                // (or, if a mirror name and this server is a mirror, but it is not OUR mirror name),
                // then PERMANENTLY redirect to the main/canonical alias,
                // preserving the URI (but not any non-standard port),
                // providing that the preferred name exists in DNS!
                // (This should also purge duplicate/dead aliases/mirrors in search engines
                // once we are redirecting them elsewhere.)
                else if(!serverName.equals(serverNameLC) ||
                        (!serverNameLC.equals(vHost.normalisedName) &&
                            !HostUtils.isMasterName(serverNameLC) &&
                            ((null != mirrorTag) && !serverNameLC.equals("mirror-" + mirrorTag + '.' + vHost.normalisedName))))
                    {
                    final String preferredAlias = vHost.normalisedName;
                    try
                        {
                        // Abort redirection if target hostname/alias
                        // does not appear to be in DNS.
                        // Not completely robust because of Java DNS cacheing,
                        // and potentially expensive,
                        // but may catch some cock-ups.
                        InetAddress.getByName(preferredAlias);

                        StatsLogger.captureDataPoint(statsIDTHRFGEN,
                                                     ThroughputMonitorFilterPG2K.THRFNAME_REDIR_ALIAS);
                        final String requestURI = reqURI;
                        final String canonURI = "http://" +
                                                    preferredAlias +
                                                    ((requestURI == null) ? "/" : requestURI);

//                        ((HttpServletResponse) response).sendRedirect(canonURI); // Temp redirect.

                        // Permanent (301, SE-friendly) redirect.
                        (httpServletResponse).setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                        (httpServletResponse).setHeader("Location", canonURI);
                        logger.log("WARNING: redirected (perm:301) from inbound server name "+serverName+" to preferred "+preferredAlias+" for client="+clientIP);
                        return;
                        }
                    catch(final UnknownHostException e)
                        {
                        // Whoops!  Our preferred alias is not even in DNS!
                        logger.log("ERROR: preferred server alias not in DNS, aborted redirect: " + preferredAlias);
                        }
                    }
                }

            // Capture changes to standard session attributes.
            // Should only do this on GET requests
            // to avoid interfering with things such as file uploads (POST).
            final boolean isGET = "GET".equalsIgnoreCase(httpServletRequest.getMethod());
            if(isGET)
                { SessionVarBean.updateSessionVarsFromRequest(httpServletRequest); }


            // Optionally redirect (bounce) the user to a "local" (to them) mirror
            // for performance and load-balancing,
            // but only extremely carefully and conservatively.
            // Only do this for GETs and with no extant session, for example.
            // Only do this on internal referrals on relatively stable items
            // (and never at the home/error pages)
            // so that the user gets to see at least one page!
            //
            // Only bounce if we know the user's country, and it is not this mirror's,
            // and EITHER:
            //   * There is a live mirror in the same country as this user, OR
            //   * The user has no proximity to us and the "optimal" mirror is closer to them.
            if(WebConsts.ALLOW_BOUNCE_TO_LOCAL_MIRROR &&
               isGET /* Do not redirect anything other than GETs. */ &&
               (null != mirrorTag) /* We must have a tag to tell if we are already "close" to the user. */ &&
               (serverName.equals(CoreConsts.MAIN_DATA_HOST) ||
                    serverName.equals(VirtualHosts.VIRTUAL_HOST_SMLPX.normalisedName)) /* On a generic/main URL. */ &&
               (null != reqURI) &&
                   (reqURI.startsWith(CATPAGE_PREFIX) || reqURI.startsWith(EX_PREFIX)) /* Cat page or exhibit. */ &&
               (null == httpServletRequest.getSession(false)) /* User has no session. */ &&
               Rnd.fastRnd.nextBoolean() /* Prevent indefinite bouncing of a user request. */ &&
               !WebUtils.requestProbablyFromSpider(httpServletRequest) /* Never bounce a spider/bot. */ &&
               (null == WebUtils.requestProbablyReferredFromExternalSite(httpServletRequest)) /* Only bounce on internal clicks (eg not on first hit). */ )
                {
                // Only consider bouncing the user
                // if the user is not already close to us
                // (ie in their country based on a FAST lookup for the user).
                final String regionByAddress = GeoUtils.getRegionByAddress(clientIP, true);
                final boolean validCcTLDForUser = CCTLD.isSyntaticallyValidCcTLD(regionByAddress);
                if(validCcTLDForUser &&
                   !mirrorTag.startsWith(regionByAddress))
                    {
                    // If we manage to find a good local mirror for the user
                    // then we set it below and bounce the user to it with a temp redirect.
                    String localMirrorTag = null;

                    // Get list of active mirrors in the user's country.
                    // We do not take into account whether
                    // the mirrors are on the same AEP version as us
                    // which means that there is a small chance that
                    // we will redirect the user to a non-existant page (404).
                    // We will eliminate dead mirrors later.
                    final DataSourceBean dsb = DataSourceBean.getApplicationInstance(config.getServletContext());
                    final SortedMap<String,Long> mirrors = LoadBalancingUtils.getActiveMirrors(dsb, false);
                    // Weed out those not in the same country as the user.
                    for(final Iterator<String> tagIt = mirrors.keySet().iterator(); tagIt.hasNext(); )
                        {
                        final String tag = tagIt.next();
                        if(!tag.startsWith(regionByAddress)) { tagIt.remove(); }
                        }
                    // Order remaining mirrors best-first by bandwidth
                    // and eliminate mirrors not known to be alive and responsive.
                    final List<String> bestFirstMirrors = LoadBalancingUtils.orderMirrorTagsBestBandwidthFirst(mirrors);

if(IsDebug.isDebug) { logger.log("INFO: candidate for redirection to closer mirror: client="+clientIP+", region="+regionByAddress+"; same-country mirrors="+bestFirstMirrors.size()); }

                    // If there are any available servers in the user's county
                    // then select that which has the highest available bandwidth.
                    if(!bestFirstMirrors.isEmpty())
                        { localMirrorTag = bestFirstMirrors.get(0); }

                    // If we haven't yet found the ideal local mirror,
                    // and we have no proximity (NONE) to the user,
                    // then bounce them to the "optimal" mirror if significantly closer to them.
                    // This has to be within a closely-connected group of countries (ie COUNTRYGROUP),
                    // not just in the huge area controlled by an address registry (ie CONTINENT).
                    if((null == localMirrorTag) &&
                       (GeoProximity.NONE == GeoUtils.computeProximity(regionByAddress, new GeoUtils.CCTLD(mirrorTag.substring(0, 2)))))
                        {
                        final String bestMirrorHostAndPort = WebUtils.chooseMirrorHostToBalanceLoad(httpServletRequest, dsb);
                        final CCTLD putativeOptimalMirrorCCTLD = HostUtils.getMirrorCC(bestMirrorHostAndPort);
                        if((null != putativeOptimalMirrorCCTLD) &&
                           (GeoUtils.computeProximity(regionByAddress, putativeOptimalMirrorCCTLD).getCloseness() >= GeoProximity.COUNTRYGROUP.getCloseness()))
                            {
                            // OK, "optimal" mirror is fairly close to user, so bounce them to it.
                            localMirrorTag = HostUtils.getMirrorTag(bestMirrorHostAndPort);
                            }
                        }

                    // If we found a better/close mirror for the user
                    // then bounce them to it.
                    if(null != localMirrorTag)
                        {
                        // There is a mirror for the user
                        // in their own country (and it's not us)
                        // so redirect (temporarily) to the local mirror,
                        // preserving the URI.
                        final String newURL = "http://" + LoadBalancingUtils.makeMirrorNameFromTag(localMirrorTag) + reqURI;
                        ((HttpServletResponse) response).sendRedirect(newURL); // Temp redirect.

                        // Try to do some of the logging
                        // AFTER pushing the redirect to the user
                        // to try to minimise overall latency.
                        StatsLogger.captureDataPoint(statsIDTHRFGEN,
                                        ThroughputMonitorFilterPG2K.THRFNAME_REDIR_TO_LOCAL);
                        final String message = "INFO: redirected (temp:302) from inbound server name "+serverName+" to local "+TextUtils.sanitiseForXML(newURL, 256, true)+" for "+regionByAddress+" client="+clientIP;
                        logger.log(message);
                        // Record which country of the bounced user.
                        dsb.setVariable(new SimpleVariableValue(
                            SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                            "bounced-to-local-mirror=" + regionByAddress));
                        return;
                        }
                    }
                }


            // Pass this request to the main monitor code to track.
            super.doFilter(request, response, chain); // Pass it up...


            // Do some processing after the request has been handled.
            _doCoda(config, httpServletRequest, isGET, reqURI, statusInRBL, clientIP, checkBL);
            }
        else
            {
            // We can't handle this request, so pass it up the chain.
            chain.doFilter(request, response);
            }
        }

    /**Handle coda processing, ie after servicing body of request.
     * Processing started after the bulk of the request has been dealt with
     * may be less visible to the end user.
     */
    private void _doCoda(final FilterConfig config,
                         final HttpServletRequest requestH,
                         final boolean isGET,
                         final String reqURI,
                         final Tuple.Pair<Long, String> statusInRBL,
                         final InetAddress clientIP,
                         final boolean checkBL)
        throws IOException
        {
        // Note when we start the wind-down...
        final long codaStart = System.currentTimeMillis();

        final DataSourceBean dsb = DataSourceBean.getApplicationInstance(config.getServletContext());
        final boolean isSpider = WebUtils.requestProbablyFromSpider(requestH);

        // If this looks like a catalogue-page isGET request then count it as such.
        // Do so only AFTER the response completes to minimise latency.
        // We use reqURI rather than pathInfo
        // to try to avoid counting some (internal) redirects.
        final boolean isCatPageGet = isGET && !isSpider &&
                       (reqURI != null) &&
                       (reqURI.endsWith("ml")) /* Eg .html or .wml */ &&
                       (reqURI.startsWith(CATPAGE_PREFIX));
        if(isCatPageGet)
            {
            // Record another 1000 cat-page hits...
            if(catPageHitCounter.incrementAndGet() >= 1000)
                {
                catPageHitCounter.addAndGet(-1000);
                StatsLogger.captureDataPoint(statsIDTHRFGEN, THRFNAME_kHIT_CAT_PAGE);
                // Log the cat-page kHit centrally too.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                    THRFNAME_kHIT_CAT_PAGE));
                }
            }

        // Note if this looks like a page hit (not exhibit, image, etc).
        // Exclude some obvious noise such as spiders and bots...
        if(isGET && !isSpider &&
           (reqURI != null) && (ExhibitMIME.getInputFileType(reqURI) == null) &&
           !((statusInRBL != null) && (statusInRBL.second != null)) /* Looks like a compromised machine... */ )
            {
            final String canonReqURI = AdUtils.canonicaliseURI(reqURI);
            dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.ACCESSPATTERN_PAGEVIEW_BY_URI,
                    canonReqURI));

            // If this page-hit was referred from outside
            // and looks as if it might be a PPC click-though,
            // then log it.
            //
            // We assume that all such referrals in practice will be GETs
            // with URIs that start and end with "/" (or end with /index.jsp")
            // (and *might* include a query string)
            // and that we won't send PPC direct to a catalogue page,
            // eg we will send to pages such as "/money/".
            // (We may send to /index.jsp pages to avoid one browser redirect.)
            // This allows us to filter out a lot of noise quickly.
            // (TODO: Later we might try to log or categorise them better.)
            final String referrerNorm;
            final String referrerRaw;
            if(!isCatPageGet &&
               (reqURI != null) && reqURI.startsWith("/") && (reqURI.endsWith("/") || reqURI.endsWith("/index.jsp")) && /* !reqURI.startsWith("/_") && */
               (null != (referrerRaw = requestH.getHeader("Referer"))) && (referrerRaw.length() != 0) && /* (referrerRaw.indexOf('?') != -1) && */
               (null != (referrerNorm = WebUtils.requestProbablyReferredFromExternalSite(requestH))) && (referrerNorm.length() != 0))
                {
                // Sanitise record slightly for safety.
                logger.log("Possible PPC referral from host: " + referrerNorm + " to URI: " + reqURI + " rawRef: "+ TextUtils.escapeHTMLMetaChars(referrerRaw) + " client: " + requestH.getRemoteAddr());

                // Note the raw landing URI.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                        "landingURI=" + reqURI));

                // Note the referring host.
                dsb.setVariable(new SimpleVariableValue(
                    SystemVariables.GENSTATS_STRING_GLOBAL_EVENT,
                        "landingURI.referrerHost=" + referrerNorm));
                }
            }

        // Note the hit by approximate geographical region...
        // Do so only AFTER the response completes to minimise latency,
        // and only for requests that did not abort with an exception.
        // We may be prepared to spend some time trying to look this up
        // dynamically, hoping that DNS cacheing will keep the cost down.
        final String regionByAddress = GeoUtils.getRegionByAddress(clientIP, ThroughputMonitorFilterPG2K.DO_QUICK_GEO_LOOKUP);
        StatsLogger.captureDataPoint(statsIDTHRFGEN,
            ThroughputMonitorFilterPG2K.THRFNAMEPR_CLIENT_REGION + regionByAddress);

        // Note the rough proximity of the client so that
        // we can monitor how well geo-sensitive load balancing is doing.
        // We can only do this if we know our physical location
        // (eg if this server is a mirror and thus has an embedded ccTLD).
        final String mirrorTag = LocalProps.getMirrorTag();
        if(mirrorTag != null)
            {
            final GeoUtils.CCTLD cc = new GeoUtils.CCTLD(mirrorTag.substring(0, 2));
            final GeoProximity proximity = GeoUtils.computeProximityByAddress(clientIP, cc, ThroughputMonitorFilterPG2K.DO_QUICK_GEO_LOOKUP);
            StatsLogger.captureDataPoint(statsIDTHRFGEN,
                ThroughputMonitorFilterPG2K.THRFNAMEPR_CLIENT_PROXIMITY + proximity);
            }

        // Look for access by a compromised machine.
        // Do this AFTER handling the request
        // to reduce the impact on legitimate clients,
        // especially on their first page load(s).
        //
        // Do we have any valid (non-expired) info on this client?
        // Atomically lock out other lookup activity if no such info,
        // while we start a lookup ourself.
        //
        // Avoid making the effort (and the DNS traffic!) for exempt URIs.
        if(checkBL)
            {
            final boolean unknownRBLStatus;
            synchronized(inRBL)
                {
                final Tuple.Pair<Long, String> after = inRBL.get(clientIP);
                unknownRBLStatus = (after == null) ||
                    (after.first.longValue() < System.currentTimeMillis());
                // If unknown then create a temporary record
                // so as to lock out wasteful concurrent lookups
                // while we are running one of our own.
                if(unknownRBLStatus)
                    {
                    // Preserve old status (if available) for a while,
                    // or give client benefit of the doubt
                    // during the lookup.
                    inRBL.put(clientIP, new Tuple.Pair<Long, String>(
                        new Long(System.currentTimeMillis() + 20000 + Rnd.fastRnd.nextInt(20000)),
                        (after == null) ? null : after.second));
                    }
                }
            // Find out RBL status of this client if we don't know...
            // Spin this off in a separate thread if possible for speed,
            // using the thread pool for I/O-bound work.
            if(unknownRBLStatus)
                {
                ThreadUtils.nonCPUThreadPool.submit(new Runnable(){
                    public final void run()
                        {
                        final Set<String> BLs = getDNSBLs();
if(IsDebug.isDebug) { logger.log("[Running DNSRBL check on " + clientIP + " in " + (new ArrayList<String>(BLs)) + "...]"); }
                        // Do a fairly fast-but-thorough check.
                        final String DNSBLServer = AddrTools.isAddrBlackholed(BLs,
                                                                     clientIP,
                                                                     false);
                        if(DNSBLServer != null)
                            {
                            // This definitely seems to be a bad client,
                            // so record this with a long-life entry in cache.
                            // We make the expiry time a little harder to guess
                            // (using goodRnd) for the black hats out there...
                            // Mean expiry time is DNSBL_RESULT_CACHE_MS.
                            inRBL.put(clientIP, new Tuple.Pair<Long, String>(
                                new Long(System.currentTimeMillis() + (DNSBL_RESULT_CACHE_MS/2) + Rnd.goodRnd.nextInt(DNSBL_RESULT_CACHE_MS)), DNSBLServer));
                            }
                        else
                            {
                            // If the client does not seem to be in any RBL,
                            // then cache a negative entry for a short(er) time.
                            // Mean expiry time is < DNSBL_RESULT_CACHE_MS.
                            inRBL.put(clientIP, new Tuple.Pair<Long, String>(
                                new Long(System.currentTimeMillis() + (DNSBL_RESULT_CACHE_MS/5) + Rnd.fastRnd.nextInt(DNSBL_RESULT_CACHE_MS)), null));
                            }
                        }
                    });
                }
            }

        // Record how long coda processing takes, if significant.
        final long codaEnd = System.currentTimeMillis();
        final long codaTime = codaEnd - codaStart;
        if(codaTime > 1)
            {
            StatsLogger.captureDataPoint(statsIDTHRFGEN,
                ThroughputMonitorFilterPG2K.THRFNAMEPR_CODATIME + GenUtils.log2Approx(codaTime));
            }
        }

    /**Get the set of DNS RBLs to screen traffic, empty if none; never null. */
    private Set<String> getDNSBLs()
        {
        try
            {
            final GenProps gp = DataSourceBean.getApplicationInstance(getConfig().getServletContext()).getGenProps(-1);
            final Set<String> RBLs = gp.getDNSRBLs();
            if(RBLs != null) { return(RBLs); }
            final Set<String> empty = Collections.emptySet();
            return(empty);
            }
        catch(final Exception e)
            {
            e.printStackTrace(); // Whinge but continue...
            final Set<String> empty = Collections.emptySet();
            return(empty);
            }
        }

    /**Approximate time we cache a positive or negative DNS BL lookup value for; strictly positive.
     * A value of the order of an hour or so
     * probably matches the policy of the underlying DNS BLs reasonably well.
     */
    private static final int DNSBL_RESULT_CACHE_MS = 3600 * 1000 +
                                          Rnd.fastRnd.nextInt(1800 * 1000);

    /**LRU set of clients currently known to be in a DNSRBL.
     * A mapping is kept from the client IP address
     * to the String name of the RBL for a bad client
     * (or to null for a good client not in any RBL).
     * <p>
     * A thread-safe LRU (access-order) map is used to limit memory use
     * (it does not grow if it can discard an old/expired entry).
     * The underlying DNS service should also be cacheing for us,
     * so a cache-miss here is probably not desperately expensive.
     */
    private final Map<InetAddress, Tuple.Pair<Long, String>> inRBL =
        Collections.synchronizedMap(
            new LinkedHashMap<InetAddress, Tuple.Pair<Long, String>>(1001, 0.7f, true){
                protected final boolean removeEldestEntry(final Map.Entry<InetAddress, Tuple.Pair<Long, String>> entry)
                    { return((entry.getValue() == null) || (entry.getValue().first == null) || (entry.getValue().first.longValue() < System.currentTimeMillis())); /* Expired. */ }
                private static final long serialVersionUID = 3013203103902112946L;
                });


    /**Exclude some traffic from connection-responsiveness monitoring.
     * Exclude stuff that we expect to be unavoidably slow,
     * or that is non-interactive and need not be fast.
     *
     * @return true to ask for this request not to be monitored
     */
    protected boolean dontMonitorMe(final HttpServletRequest httpServletRequest)
        {
        // Ignore tunnel traffic from mirrors/peers.
        if("POST".equalsIgnoreCase(httpServletRequest.getMethod()))
            { return(true); }

        return(false); // The default is to monitor all connections.
        }
    }

