#!/bin/sh
# Used to back up the whole exhibit area using ZIP (non-compressing),
# breaking out separate ZIP archives for sub-directories
# where the whole directory >>~1G.
#
# Will fail for leaf (sub-) directories >2GB, ZIP's limit for one archive.
#
# Pass as arguments one or more directories to back up.
#
# Will write ZIP archives to $BACKUPS dir, which can be symlinked elsewhere.

# Max size (~1.5G) that we will volunarily put in one ZIP archive.
ZIPLIMIT=1500000

BACKUPS=../backups
if [ ! -d $BACKUPS ]; then
    echo Backup directory "$BACKUPS" does not exist, please create.
    exit 1
fi

if [ $# -lt 1 ]; then
    echo Please supply one or more directories to ZIP up.
    exit 1
fi

for d in $*; do
    echo Processing $d...
    if [ ! -d $d ]; then
        echo Not a directory, skipping...
    else
        size="`du -ks $d | awk '{print $1}'`"
        echo "  Size: $size kBytes."
        ZIPNAME="$BACKUPS/`echo $d | sed -e s/[/]/_/g`.zip"
        if [ "$size" -le "$ZIPLIMIT" ]; then
            echo "  Storing in one (recursive) ZIP archive $ZIPNAME ..."
            zip -0rq $ZIPNAME $d || exit 1
            ls -al $ZIPNAME
        else
            echo "  TOO BIG FOR ONE ZIP: using $ZIPNAME for this dir, and sub-archives..."
            zip -0q $ZIPNAME $d/* || exit 1
            ls -al $ZIPNAME
            echo "  Recursing for sub-directories of $d..."
            for f in $d/*; do
                if [ -d $f ]; then
                    $0 $f || exit 1
                fi
            done
        fi
    fi
done
