#!/bin/sh
# Utility to ensure each significant source file contains a copyright notice.

# egrep pattern to search for to ensure file is copyrighted.
COPYRIGHTPAT="[Cc]opyright "

# Path to plain-text licence file.
LICFILE=LICENCE.txt
if [ ! -r $LICFILE ]; then
    echo "Licence file $LICFILE does not exist."
    exit 1
fi
if egrep -s "$COPYRIGHTPAT" $LICFILE; then
    : OK
else
    echo "Licence file $LICFILE does not contain required copyright string."
    exit 1
fi

# If ADDIT is true, add a header containing the notice if absent,
# else simply report missing headers.
ADDIT=true

# Gather a list of Java/JSP sources.
SRC=`find websrc javasrc \( -name '*.java' -o -name '*.jsp' \) -type f`
for f in $SRC;
    do
        if [ ! -s $f ]; then 
            echo SKIPPING ZERO-LENGTH FILE: $f
        fi
        if egrep -s "$COPYRIGHTPAT" $f; then
            echo OK: $f
        else
            echo MISSING: $f
            if $ADDIT; then
                # CAREFULLY prepend the licence text iff:
                #   * Licence text file is readable and contains magic text.
                #   * Target file is writable.
                #   * There is not already a "pre-update" file from earlier.
                DN=`dirname $f`
                BN=`basename $f`
                OLDFILE="$DN/.withoutcopyright.$BN"
                WORKFILE="$DN/.tmp.$BN"
                if [ ! -w $f ]; then
                    echo "$f not writable: skipping."
                    continue;
                fi
                if [ -f $OLDFILE -o -f $WORKFILE ]; then
                    echo "$OLDFILE or $WORKFILE already exists: skipping."
                    continue;
                fi

                # Base exact behaviour on file extension...
                case $f in
                    *.java)
                        # Put in C-style comment containing licence text.
                        echo "/*" > $WORKFILE
                        cat $LICFILE >> $WORKFILE
                        echo "*/" >> $WORKFILE
                        cat $f >> $WORKFILE
                        ;;
                    *.jsp)
                        # Put in JSP comment containing licence text.
                        echo "<%--" > $WORKFILE
                        cat $LICFILE >> $WORKFILE
                        echo "--%>" >> $WORKFILE
                        cat $f >> $WORKFILE
                        ;;
                    *)
                        echo "Skipping unrecognised file type."
                        continue
                        ;;
                esac

                # Move updated file into place...
                if [ ! -s $WORKFILE ]; then
                    echo "Internal error: zero-length work file $WORKFILE."
                    exit 1
                else
                    if /bin/mv $f $OLDFILE && /bin/mv $WORKFILE $f; then
                        echo "$f updated..."
                    else
                        echo "ABORTING: problem with $f."
                        exit 1
                    fi
                fi
            fi
        fi
    done
