<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%><%--

     Displays search results in catalogue; include in XHTML JSP page.

     I make this as compact (and compressable) as reasonably possible.
     
     This expects the following Java variables to be available in scope:
       * String [] searchResults; non-null ordered set of short exhibit names
       * int MAX_TO_SHOW; maximum number of results to show (must be positive)
       * int firstIndex; the index of the first result to display (must be non-negative)
       * dataSource [JavaBean].

     At most MAX_TO_SHOW results are shown starting at firstIndex in
     searchResults.

     If firstPos >= searchResults.length then no results are shown and
     and error message is displayed.

     If firstPos + MAX_TO_SHOW searchResults.length then results are
     shown from firstPos to the end of the array.

     The text generated is a table; it would probably look best if centred.

     If we discover that we are taking longer than we should to generate
     our output, we may stop including thumbnails, etc.

     We insert description text in the tables if possible,
     though we may omit repeated or overlong text.

--%><%@ page
    import="java.io.*,
            java.net.*,
            java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            org.hd.d.pg2k.svrCore.location.Location,
            org.hd.d.pg2k.svrCore.uploader.UploaderUtils" %><%
{ /* Scope the variables locally... */

// Note when we start generating output
// so as to limit page-display delay for the user.
// Uses whole-request-start time if available as more conservative.
final long _srNow = System.currentTimeMillis();
final Object _srPageStartTime = (request.getAttribute(org.hd.d.pg2k.webSvr.util.WebConsts.REQUEST_START_DATETIME_ATTR_NAME));
final long _srStartTime = (_srPageStartTime instanceof Date) ? (((Date) _srPageStartTime)).getTime() : _srNow;

// Note the set of description (and AKA) texts displayed so far.
final Set descriptionsShown = new HashSet(searchResults.length * 2 + 1);

final SessionVarBean _srSvb = SessionVarBean.getExtantSessionVars(request, false);
final boolean _srIsLite = (_srSvb != null) && _srSvb.isSessionVarLiteUI();

%><%--

RESULTS (as list)
--%><%
if(firstPos >= searchResults.length)
    { %><span style="color: red;"><%= localeBean.getLocalisedMessage("common.cat.nothingFound") %></span><% }
else
    {
    // Get access to exhibit properties.
    final AllExhibitProperties _srAep = dataSource.getAllExhibitProperties(-1);
    // Get access to generic properties.
    final GenProps _srGp = dataSource.getGenProps(-1);
    // Get alternate background colour, quoted if need be.
    final String bgAltColQ = UploaderUtils.quoteHTMLArg(
        PageSkinUtils.getAltBgColour(pageContext));
    final int _sr_pageDisplayTarget_ms = ((siteIsBusy || _srIsLite ||
                                           (WebConsts.FIRST_PAGE_AFAP && (WebUtils.requestProbablyReferredFromExternalSite(request) != null))) ?
                                            CoreConsts.MAX_INTERACTIVE_DELAY_MS : WebConsts.MAX_PG_DOWNLOAD_MS) / 4;
    %><dl><%-- Force a linebreak for XHTML readability. --%>
    <%
    final int _srHigh = Math.min(firstPos + MAX_TO_SHOW, searchResults.length) - 1;

    // Find out what thumbnails are to hand locally right now,
    // possibly forcing them to be created or fetched in parallel.
    final int nItems = _srHigh+1 - firstPos;
    final String _srFullNames[] = new String[nItems];
    for(int _srIndex = firstPos; _srIndex <= _srHigh; ++_srIndex)
        { _srFullNames[_srIndex-firstPos] = _srAep.aeid.getFullName(searchResults[_srIndex]); }
    final BitSet _srThumbnails = WebUtils.exhibitsHaveThumbnail(dataSource, Arrays.asList(_srFullNames), false,
        !siteIsBusy && !_srIsLite && (System.currentTimeMillis() - _srStartTime) < _sr_pageDisplayTarget_ms);

    // Now create the table...
    for(int _srIndex = firstPos; _srIndex <= _srHigh; ++_srIndex)
        {
        if(searchResults[_srIndex] == null) { continue; } // Should not really happen...

        //final boolean _srOddRow = (((_srIndex - firstPos) & 1) != 0);
        //final String _srBgCol = _srOddRow ? "" : " bgcolor="+bgAltColQ;

        // Tie label/bullet into accesskey availability.
        final boolean useAccessKey = (nextAccessKeyValue <= MAX_AK);
        final char bulletChar = useAccessKey ? ((char) ('0' + (nextAccessKeyValue++))) : '*';

        %><dt><%= bulletChar %></dt><%

        %><dd><%
        try
            {
            // Get full name of exhibit...
            final String _srFullName = _srFullNames[_srIndex-firstPos];
            if(_srFullName == null) { continue; } // Should not really happen...
            final String _srCategory = ExhibitName.getCategoryComponent(_srFullName);

		    // Get the exhibit type (and if it can be inlined for this user agent)...
		    final ExhibitMIME.ExhibitTypeParameters _srExhibitType = (ExhibitMIME.getInputFileType(_srFullName));
            final boolean _canInline = WebUtils.canInlineInXHTMLPage(_srExhibitType, request);

            // If the site is busy or the user has requested a "lite UI" or
            // if we're using too much of the available page-generation time,
            // or the thumbnails are not a suitable MIME-type for .mobi XHTML,
            // then don't force thumbnails current not-to-hand to be created/fetched
            // (and avoid some other time-consuming activities too).
            final boolean _srDontCreateTn = siteIsBusy || _srIsLite ||
                !_canInline ||
                ((System.currentTimeMillis() - _srStartTime) >= _sr_pageDisplayTarget_ms);

            // Compute root-relative URL to catalogue page.
            final String _srCatPageRRURL = WebUtils.makeCatPageRRURL(_srFullName, WebConsts.F_secondary_generated_XHTML_suffix);

            // Get RRURL for small thumbnail.
            final String _srThumbnailRRURL = WebUtils.makeThumbnailRRURL(_srFullName, false);
            // Avoid computing the EPC, etc, if we can't inline a thumbnail anyway.
            final ExhibitPropsComputable _srEpc = !_canInline ? null : _srAep.getExhibitPropsComputable(_srFullName);
            final java.awt.Dimension _srXyDim = (_srEpc == null) ? null : _srEpc.getXyDimensions();
            // Compute what thumbnail dimensions should be...
            final java.awt.Dimension _srThumbnailXyDim = (_srXyDim == null) ? null : ExhibitThumbnails.computeThumbnailDimensions(_srXyDim, false);

            if(_canInline &&
               (_srThumbnailRRURL != null) &&
               (_srThumbnailXyDim != null) &&
               WebUtils.exhibitHasThumbnail(dataSource, _srFullName, false, !_srDontCreateTn))
                {
                // Insert XHTML for inline thumbnail...
                %><a href="<%= _srCatPageRRURL %>"><%
                %><img src="<%= _srThumbnailRRURL %>" width="<%= _srThumbnailXyDim.width %>" height="<%= _srThumbnailXyDim.height %>" alt="<%= localeBean.getLocalisedMessage("common.thumbnail") %>" /><%
                %></a><%
                }
            else
                {
                // Can't inline a thumbnail; put in size instead...
                %>[<%= TextUtils.sizeAsText(_srAep.aeid.getStaticAttr(_srFullName).length, true) %>]<%
                }

            // Insert indicator icons if/as appropriate:
            //   * New indicator.
            //   * Smile/frown if notably popular/unpopular.
            //   * Earth icon if we have an Estd location.
            try
                {
                // Newness indicator.
                if((_srNow - _srAep.aeid.getStaticAttr(_srFullName).timestamp) <= CoreConsts.MAX_EXHIBIT_AGE_NEW_MS)
                    {
                    %><span style="font-weight: bold; color: red;"><%= localeBean.getLocalisedMessage("common.selection.newExhibits.title") %></span> <%
                    }

                // Good/bad indicator.
                // (Only compute if not short of time: may require significant effort).
                if(!_srDontCreateTn && !GenUtils.isSensitive(_srFullName, _srAep, _srGp))
                    {
                    final ExhibitPropsComputableMutable _srEpcm = _srAep.getExhibitPropsComputableMutable(_srFullName, _srDontCreateTn, _srGp, dataSource, dataSource.getScorerCache());
                    if(_srEpcm != null)
                        {
                        final Boolean isGood = _srEpcm.isGood();
                        if(isGood != null)
                            {
                            %><img src="/_static/icon/<%= isGood.booleanValue() ? WebConsts.PRO_ICON_15x15_NAME : WebConsts.CON_ICON_15x15_NAME %>" width="15" height="15" alt="<%= isGood.booleanValue() ? "good" : "bad"%>" /> <%
                            }
                        }
                    }

                // Has-location indicator.
                if(_srAep.getLocation(_srFullName) instanceof Location.Estd)
                    {
                    %><img src="/_static/icon/E.gif" width="16" height="11" alt="Has location information" /> <%
                    }
                }
            catch(final Exception e)
                {
                /* Skip these icons silently if we encounter any difficulty. */
                application.log("Error displaying indicator icons for "+_srFullName+": " + e.getMessage());
                }

	        %> <a href="<%= _srCatPageRRURL %>"<%
	            if(useAccessKey) { %> accesskey="<%= bulletChar %>"<% }
	            %>><%= WebUtils.hyphenateHTMLText(searchResults[_srIndex]) %></a><%

            // Insert category on end of line in brackets.
            %> (<%-- <a href="<%= WebConsts.VIRTUAL_COLLECTIONS_BYCATEGORY_ROOT %><%= _srCategory %>/"> --%><%= GenUtils.computeSectionTitle(_srCategory, localeBean) %><%-- </a> --%>)<%
            }
        catch(final Throwable e)
            { %><%= WebUtils.hyphenateHTMLText(searchResults[_srIndex]) %><% }
        %></dd><%-- Force a linebreak for XHTML readability. --%>
        <%
        }
    %></dl><%
    }
%><%--


--%><% } /* End local variable scope. */ %>
