<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%--Selects and redirects to a random catalogue page handler for HTML.
--%><%@ page
    contentType="text/html;charset=UTF-8"
    import="java.io.*,
            java.net.*,
            java.text.*,
            java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.MIME.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.svrCore.vars.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            org.hd.d.pg2k.webSvr.exhibit.*"
    session="false"
    %><%--
--%><% final long startTime = System.currentTimeMillis(); %><%---
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...
--%><jsp:useBean id="RndExhibitSet"
             class="org.hd.d.pg2k.webSvr.exhibit.FilterBean"
             scope="application"><%--
    --%><%
    // This bean is private to this random page.
    // We want all exhibits.
    // We aim to have "better" exhibits first if possible.
    RndExhibitSet.setExpr(new SortExpr(null, new BuiltInFilters.sortByGoodness(null)));

    // Set this to automatically time out and recompute periodically.
    // Aim to for a period of about the default system temporal slackness
    // or the computable-mutable recalc time if more.
    RndExhibitSet.setExpiryInterval(Math.min(2*CoreConsts.DEFAULT_TEMPORAL_SLACKNESS_S,
                                             ExhibitPropsComputableMutable.MAX_AGE_BEFORE_STALE_MS/2000) *
                                    (500 + Rnd.fastRnd.nextInt(1000)));
    %><%--
--%></jsp:useBean><%--
--%><%

    // Make this page uncacheable.
    response.setHeader("Cache-Control", "no-cache");
    response.setHeader("Pragma", "no-cache");
    response.setHeader("Expires", "0");

    // Get full list of exhibits in best-first order.
    final List<String> rndExhibits = RndExhibitSet.select(application);

    // If no exhibits at all
    // then report the service to be temporarily unavailable.
    final int n = rndExhibits.size();
    if(n < 1)
        {
        response.setStatus(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
        return;
        }

    // If true then skew results towards showing "good" exhibits.
    // These are more pleasant to see, but we might want to stop doing it
    // it our votes are unbalanced too far towards "pro".
    // We tally the current and (full) previous period pro-and-con votes
    // to get a representative picture and to help revert to the skew-to-good mode ASAP.
    // We can allow pro votes to exceed con somewhat
    // since we hope that visitors actually like most of what they see!
    //
    // Allow the skew by default some of the time (eg for speed).
    boolean skewToGood = Rnd.fastRnd.nextBoolean();
    if(!skewToGood)
	    {
	    // Measure any current skew...
	    final int recentVotesPro =
	        dataSource.getEventValue(SystemVariables.VOTE_PRO, EventPeriod.VLONG, false).getTotalEventCount() + 
	        dataSource.getEventValue(SystemVariables.VOTE_PRO, EventPeriod.VLONG, true).getTotalEventCount();
	    final int recentVotesCon = 
	        dataSource.getEventValue(SystemVariables.VOTE_CON, EventPeriod.VLONG, false).getTotalEventCount() + 
	        dataSource.getEventValue(SystemVariables.VOTE_CON, EventPeriod.VLONG, true).getTotalEventCount();
	    skewToGood = (recentVotesPro <= 2*recentVotesCon);
	    }    

    // Choose the exhibit to punt the visitor to.
    // Weight more towards earlier (assumed better) exhibits if skewToGood is true.
    // We try (a few times, so as to improve the user experience)
    // to find an exhibit with at least a small thumbnail immediately available to show
    // that can be viewed on virtually any device, eg including handhelds.
    int triesLeft = 3 + Rnd.fastRnd.nextInt(7);
    int which;
    String exhibitName;
    do  {
        which = skewToGood ? Rnd.goodRnd.nextInt(1+Rnd.goodRnd.nextInt(n)) : Rnd.goodRnd.nextInt(n);
        exhibitName = rndExhibits.get(which);
        }
    while((--triesLeft >= 0) &&
          !WebUtils.exhibitHasThumbnail(dataSource, exhibitName, false, false) /* No small thumbnail immediately/locally available. */ &&
          !WebUtils.canInlineInXHTMLPage(ExhibitMIME.getInputFileType(exhibitName), request) /* Thumbnail type not Accept-able to this User-Agent. */ );

    try
        {
        // If we find that the computable properties for the exhibit selected are stale...
        // then force recomputation for that exhibit
        // (and some others) and invalidate the "best" cache
        // (but only if the site is lightly loaded).
        final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);
        final GenProps gp = dataSource.getGenProps(-1);
        final ExhibitPropsComputableMutable epcm = aep.getExhibitPropsComputableMutable(exhibitName, true, gp, dataSource, dataSource.getScorerCache());
        if((epcm != null) && epcm.isStale() && WebUtils.isLightlyLoaded(application))
            {
            // Force at least this exhibit computable-mutable data up-to-date...
            aep.getExhibitPropsComputableMutable(exhibitName, false, gp, dataSource, dataSource.getScorerCache());

            // Spend a few moments bringing some other exhibits up to date too.
            // Limit the time we spend doing this so as not to madden users too much.
            final long stopBy = startTime + CoreConsts.MAX_INTERACTIVE_DELAY_MS/2;
            int index = which;
            do
                {
                // Just try exhibits in order: why churn the random number generator more?
                if(++index >= n) { index = 0; }
                // Stop when we find an exhibit up-to-date...
                final ExhibitPropsComputableMutable epcm2 =
                        aep.getExhibitPropsComputableMutable(rndExhibits.get(index), true, gp, dataSource, dataSource.getScorerCache());
                if((epcm2 != null) && !epcm2.isStale()) { break; }
                // Force this stale one to be computed...
                aep.getExhibitPropsComputableMutable(rndExhibits.get(index), false, gp, dataSource, dataSource.getScorerCache());
                } while(System.currentTimeMillis() <= stopBy);

            // Force re-sort of the exhibit set next time to reflect non-stale value(s)...
            RndExhibitSet.invalidate();
            }
        }
    catch(final Exception e)
        {
        /* Ignore any errors and continue... */
        }

    // Default to redirecting to the HTML catalogoue page.
    String suffix = WebConsts.F_secondary_generated_HTML_suffix;
    final Object requestedSuffix = request.getParameter("suffix");
    // Explictly allow one or two standard/approved/safe alternatives.
    if(WebConsts.F_secondary_generated_XHTML_suffix.equals(requestedSuffix))
        { suffix = (String) requestedSuffix; }

    // Now make the URL.
    final String catPg = WebUtils.makeCatPageRRURL((String) rndExhibits.get(which), suffix);

    // Now do (temporary) redirect.
    response.sendRedirect(catPg);
    //return;
%>
