<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%--Handles display of one node of a browsable exhibit tree, including links up and down as needed.

    A presorted (in case-insensitive String order) unmodifiable and unchanging List
    of lower-cased exhibit stem prefixes and full exhibit names
    as returned by TreeFilterBean.select() is expected to be accessible in
    the request-scope attribute named by WebConsts.BETDJ_LIST_ATTR_NAME.
    A null list is treated like an empty list, ie as no entries...

    A boolean array of uninteresting nodes above the current node (true means uninteresting)
    (excluding the root and current node, lowest index is nearest to root)
    to which we should not bother to provide upwards links, is expected to be accessible in
    the request-scope attribute named by WebConsts.BETDJ_LIST_UNIN_NAME.
    A null/missing value is treated as if an all-false array of the right size.

    The base URI (starting with a "/") of the tree is expected as a String in
    the request-scope attribute named by WebConsts.BETDJ_BASE_URI_ATTR_NAME.

    The title (or part thereof) in plain ASCII if possible is expected as a String in
    the request-scope attribute named by WebConsts.BETDJ_TITLE_ATTR_NAME.

    The (lowercase) prefix word sequence (or "" if none) is expected as a String in
    the request-scope attribute named by WebConsts.BETDJ_PRWSEQ_ATTR_NAME.

    If breadcrumbs are already put in then this will not override them.


    In future we may break this into pages, but for now everything goes in one page.

--%><%@ page
    contentType="text/html;charset=UTF-8"
    import="java.io.*,
            java.net.*,
            java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            org.hd.d.pg2k.svrCore.uploader.*,
            org.hd.d.pg2k.svrCore.vars.*,
            org.hd.d.pg2k.webSvr.ads.*,
            org.hd.d.pg2k.webSvr.catalogue.*"
    session="false"
    buffer="32kb" %><%--
--%><%@ include file="/_common/_SetPageStartTime.inc.jsp" %><%-- Note start time if not already done for standard footer.
--%><%@ include file="/_common/_localeBean.inc.jsp" %><%-- Get locale bean set up...
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...

SET UP THE PAGINATION BEAN...
--%><jsp:useBean id="paginationBean"
             class="org.hd.d.pg2k.webSvr.catalogue.PaginationBeanTree"
             scope="page"/><%-- Create the backing JavaBean, just to the end of this page...
<% final PaginationBeanTree paginationBean = new PaginationBeanTree(); %>

--%><%

    // Get the list of sub-nodes and exhibit names...
    List<String> nodeContent = (List<String>) request.getAttribute(WebConsts.BETDJ_LIST_ATTR_NAME);
    // If it's null, make it non-null.
    // TODO: maybe this should redirect to top of tree or/then to "Not found" JSP?
    if(nodeContent == null) { nodeContent = Collections.EMPTY_LIST; }

    // Get the Map (String->Integer) of subnodes to exhibit count; may be null.
    final Map<String,Integer> snExCounts = (Map<String,Integer>) request.getAttribute(WebConsts.BETDJ_LIST_SNEXCOUNT_NAME);

    // Get the "uninteresting" parent-node list.
    final boolean[] uninterestingNodes = (boolean[]) request.getAttribute(WebConsts.BETDJ_LIST_UNIN_NAME);

    // Get the base URL for the tree.
    final String baseURI = (String) request.getAttribute(WebConsts.BETDJ_BASE_URI_ATTR_NAME);

    // Get the prefix word sequence ("" for none).
    final String prwseq = (String) request.getAttribute(WebConsts.BETDJ_PRWSEQ_ATTR_NAME);

    // Get the title.
    final String title = (String) request.getAttribute(WebConsts.BETDJ_TITLE_ATTR_NAME);

    // Get the common section for this node, if any.
    final String commonSection = (String) request.getAttribute(WebConsts.BETDJ_COMMON_SECTION);
    %><%--
--%><%!
    /**Prefix to session attribute name to hold "sticky" page labels.
     * Suffix the URI of a node (as returned by _makeOfficialURI() to get its page label.
     */
    private static final String pgAttrNamePrefix = "org.hd.pg2k.tree.pgAttr.";

    /**Gets the node's sticky page label from the session if any, else null.
     * This is driven by the target node URI.
     */
    private static final String _getStickyPageLabel(HttpServletRequest request, String nodeURI)
        {
        final HttpSession optionalSession = request.getSession(false);
        if(optionalSession != null)
            {
            final Object pgSessionValO = optionalSession.getAttribute(pgAttrNamePrefix + nodeURI);
//System.err.println("Session attr get: " + pgAttrName + " = " + pgSessionValO);
            if(pgSessionValO instanceof String) { return((String) pgSessionValO); }
            }
        return(null); // No value available.
        }

    /**Given a node's URI as returned by _makeOfficialURI(), modify it with the node's sticky page if extant.
     * Doing this may help prevent a browser retrieving the wrong page from cache.
     */
    private static final String _makeModifiedPageURI(final HttpServletRequest request, final String nodeURI)
        {
        final String stickyPage = _getStickyPageLabel(request, nodeURI);
        return((stickyPage == null) ? nodeURI : // If no sticky page return URI unmolested.
            (nodeURI + '?' + WebConsts.PAGE_NUMBER_PARAMETER + '=' + stickyPage));
        }
    %><%--
--%><%

    // Compute the URI of this page (where it was redirected from).
    // We MUST construct this in the same way that we construct links up and down within the tree.
    final String thisURI = TreeFilterServletBase.makeOfficialURI(baseURI, prwseq);

    // 301 PERM REDIRECTS FOR NON-CANONICAL URI.
    // Do a permanent redirect if the actual URI does not match the canonical one.
    // We do this to eliminate from SEs and caches all the infortmation-free variants.
    // We do not (yet) attempt to preserve any query strings/parameters, etc...
    if(WebConsts.CANON_TREE_URIS_WITH_REDIR)
	    {
	    final String savedURI = (String) request.getAttribute(ThroughputMonitorFilterPG2K.PNAME_SAVED_REQ_URI);
	    final String actualURI = (null != savedURI) ? savedURI : request.getRequestURI();
	    if(!thisURI.equals(actualURI))
	        {
//application.log("INFO: tree: redirecting (301:perm) from tree page "+actualURI+" to "+thisURI);
	        response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
	        response.setHeader("Location", thisURI);
	        return;
	        }
	    }

    // Tell the pagination bean the items that we will be paginating...
    paginationBean.setTreeNodeDetails(prwseq, nodeContent);
    // And capture the user's requested page.
    final String pgParam = request.getParameter(WebConsts.PAGE_NUMBER_PARAMETER);
    paginationBean.setPg(pgParam);

    // If an old/invalid label has been used (eg by an SE/spider with a long memory),
    // then redirect (302/temp) back to unlabelled page to trim the state space.
    // (Don't use a 301/perm redirect, since this label may become valid again.)
    if((pgParam != null) && !pgParam.equals(paginationBean.getPageLabel(paginationBean.getPg())))
        {
application.log("INFO: tree: redirecting (302:temp) from invalid/old label '"+pgParam+"' at "+thisURI);
        response.sendRedirect(thisURI);
        return;
        }

    // Compute session attribute name for page variable at this level of this tree.
    // We make a canonical unique attribute name for this node based on servlet mount point and word prefix.
    final String pgAttrName = pgAttrNamePrefix + thisURI;

    // If no page parameter supplied, look for session value (sticky page label) instead.
    // Don't force session (yet) if there isn't one.
    if(pgParam == null)
        { paginationBean.setPg(_getStickyPageLabel(request, thisURI)); }

    // Total pages.
    final int totalPages = paginationBean.getNumberOfPages();
    // This page, numbered from 1.
    final int pageNum = paginationBean.getPg();

    // Store page number for this node in the session if we have to,
    // creating a session if need be.
    if(pageNum != 1)
        {
        // Force session value to hold non-default page label; this may force a cookie on the user.
        request.getSession(true).setAttribute(pgAttrName, paginationBean.getPageLabel(pageNum));
//System.err.println("Session attr set: " + pgAttrName + " := " + pageNum);
        }
    // If we wish to indicate default page (1) or don't need to store anything, clear any session value...
    else
        {
        final HttpSession optionalSession = request.getSession(false);
        if(optionalSession != null) { optionalSession.removeAttribute(pgAttrName); } // No session value implies page 1.
        }

    // Compute the sub-title from prefix and note of page-in-range number.
    final StringBuilder subTitle = new StringBuilder(prwseq);
    if(totalPages > 1)
        {
        if(subTitle.length() > 0) { subTitle.append(' '); } // Add space between components.
        subTitle.append('[').append(pageNum).append(" / ").append(totalPages).append(']');
        }

    // Show all the entries in one page for now.
    final int subListStart = paginationBean.getStartIndex();
    final int subListEnd = paginationBean.getEndIndex();
    final List thisPageItems = nodeContent.subList(subListStart, subListEnd);
%><%--


HELP AVOID RECRAWLING/REGENERATING SLOWLY-CHANGING TREE NODES TOO OFTEN.
Any (small) tree node consisting only of relatively old exhibits
can be marked as last modified with the date of the newest of the exhibits
for the benefit of SEs/spiders,
with unnecessary refetches aborted with 304 SC_NOT_MODIFIED to save bandwidth/effort.
(Humans should always get the freshest page possible, eg with new ads.)
We need not do this with more than leaf and near-leaf nodes,
since they should in fact be the largest, and slowest-changing, set.
We'll all spiders to cache ALL tree pages for a little longer than usual.
--%><%
	if(WebUtils.requestProbablyFromSpider(request))
	    {
	    final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);

	    // Usually put a little delay in (re)spidering all virtual/tree pages,
	    // so that these pages are generally (re)spidered less than catalogue pages.
	    // Set the change threshold relative to the last-known AEP change.
	    request.setAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME, new Long(aep.hashNotChangedSince - WebConsts.SPIDER_PAGE_EXPIRY_MS/2));

        // For small (near-)leaf nodes a more precise control may be possible...
        if(totalPages < 2)
	        {
	        // Consider last-modified to be timestamp of newest featured exhibit.
		    long lastModified = -1;
	
		    // Find the newest exhibit, and use its timestamp as our timestamp
		    // with a backstop of the oldest page we will let an SE cache.
		    // We compare "If-Modified-Since" against this "true" timestamp,
		    // but generate and new page with a stamp which is
		    // the later of this "true" timestamp and the whole AEP timestamp
		    // on the grounds that the exact page format does not matter in the SE's cache
		    // but the semantics may change slowly over time
		    // and upon actually fetching the page a new-ish "optimistic" timestamp
		    // should displace any really old version in cache.)
		    //
		    // Abort to avoid creating a Last-Modified header or using "If-Modified-Since"
		    // if any entry is not a valid exhibit short name (eg is a sub-node link).
		    // Note that a more general-purpose set of cache headers
		    // may still be set by the general HTML header/template code.
		    final AllExhibitImmutableData aeid = aep.aeid;
		    final Map<String,String> s2fm = aeid.getShortToFullMap();
		    for(final String name : nodeContent)
		        {
		        final String fullName = s2fm.get(name);
		        if(fullName == null) { lastModified = -1; break; }
		        final ExhibitStaticAttr esa = aeid.getStaticAttr(fullName);
		        if(esa == null) { lastModified = -1; break; }
		        if(esa.timestamp > lastModified) { lastModified = esa.timestamp; }
		        }

	        // If we have a suitable synthetic "last modified" date
	        // then mark this page as "last-modified" with it.
		    if(lastModified > -1)
	            { request.setAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME, new Long(lastModified)); }
		    }
	    }
%><%--


PREV/NEXT LINKS (partly to ensure spiderability)
--%><% if(totalPages > 1)
    {
    if(pageNum > 1) // "Prev"
        { pageContext.setAttribute("prev", "?pg=" + paginationBean.getPageLabel(pageNum-1)); }
    if(pageNum < totalPages) // "Next"
        { pageContext.setAttribute("next", "?pg=" + paginationBean.getPageLabel(pageNum+1)); }
    } %><%--


HEADER
--%><% if(WebConsts.USE_NOINDEX_ON_LOW_CONTENT_PAGES) { request.setAttribute(WebConsts.STDHDR_ATTR_NAME_NOINDEX, Boolean.TRUE); } %><%--
--%><% request.setAttribute(WebConsts.STDHDR_ATTR_NAME_TITLE, title); %><%--
--%><% if(subTitle.length() > 0) { request.setAttribute(WebConsts.STDHDR_ATTR_NAME_SUBTITLE, subTitle.toString()); } %><%--
--%><%@ include file="/_common/_HTMLStdHd.inc.jsp" %><%--


BODY STARTS...


IF GENERAL DESCRIPTION TEXT EXISTS, INSERT IT.
--%><%
    final Object generalDescO = request.getAttribute(WebConsts.BETDJ_HTML_GENERAL_DESC_ATTR_NAME);
    if(generalDescO instanceof String)
        { %><%= generalDescO %><p align=center><hr width="30%"/></p><% }
    %><%---


GIVE IDEA OF RANGE...
--%><% if(nodeContent.size() > 1) { %><%--
--%><p><strong><%= nodeContent.get(0).toString().replace(ExhibitName.WORD_SEP, ' ') %></strong> ... <strong><%= nodeContent.get(nodeContent.size() - 1).toString().replace(ExhibitName.WORD_SEP, ' ') %></strong></p><%--
--%><% } %><%--


OTHER PAGES BUTTONS (top copy)
--%><% if(totalPages > 1) { %><%--
--%><form method=<%=WebConsts.SEARCH_FORM_METHOD%> action="<%= thisURI %>"><%--
--%><p align=center><%@ include file="/_common/_HTMLPageButtons.inc.jsp" %></p><%--
--%></form><%--
--%><% } %><%--


--%><%
    // Try to allow some extra concurrency for fetching furniture images,
    // eg for non-pipelining broswers,
    // so that fetching these does not delay other page content.
    final String sneakyHostPrefix = WebUtils.getOptionalSneakyConcurrencyRRURLPrefix(request);

    // If this is not the top of the tree, provide links up to earlier elements.
    if(prwseq.length() > 0)
        {
        %><p><%

        // Link up to root of this tree (unconditionally).
        final String rootNodeURI = TreeFilterServletBase.makeOfficialURI(baseURI, "");
        %><a href="<%= _makeModifiedPageURI(request, rootNodeURI) %>" ><img src="<%= sneakyHostPrefix %>/_static/icon/upleft.gif" alt="^" border=0 height=32 width=32 align=middle></a> <%


        // FIXME: this is a VERY inefficient way to make the sub-prefixes...
        final StringTokenizer st = new StringTokenizer(prwseq, ExhibitName.WORD_SEPS);

        // Partial word prefix...
        final StringBuilder sb = new StringBuilder(prwseq.length());

        for(int w = 0; st.hasMoreTokens(); ++w)
            {
            final String word = st.nextToken();
            if(!st.hasMoreTokens()) { break; } // Don't do the last word...
            sb.append(word).append('-');

            // Found out if we should bother to link up...
            // We do unless we find an explict entry telling us the node is uninteresting...
            final boolean doLink =
                (uninterestingNodes == null) ||
                (uninterestingNodes.length <= w) ||
                (uninterestingNodes[w] == false);

            if(doLink)
                {
                final String nodeURI = TreeFilterServletBase.makeOfficialURI(baseURI, sb.toString());
                %><a href="<%= _makeModifiedPageURI(request, nodeURI) %>"><%
                }
            %><%= word %><%
            if(doLink) {%></a><% }
            %> <%
            }
        %></p><%
        }
%><%--


NOW DUMP LINKS AND EXHIBITS
--%><%
    final SessionVarBean svb = SessionVarBean.getExtantSessionVars(request, false);
    final boolean isLite = (svb != null) && svb.isSessionVarLiteUI();

    // We accumulate any (short) exhibit names to be pushed out in a block in exhibitNames.
    final ArrayList exhibitNames = new ArrayList(thisPageItems.size());
    for(final Iterator it = thisPageItems.iterator(); it.hasNext(); )
        {
        final String name = (String) it.next();

        final boolean isSubTree = name.endsWith(ExhibitName.WORD_SEPS);

        // If the item is an exhibit, simply add it to the exhibit list to dump later.
        if(!isSubTree)
            { exhibitNames.add(name); }

        // If this item is a sub-tree or the last item,
        // dump any accumulated exhibits
        // and then clear the list
        // (providing that there was something in the list).
        if((isSubTree || !it.hasNext()) &&
            (exhibitNames.size() > 0))
            {
            final int count = exhibitNames.size();

            // Build the search-result HTML.
            // Note that exhibit names are already in short format
            // (ie the file component only).
            final String searchResults[] = new String[count];
            exhibitNames.toArray(searchResults);
            final int MAX_TO_SHOW = count;
            final int firstPos = 0;
            %><p><%@ include file="HTMLSearchResults.inc.jsp" %></p><%

            // Clear the exhibits list.
            exhibitNames.clear();
            }

        // If this is a sub-tree,
        // then print this sub-link.
        if(isSubTree)
            {
            final String subTreeURI = TreeFilterServletBase.makeOfficialURI(baseURI, name);
            %><p><%
            %><a href="<%= _makeModifiedPageURI(request, subTreeURI) %>"><img src="<%= sneakyHostPrefix %>/_static/icon/dir.gif" alt=<%= UploaderUtils.quoteHTMLArg(localeBean.getLocalisedMessage("common.directory")) %> border=0 height=32 width=32 align=middle> <%= name.replace(ExhibitName.WORD_SEP,  ' ') %>...</a><%
            // If we have a sub-node exhibit count, show it...
            if(snExCounts != null)
                {
                final Object co = snExCounts.get(name);
                if(co instanceof Integer)
                    { %> <i>(<%= co %>)</i><% }
                }
            // If we can find any AKA text then print it,
            // if not in "lite" UI mode.
            if(!isLite && (null != commonSection))
                {
                final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);
                final String fakeName = commonSection + '/' + name + "A.a";
                final String AKAText = GenUtils.getLocalisedTreeDesc(aep,
                                                                     fakeName,
                                                                     localeBean,
                                                                     false, true, true, false);
                if(!"".equals(AKAText))
                    { %><small><small><ul compact><%= AKAText %></ul></small></small><% }
                }
            %></p><%
            }
        }
%><%--


INSERT OPTIONAL AD
Regulate display by traffic (page views) rather than CTR.
--%><% if(AdConsts.ENABLE_ADBRITE && AdUtils.showAds(dataSource, request) &&
          !AdUtils.skipUnderperformingAdSlot(dataSource, SystemVariables.ACCESSPATTERN_PAGEVIEW_BY_URI, EventPeriod.VLONG, AdUtils.canonicaliseURI((String) request.getAttribute(ThroughputMonitorFilterPG2K.PNAME_SAVED_REQ_URI)))) { %>
<center><br /><table border=1><tr><td>
<%-- <%= (new AdBriteBean()).setParams(AdBriteBean.AD_FORMAT_AB_HORIZONTAL).getHTMLBodyInsert(null, Rnd.fastRnd) %> --%>
<iframe width="<%= 500 + Rnd.fastRnd.nextInt(100) %>" height="95" src="<%= sneakyHostPrefix %>/_static/ads/468x60ABAd.html" title="AdBrite Ads" frameborder="0" marginwidth="0" marginheight="0" scrolling="no" allowTransparency="true"></iframe>
</td></tr></table><br /></center>
<% } %><%--



OTHER PAGES BUTTONS (bottom copy)
--%><% if(totalPages > 1) { %><%--
--%><form method=<%=WebConsts.SEARCH_FORM_METHOD%> action="<%= thisURI %>"><%--
--%><p align=center><%@ include file="/_common/_HTMLPageButtons.inc.jsp" %></p><%--
--%></form><%--
--%><% } %><%--



FOOTER
--%><%@ include file="/_common/_HTMLStdFt.inc.jsp" %>
