<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%--Catalogue page handler for serving exhibit XML metadata.
    Since we serve a lot of these we'd like to be extra-efficient on space
    if we can, so, for example, not adding lots of spurious whitespace,
    which implies odd comment start/end to eliminate blank lines...
--%><%@ page
    import="java.io.*,
            java.net.*,
            java.text.*,
            java.util.*,
            java.util.concurrent.*,
            java.util.concurrent.atomic.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            ORG.hd.d.jIndexer.server.JIndexBean,
            org.hd.d.pg2k.svrCore.location.*,
            org.hd.d.pg2k.svrCore.MIME.ExhibitMIME,
            org.hd.d.pg2k.webSvr.location.*,
            org.w3c.dom.Node,
            org.hd.d.pg2k.svrCore.vars.*,
            org.hd.d.tmf.ThroughputMonitorFilterBase,
            org.hd.d.pg2k.webSvr.ads.*,
            org.hd.d.pg2k.webSvr.virtualHosts.AlohaEarth.*,
            org.hd.d.pg2k.webSvr.virtualHosts.VirtualHosts,
            org.hd.d.pg2k.svrCore.mediahandler.swf,
            org.hd.d.pg2k.svrCore.uploader.*,
            java.nio.ByteBuffer"
    %><%-- Should inherit session="false"...
--%><% response.setContentType("application/xhtml+xml;charset=UTF-8"); %><%--
--%><%@ include file="/_common/_SetPageStartTime.inc.jsp" %><%-- Note start time if not already done for standard footer.
--%><%@ include file="/_common/_localeBean.inc.jsp" %><%-- Get locale bean set up...
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...
--%><%
    // Provide local aliases for some imported values...
    final DataSourceBean dsb = dataSource;
    final LocaleBeanBase lb = localeBean;

    // Get all exhibit properties; never null.
    final AllExhibitProperties aep = dsb.getAllExhibitProperties(-1);
    final Comparator<String> exhibitComparator = ExhibitAttrUtils.getAttrWords().SMART_ORDER;

    // Retrieve the exhibit name from the request...
    final String exhibitName = (String) request.getAttribute("exhibitName");
    // Check that the syntax is valid and that the exhibit exists...
    if(!ExhibitName.validNameSyntax(exhibitName))
        {
        request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
        return;
        }
    // Get this exhibit's properties; never null.
    final ExhibitStaticAttr esa = aep.aeid.getStaticAttr(exhibitName);
    // If it doesn't exist gently tell the user so...
    if(esa == null)
        {
        request.setAttribute("javax.servlet.error.request_uri", request.getRequestURI()); // Force setup of request URI.
        request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
        return;
        }
    // Compute the short (file-component-only) name.
    final String shortName = ExhibitName.getFileComponent(exhibitName);

    // Get the exhibit type...
    final ExhibitMIME.ExhibitTypeParameters exhibitType = (ExhibitMIME.getInputFileType(esa.filePath));

    // Get the author initials...
    final String auth = ExhibitName.getAuthorComponent(esa.filePath);

    // Get the exhibit loadable properties; never null.
    final ExhibitPropsLoadable epl = aep.getExhibitPropsLoadable(exhibitName);

    // Get the exhibit immutable computable properties if possible.
    final ExhibitPropsComputable epc = aep.getExhibitPropsComputable(exhibitName);

    final GenProps gp = dsb.getGenProps(-1);

    // Does our visitor seem to be a spider?
    final boolean probablySpider = WebUtils.requestProbablyFromSpider(request);

    // Reduce server load when being spidered (or possibly if busy)
    // by generating more 304s for unpopular old exhibits,
    // ie allow their pages to be cached for longer.
    if(probablySpider)
        {
        // Get the exhibit immutable computable properties,
        // but don't force recomputation at all if not immediately available.
        final ExhibitPropsComputableMutable epcm =
            aep.getExhibitPropsComputableMutable(exhibitName, true, gp, dsb, dsb.getScorerCache());
        // If unpopular, allow spider to use much older cached page.
        if((epcm != null) && !epcm.isStale())
            {
            final int goodness = epcm.getGoodness();
            // "Bad" (and not new) exhibits can be respidered very infrequently.
            if(goodness < 0)
                { request.setAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME, new Long(esa.timestamp)); }
            // Not-quite "good" (and not new) exhibits can be respidered reasonably frequently.
            else if(goodness < Integer.MAX_VALUE - ExhibitPropsComputableMutable.GOODBAD_LIMIT_INT)
                { request.setAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME, new Long(Math.max(aep.hashNotChangedSince - WebConsts.SPIDER_PAGE_EXPIRY_MS/4, esa.timestamp))); }          
            // Implicitly, "good" exhibits get respidered most frequently.
            }
        }

    // Does this look like a new (or newly-sticky) visitor?
    final boolean probablyNewClient = (null != request.getAttribute(ThroughputMonitorFilterBase.REQ_ATTR_NAME_NEW_CLIENT_FLAG));

    // FIND PREV AND NEXT EXHIBITS (if any)...
    // We use this to tie all the exhibit catalogue pages together
    // for the benefit of spiders and humans!
    // Get full exhibit set (full names), sorted in SMART_ORDER order.
    final List<String> allExhibits = FullSortedExhibitSet.select(application);
    // Find our own location in the array using same order as sort used...
    final int ourPos = Collections.binarySearch(allExhibits, exhibitName, exhibitComparator);
    // Get PREV and NEXT catalogue URLs, if any.
    final String prevPage = (ourPos > 0) ? WebUtils.makeCatPageRRURL((String) allExhibits.get(ourPos-1), WebConsts.F_secondary_generated_XHTML_suffix) : null;
    final String nextPage = ((ourPos >= 0) && (ourPos < allExhibits.size() - 1)) ? WebUtils.makeCatPageRRURL((String) allExhibits.get(ourPos+1), WebConsts.F_secondary_generated_XHTML_suffix) : null;

    final String category = ExhibitName.getCategoryComponent(exhibitName);
    final String categoryTitle = GenUtils.computeSectionTitle(category, lb);
    final String categoryURL =
        WebConsts.VIRTUAL_COLLECTIONS_BYCATEGORY_ROOT + category + '/';

    // Make the breadcrumbs.
    // Claim that the page is "within" the category tree,
    // and show similar breadcrumbs.
    final Breadcrumbs crumbs = new Breadcrumbs();
    crumbs.append(new Breadcrumbs.Breadcrumb(lb.getLocalisedMessage("common.main.VirtualCollections"), WebConsts.VIRTUAL_COLLECTIONS_ROOT));
    crumbs.append(new Breadcrumbs.Breadcrumb(lb.getLocalisedMessage("common.virtCol.byCategory.title"), WebConsts.VIRTUAL_COLLECTIONS_BYCATEGORY_ROOT));
    crumbs.append(new Breadcrumbs.Breadcrumb(categoryTitle, categoryURL));
%><%--


START COMPUTING "SIMILAR EXHIBIT" RESULTS
This is likely to be slow, so we start it ASAP,
in the hope of doing this concurrently with other activity.
--%><% final Future<List<String>> searchResultFuture = SearchResultSimpleCache.doCachedCatPageSimilarItemsFuture(dsb, exhibitName, 2); %><%--


HEADER
--%><% request.setAttribute(WebConsts.STDHDR_ATTR_NAME_TITLE, categoryTitle); %><%--
--%><%
    // Insert page-specific keywords in title and meta-headers if extant.
    final String specificAKA = GenUtils.getLocalisedTreeDesc(aep, exhibitName, lb, false, true, false, true);
    final boolean hasSpecificAKA = (specificAKA.length() != 0);
    final String keywords = ExhibitName.getMainWordsComponent(exhibitName, ExhibitAttrUtils.getAttrWords().getAttrWordsSortedSet()).replace(ExhibitName.WORD_SEP, ',') +
        (hasSpecificAKA ? (',' + specificAKA) : "");
    request.setAttribute(WebConsts.STDHDR_ATTR_NAME_KEYWORDS, keywords);
    request.setAttribute(WebConsts.STDHDR_ATTR_NAME_SUBTITLE, WebUtils.hyphenateHTMLText(WebUtils.minimalUniqueENTitlePrefix(allExhibits, ourPos)) +
        (!hasSpecificAKA ? "" : (" - " + specificAKA) ));
    %><%--
--%><% if(prevPage != null) { pageContext.setAttribute("prev", prevPage); } %><%--
--%><% if(nextPage != null) { pageContext.setAttribute("next", nextPage); } %><%--
--%><% request.setAttribute(WebConsts.STDHDR_ATTR_NAME_BREADCRUMBS, crumbs); %><%--
--%><%@ include file="/_common/_XHTMLStdHd.inc.jsp" %><%--
--%><% final boolean isBusy = siteIsBusy; %><%--


BODY
--%><%
// Try to compute what XxY of image (if exhibit is an image)...
final java.awt.Dimension xyDim = (epc == null) ? null : epc.getXyDimensions();

// Parameter name for selecting a bigger (ie standard) thumbnail.
final String bigTnParamName = "bigTn";

// True if the user requested a big thumbnail.
// False by default.
// Overridden (to default/false) if we don't have a large thumbnail.
boolean bigTn = "true".equals(request.getParameter(bigTnParamName));

// Insert (small by default) thumbnail if possible...
// FOR .mobi THIS IS ONLY FOR GIF AND JPEG IMAGES
final URL exhibitURL = WebUtils.makeExhibitURL(exhibitName, request, dsb);

final boolean canInline = WebUtils.canInlineInXHTMLPage(exhibitType, request);
final ExhibitThumbnails tns = dsb.getThumbnails(exhibitName, !isBusy && !probablySpider);
final boolean hasSmlTn = (null != tns) && (null != tns.getSmall());
final boolean hasStdTn = (null != tns) && (null != tns.getStandard());
if(canInline)
    {
    // Try and get the thumbnail.
    // Don't bother trying to create the tn if we're busy or the visitor is a bot.
    if(bigTn && !hasStdTn) { bigTn = false; }
    if((null != tns) && (bigTn ? hasStdTn : hasSmlTn))
        {
        // Get RRURL for small thumbnail.
        final String rrURL = WebUtils.makeThumbnailRRURL(exhibitName, bigTn);

        // Compute what thumbnail dimensions should be...
        final java.awt.Dimension thumbnailXyDim = (xyDim == null) ? null : ExhibitThumbnails.computeThumbnailDimensions(xyDim, bigTn);

        // Link tn to other tn size if available.
        final boolean haveOtherTn = (bigTn ? hasSmlTn : hasStdTn);
        // Use full RRURL to work round buggy browsers (eg embedded Opera in Nokia 9300).
if(haveOtherTn) { %><a href="<%= WebUtils.makeCatPageRRURL(exhibitName, WebConsts.F_secondary_generated_XHTML_suffix) %><% if(!bigTn) { %>?<%= bigTnParamName %>=true<% } %>"><%-- Link to other thumbnail. --%><% }

%><img src="<%= rrURL %>" alt="<%= localeBean.getLocalisedMessage("common.thumbnail") %>"<%
        if(thumbnailXyDim != null)
            {
            %> width="<%= thumbnailXyDim.width %>"<%
            %> height="<%= thumbnailXyDim.height %>"<%
            }
%> /><%

if(haveOtherTn) { %> <%= bigTn ? "-" : "+" %></a><% }

%><br /><%
        }
    }
// Can handle some non-embeddable media specially...
else if(exhibitType != null)
    {
    if(exhibitType.mimeType.startsWith("audio/"))
        {
        %><img src="/_static/icon/audio.gif" width="64" height="64" alt="audio" /><br /><%
        }
    else if(exhibitType.mimeType.startsWith("video/"))
        {
        %><img src="/_static/icon/video.gif" width="64" height="64" alt="video" /><br /><%
        }
    }

final Location.Base location = aep.getLocation(exhibitName); // Never null.
%><%--
Emphasise and label the download link...
--%><span style="font-weight: bold; color: red;"><%= lb.getLocalisedMessage("common.cat.Download") %></span><%--

Now the small/medium/full download links...
--%>
<% if(hasSmlTn) { %><small><a href="<%= WebUtils.makeThumbnailRRURL(exhibitName, false) %>"><%= lb.getLocalisedMessage("common.cat.Download.smlTn") %></a></small><% } %>
<% if(hasStdTn) { %><a href="<%= WebUtils.makeThumbnailRRURL(exhibitName, true) %>"><%= lb.getLocalisedMessage("common.cat.Download.stdTn") %></a><% } %>
<big><a href="<%= exhibitURL %>"><%= lb.getLocalisedMessage("common.cat.Download.full") %></a></big>
<br />
<%--

Now the full exhibit name and info...
--%><strong><%= WebUtils.hyphenateHTMLText(ExhibitName.getFileComponent(exhibitName)) %></strong><br />
<%
    // Newness indicator.
    if((System.currentTimeMillis() - esa.timestamp) <= CoreConsts.MAX_EXHIBIT_AGE_NEW_MS)
        { %><span style="font-weight: bold; color: red;"><%= lb.getLocalisedMessage("common.selection.newExhibits.title") %></span> <% }

      // Flag up big (>~250kB) exhibits, eg that may be expensive to download over wireless.
      final boolean isBig = (esa.length >= 250123);
      %><%--
      --%>D=<%= CoreConsts.DEFAULT_DATE_FORMAT.format(new Date(esa.timestamp)) %><%--
          now do date in user's locale... --%> [<%= DateFormat.getDateInstance(DateFormat.MEDIUM, lb.getLocale()).format(new Date(esa.timestamp)) %>]<%--
      --%>; S=<% if(isBig) { %><span style="font-weight: bold; color: red;"><% } %><%= TextUtils.sizeAsText(esa.length, true) %><% if(isBig) { %></span><% } %><%--
          --%><%= (xyDim == null) ? "" : (", " + xyDim.width + "x" + xyDim.height) %><%--
      --%>; T=<%= (exhibitType == null) ? "?" : (exhibitType.description + " [MIME:" + exhibitType.mimeType + "]") %><%--
      --%><% if(location != Location.NONE) { %>; <br />L=<%= location %><% } %><%--
      --%>.<br /><%--

PUT IN COPYRIGHT NOTICE AND LINKS (to Ts&Cs and author's virtual collection)
--%><strong><%= lb.getLocalisedMessage("common.cat.freeToUse") %></strong> <%--
--%><a href="http://<%= CoreConsts.MAIN_DATA_HOST %>/terms.html">&copy;</a> <%= auth %><%--
--%>
<%-- Insert linebreak for sanity of anyone reading the XHTML...


DESCRIPTION
// Insert description (and location info, etc) if extant, and safe...
// We don't allow text with any (HTML) tags at all because it might be XHTML-unsafe,
// but we do allow entity codes because those are generally OK.
--%><%
final String description = epl.getDescription();
if((description != null) &&
   (description.indexOf('<') == -1) &&
   (description.indexOf('>') == -1))
    { %><p><%= description %></p><% }
%><%--


SIMILAR EXHIBITS
Do simple query based on full name of exhibit and possibly any descriptive text...
Only show results if we get any!
Possibly don't show this if the site is busy as it is expensive,
but still allow a user to explicitly search for similar exhibits.
--%><%

    { // Scope our variables.

// Force out partial page so far iff search results are not yet ready.

if(!searchResultFuture.isDone()) { out.flush(); }

try
    {
    List<String> wrappedResult = searchResultFuture.get();
    
    // Make sure we remove any duplicates wrt those already shown.
    if(!wrappedResult.isEmpty() /* && !alreadyShownShortNames.isEmpty() */ )
        {
        wrappedResult = new ArrayList<String>(wrappedResult);
        //wrappedResult.removeAll(alreadyShownShortNames);
        }
    final String[] searchResults = new String[wrappedResult.size()];
    wrappedResult.toArray(searchResults);

    // Now display the results, if any...
    if(searchResults.length > 0)
        {
        // Display from first item.
        final int firstPos = 0;
        // Show the first couple of results...
        final int MAX_TO_SHOW = Math.min(2, searchResults.length);

%>
<h2><%= lb.getLocalisedMessage("common.cat.similarExhibits") %></h2>
<%@ include file="XHTMLSearchResults.inc.jsp" %><br />
<%

        // Don't show any of the `similar exhibits' again...
        //alreadyShownShortNames.addAll(Arrays.asList(searchResults));

        } // End `if any results' if.

    } // End `doing search' if.
catch(final Throwable e) { e.printStackTrace(); }

// End local variable scope...
} %>
<%-- Insert linebreak for sanity of anyone reading the HTML...


NOW COLLECT SOME STATS...
...since we've generated all the important page content
so any time here may be hidden by network latency...
Try to ignore spiders/robots and lunatics and the author testing the site!  B^>
--%><%
    if(!probablySpider &&
       !WebUtils.isPrecacheRequest(request))
        {
        try
            {
            // Record the short exhibit name as an event...
            final SimpleVariableValue svv = new SimpleVariableValue(SystemVariables.ACCESSPATTERN_CAT_PAGE_VIEW,
                                                                    shortName);
            dsb.setVariable(svv);
            }
        catch(final Throwable e) // Absorb any errors...
            {
            e.printStackTrace();
            }
        }
    %><%--


Link to HTML (desktop) page.
--%>
<p><a href="http://<%= CoreConsts.MAIN_DATA_HOST %><%= WebUtils.makeCatPageRRURL(exhibitName, WebConsts.F_secondary_generated_HTML_suffix) %>">HTML (desktop) --- <%= lb.getLocalisedMessage("common.cat.title") %></a></p>
<%--


FOOTER
--%><%@ include file="/_common/_XHTMLStdFt.inc.jsp" %>
