<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%--Catalogue page handler for serving exhibit XML metadata.
    Since we serve a lot of these we'd like to be extra-efficient on space
    if we can, so, for example, not adding lots of spurious whitespace,
    which implies odd comment start/end to eliminate blank lines...
--%><%@ page
    import="java.io.*,
            java.net.*,
            java.text.*,
            java.util.*,
            java.util.concurrent.*,
            java.util.concurrent.atomic.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            ORG.hd.d.jIndexer.server.JIndexBean,
            org.hd.d.pg2k.svrCore.location.*,
            org.hd.d.pg2k.svrCore.MIME.ExhibitMIME,
            org.hd.d.pg2k.webSvr.location.*,
            org.w3c.dom.Node,
            org.hd.d.pg2k.svrCore.vars.*,
            org.hd.d.tmf.ThroughputMonitorFilterBase,
            org.hd.d.pg2k.webSvr.ads.*,
            org.hd.d.pg2k.webSvr.virtualHosts.AlohaEarth.*,
            org.hd.d.pg2k.webSvr.virtualHosts.VirtualHosts,
            org.hd.d.pg2k.svrCore.mediahandler.swf,
            org.hd.d.pg2k.svrCore.uploader.*,
            java.nio.ByteBuffer"
    %><%-- Should inherit session="false"...
--%><% response.setContentType("application/xhtml+xml;charset=UTF-8"); %><%--
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...
--%><%
    // Provide local aliases for some imported values...
    final DataSourceBean dsb = dataSource;

    // Get all exhibit properties; never null.
    final AllExhibitProperties aep = dsb.getAllExhibitProperties(-1);

    // Retrieve the exhibit name from the request...
    final String exhibitName = (String) request.getAttribute("exhibitName");
    // Check that the syntax is valid and that the exhibit exists...
    if(!ExhibitName.validNameSyntax(exhibitName))
        {
        response.sendError(response.SC_FORBIDDEN);
        return;
        }
    // Get this exhibit's properties; never null.
    final ExhibitStaticAttr esa = aep.aeid.getStaticAttr(exhibitName);
    // If it doesn't exist gently tell the user so...
    if(esa == null)
        {
        response.sendError(response.SC_NOT_FOUND);
        return;
        }


    // Get the exhibit loadable properties; never null.
    final ExhibitPropsLoadable epl = aep.getExhibitPropsLoadable(exhibitName);

    // Get the exhibit immutable computable properties if possible.
    final ExhibitPropsComputable epc = aep.getExhibitPropsComputable(exhibitName);
    final String metadata = (epc == null) ? null : epc.getMetadataAsXML();
    final AccessionData accessionMetadata = epl.getAccessionMetadata();

    final GenProps gp = dsb.getGenProps(-1);

    String mdXML = "";

    if((metadata != null) || (accessionMetadata != null))
        {
        final StringBuilder sb = new StringBuilder(512);
        if(metadata != null)
            { sb.append(metadata); }
        if(accessionMetadata != null)
            { sb.append(TextUtils.toXML(accessionMetadata.getAsDOM(), false, true)); }
        mdXML = sb.toString();
        }


    final boolean siteIsBusy = WebUtils.isOverloaded(application);
    // See if the client is likely a spider/bot.
    final boolean clientIsSpider = WebUtils.requestProbablyFromSpider(request);

    // Reduce server load for all users (but especially spiders)
    // by generating more 304s for these pages
    // since they change very slowly, if ever.
    //
    // We use the accession metadata timestamp if available,
    // else the exhibit timestamp as a fallback.
    if((accessionMetadata != null) && (accessionMetadata.date != null))
        { request.setAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME, accessionMetadata.date); }
    else
        { request.setAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME, new Long(esa.timestamp)); }

    // Set up default HTTP cache headers.
    %><%@ include file="/_common/_HTTPStdCacheHdrs.inc.jsp" %><%


    // Flush any spurious leading whitespace/rubbish accumulated to now.
    // Marginally reduces output size and is more conformant / well-formed.
    // We don't clear any headers or status code.
    response.resetBuffer();
    out.clearBuffer();

    %><?xml version="1.0" encoding="UTF-8"?>
<exhibit name="<%= exhibitName %>" size="<%= esa.length %>" timestamp="<%= esa.timestamp %>">
<%= mdXML %>
</exhibit>
