<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%-- Handler for all new-style catalogue pages.

     Punts to or embeds HTML/XML/WAP page.
  
     The request URI must end with recognised HTML/XL/WML
     page suffix and must be syntactically valid.

     (Syntactically-valid but non-existent items can be dealt with in the page.)

     We default to a (plain) text type, UTF-8 (ie 7-bit-ASCII superset) char-set.

--%><%@ page
    contentType="text/plain;charset=UTF-8"
    import="org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            java.io.IOException"
    session="false"
    buffer="32kb" %><%-- Allow for large-ish pages/writes even after (say) x4 compression.
--%><%@ include file="/_common/_SetPageStartTime.inc.jsp" %><%-- Note start time if not already done for standard footer.
--%><%@ include file="/_common/_FullSortedExhibitSetBean.inc.jsp" %><%-- Get sorted-exhibits bean...
--%><%
final String ci_rawPathInfo = request.getPathInfo();
if(ci_rawPathInfo == null)
    {
    request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
    return;
    }

// Adjust for any leading slash.
final String ci_pathInfo = (ci_rawPathInfo.startsWith("/")) ?
    (ci_rawPathInfo.substring(1)) : ci_rawPathInfo;

// Strip off last extension and make sure that this represents
// a valid (full) exhibit ci_name,
// and then post this valid ci_name as an attribute at request level.
final int lastDot = ci_rawPathInfo.lastIndexOf('.');
if(lastDot >= 2)
    {
    final String ci_name = ci_pathInfo.substring(0, lastDot-1);

    // If this is a syntactically-invalid exhibit ci_name then give up immediately.
    if(!ExhibitName.validNameSyntax(ci_name))
        {
        request.setAttribute("javax.servlet.error.request_uri", request.getRequestURI()); // Force setup of request URI.
        request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
        return;
        }

    final AllExhibitImmutableData ci_aeid = DataSourceBean.getApplicationInstance(application).
        getAllExhibitImmutableData(-1L);

    // In the special case that our exhibit set is empty,
    // probably because of a transient problem or at start-up,
    // don't allow a 404 to be generated which might cause
    // caches and search engines to loose data,
    // but instead punt with a 5XX `retry-later' code.
    if(ci_aeid.size() == 0)
        {
        response.setStatus(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
        return;
        }

    // If final component is a syntactically-valid exhibit name but not a current one
    // but its short name is valid (ie the exhibit has moved)
    // then do a (permanent) redirect to the new location.
    // Preserves the URL extension to select the correct source type.
    // Hopefully spiders will remember the new location for efficiency.
    if(null == ci_aeid.getStaticAttr(ci_name))
        {
        final String newName = ci_aeid.getFullName(ExhibitName.getFileComponent(ci_name));
        if(newName != null)
            {
            response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
            response.setHeader("Location", WebUtils.makeCatPageRRURL(newName, ci_pathInfo.substring(ci_pathInfo.lastIndexOf('.'))));
            return;
            }

        // Exhibit seems to have "gone away".
        request.setAttribute("javax.servlet.error.request_uri", request.getRequestURI()); // Force setup of request URI.
        request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
        return;
        }

    // ci_name is valid and current, so record it for the view/display page to access.
    request.setAttribute("exhibitName", ci_name);

    try
        {
        // If this is HTML, punt to the HTML handler...
        if(ci_pathInfo.endsWith(WebConsts.F_secondary_generated_HTML_suffix))
            {
            if(WebConsts.CAT_HANDLER_USE_JSP_FORWARD)
                { %><jsp:forward page="/_cat/doHTML.jsp"/><% }
            else
                { %><%@ include file="/_cat/doHTML.jsp" %><% }
            return;
            }
        // If this is XHTML, punt to the XHTML handler...
        else if(ci_pathInfo.endsWith(WebConsts.F_secondary_generated_XHTML_suffix))
            {
            if(WebConsts.CAT_HANDLER_USE_JSP_FORWARD)
                { %><jsp:forward page="/_cat/doXHTML.jsp"/><% }
            else
                { %><%@ include file="/_cat/doXHTML.jsp" %><% }
            return;
            }
        // If this is XML, punt to the XML handler...
        else if(ci_pathInfo.endsWith(WebConsts.F_secondary_generated_XML_suffix))
            {
            if(WebConsts.CAT_HANDLER_USE_JSP_FORWARD)
                { %><jsp:forward page="/_cat/doXML.jsp"/><% }
            else
                { %><%@ include file="/_cat/doXML.jsp" %><% }
            return;
            }
        // If this is WML, punt to the WML handler...
        else if(ci_pathInfo.endsWith(WebConsts.F_secondary_generated_WML_suffix))
            {
            // Hardly used, if at all, at 2006/08, so left as jsp:forward for now.
            %><jsp:forward page="/_cat/doWML.jsp"/><%
            return;
            }
        }
    catch(final Exception e)
        {
        // Catch exceptions such as broken pipes
        // (eg when clients abort the connection)
        // and log a brief error rather than a whole stack trace.
        application.log("Caught Exception in /_cat/index.jsp: " + e.getClass().getName() + ": " + e.getMessage());
        return;
        }

    // Fall through to complain that we don't know the media type...
    }

// Else note that there is no such page,
// but we don't even really know what type of page to generate...
response.setStatus(HttpServletResponse.SC_UNSUPPORTED_MEDIA_TYPE);
// Fall through to return...
%>
