<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


     Standard HTTP cache-header generator for JSP (HTML/XHTML/XML) pages.

     This expects to find the following bean available:
       * dataSource (JavaBean)

     This expects to be able to use the following variables:
       * clientIsSpider (boolean): true if our caller seems to be a spider
       * siteIsBusy (boolean): true if our site seems to be very busy

     We supply standard, spider-sensitive values for the following headers:
       * Cache-control
       * Expires
       * ETag (based on AEP hash)
       * Last-Modified* (based on AEP timestamp)
     unless they have already been set.

--%><%@ page
    import="java.io.*,
            java.net.*,
            java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            org.hd.d.pg2k.webSvr.virtualHosts.VirtualHosts,
            org.hd.d.pg2k.webSvr.ads.AdUtils"
%><%
    // Set "standard" HTTP headers for cache-ability, etc.

    // SET CACHE PARAMETERS (only if not already set)
	// Cache for a reasonable time for normal visitors;
	// postpone expiry a long time for spiders...
	// We want this to be fresh for normal visitors,
	// but spiderable by search engines.
	// We make pages private if a session is in place.
	// Note that we will not override any header already set,
	// and in particular pages may want to set their own Last-Modified value
	// for rarely-modified data, eg to help keep spider/bot load down.
    boolean _ccHdrsSet = false;
	if(!response.containsHeader("Cache-Control"))
	    {
	    // Should be safe to use "public" since we don't use any
	    // HTTP basic/digest-type authentication
	    // for which we wouldn't want pages cached in public caches/proxies,
	    // but we don't include "public" by default so as to achieve
	    // the smallest/simplest header possible
	    // and with the least chance of misinterpretation.
	    response.setHeader("Cache-Control",
	        ((request.getSession(false) == null) ? "" : "private,") +
	        (clientIsSpider ?
	            "max-age="+(WebConsts.SPIDER_PAGE_EXPIRY_MS/1000) :
	            "max-age="+(WebConsts.DEFAULT_PAGE_CACHE_MS/1000)));
	    }
	else { _ccHdrsSet = true; }
	if(!response.containsHeader("Expires"))
	    {
	    response.setDateHeader("Expires", System.currentTimeMillis() +
	        (clientIsSpider ?
	            WebConsts.SPIDER_PAGE_EXPIRY_MS :
	            WebConsts.DEFAULT_PAGE_CACHE_MS));
	    }
	else { _ccHdrsSet = true; }
	final AllExhibitProperties _aep_h = dataSource.getAllExhibitProperties(-1);
	if(!response.containsHeader("ETag"))
	    {
	    // Generate a *weak* ETag based on the AEP hash.
	    // This says that pages have only minor non-semantic changes
	    // unless the AEP changes.
	    // This might be a lie when the code changes but the data doesn't.
	    response.setHeader("ETag", "W/\"" + _aep_h.longHash + '"');
	    }
	else { _ccHdrsSet = true; }
	// Only set/use Last-Modified if no cache header has been explicitly set
	// AND if the client looks like a spider
	// (or the site is busy so we need to load-shed)
	// so that humans periodically get new fresh pages, with new ads, etc.
	// By default compare against the AEP has-not-changed-since time,
	// but allow that to be up to WebConsts.SPIDER_PAGE_EXPIRY_MS ago
	// c/o a WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME request attribute.
	// We put the backstop in to ensure that SEs (etc) eventually get updates.
	if(!_ccHdrsSet && (clientIsSpider || siteIsBusy) && !response.containsHeader("Last-Modified"))
	    {
	    final Object ims = request.getAttribute(WebConsts.SPIDER_IF_MOD_SINCE_THRESHOLD_ATTR_NAME);
	    final long lastModified = (!(ims instanceof Long)) ? _aep_h.hashNotChangedSince :
	        Math.max(((Long)ims).longValue(), System.currentTimeMillis() - WebConsts.SPIDER_PAGE_EXPIRY_MS);
	    final long ifModifiedSince = request.getDateHeader("If-Modified-Since");
	    // Use 1s precision for the compare, as per the HTTP header precision.
 		if((ifModifiedSince > -1) && (lastModified/1000 <= ifModifiedSince/1000))
            {
            // Abort if page has not (substantially) changed...
            response.sendError(HttpServletResponse.SC_NOT_MODIFIED);
application.log("INFO: 304 response to If-Modified-Since '"+request.getHeader("If-Modified-Since")+"' (limit "+(new Date(lastModified))+") on request "+request.getRequestURL());
            return;
            }
        // Report the timestamp of this download as the newer of the AEP timestamp
        // and the date that we checked any "If-Modified-Since" against.
	    response.setDateHeader("Last-Modified", Math.max(lastModified, _aep_h.hashNotChangedSince));
	    }
%>