<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
<%--
Minimal page for logging in for upload.

Can me used for manual or automated logins,
but is intended mainly for automated logins so is minimal.

If logged in, the X-LoggedIn header is present and "true", else not.
--%>
<%@ page
    import="java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            org.hd.d.pg2k.webSvr.upload.*,
            java.io.File,
            org.hd.d.pg2k.svrCore.datasource.ExhibitDataFileSource,
            org.hd.d.pg2k.svrCore.MIME.ExhibitMIME,
            ORG.hd.d.IsDebug,
            org.hd.d.pg2k.svrCore.uploader.UploaderConsts,
            org.hd.d.pg2k.svrCore.uploader.UploaderConsts"
    session="true"
%><%
    // Log accesses, at least in debug mode.
    /* if(IsDebug.isDebug) */ { System.out.println("[Access (method="+request.getMethod()+") to login page from: "+request.getRemoteAddr()+".]"); }
    /* if(IsDebug.isDebug) */ { application.log("[Access (method="+request.getMethod()+") to login page from: "+request.getRemoteAddr()+".]"); }

    // Make this page uncacheable else
    // the browser may simply go on using the last value it got.
    response.setHeader("Cache-Control", "no-cache");
    response.setHeader("Pragma", "no-cache");
    response.setHeader("Expires", "0");

    // Did we just receive valid user credentials in the arguments?
    // We insist on:
    //  1) A syntactically-valid author name.
    //  2) The author being already extant in the Gallery.
    //  3) A valid-length password being supplied.
    //  4) The password actually being correct!
    final String userIDParam = request.getParameter(UploaderConsts.USERID_FIELD_NAME);
    final String userPass = request.getParameter(UploaderConsts.PASSWD_FIELD_NAME);

/* if(IsDebug.isDebug) */ { System.out.println("[***userID = "+userIDParam+".]"); }

    if(ExhibitName.validAuthorSyntax(userIDParam) && // Valid author syntax.
       (DataSourceBean.getApplicationInstance(application).getAllExhibitProperties(-1).
            getAuthorExhibitCounts().get(userIDParam) != null) && // Extant.
       (userPass != null) && // Must be a password and a suitable length.
           (userPass.length() >= CoreConsts.MIN_PASSWORD_LEN) &&
           (userPass.length() <= CoreConsts.MAX_PASSWORD_LEN) &&
        SimplepassProps.isAuthorUploadPasswordCorrect(userIDParam, userPass))
        {
        /* Store the ID in the session and continue. */
        session.setAttribute(UploaderConsts.USERID_FIELD_NAME, userIDParam);
        if(IsDebug.isDebug) { application.log("[Login from: "+request.getRemoteAddr()+".]"); }
        }

    // If the user has asked to log out,
    // then drop their credentials, at least for this upload mechanism.
    if(request.getParameter(UploaderConsts.LOGOUT_FIELD_NAME) != null)
        {
        // Forget about this userID.
        session.setAttribute(UploaderConsts.USERID_FIELD_NAME, null);
        /* if(IsDebug.isDebug) */ { application.log("[Logout from: "+request.getRemoteAddr()+".]"); }
        }

    // Set the header if already logged in.
    final String loggedInAs = HTTPUploaderUtils.isLoggedInForUploads(request);
    if(loggedInAs != null)
        {
        // Flag if already logged in.
        response.setHeader(UploaderConsts.LOGGED_IN_HEADER, loggedInAs);
        /* if(IsDebug.isDebug) */ { application.log("[User "+loggedInAs+" is logged in from: "+request.getRemoteAddr()+".]"); }
        }

%>
<html>
<head>
<title>Upload Login Page</title>
<meta name="robots" content="noindex,nofollow">
</head>
<body bgcolor="white">
<h1>Upload Login Page</h1>

<%
    if(loggedInAs != null)
        {
        %>
<p>You are logged in as: <%= loggedInAs %>.</p>
<form action="login.jsp" method="post">
<input type="hidden" name="<%= UploaderConsts.LOGOUT_FIELD_NAME %>" value="true"/>
<p>If you want to <font color="red">log out</font>, press this button: <input type="submit" value="Log out">.</p>
</form>
        <%
        }
    else
        {
        %>
<p>Please fill in the information below to log in.
   You must have (at least temporary) cookies enabled.</p>
<form action="login.jsp" method="post">
<p><table border="1">
<tr><th>Your login ID:</th><td><input type="text" name="<%= UploaderConsts.USERID_FIELD_NAME %>" size="<%= ExhibitName.MAX_AUTH_INITIALS_LENGTH %>" maxlength="<%= ExhibitName.MAX_AUTH_INITIALS_LENGTH %>"></td></tr>
<tr><th>Your password:</th><td><input type="password" name="<%= UploaderConsts.PASSWD_FIELD_NAME %>" size="<%= CoreConsts.MAX_PASSWORD_LEN %>" maxlength="<%= CoreConsts.MAX_PASSWORD_LEN %>"></td></tr>
<tr><td colspan="2"><input type="submit" value="Log in"></td></tr>
</table>
</form>
        <%
        }
%>

</body>
</html>
