<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">
<%--
Page where we actually receive the exhibit file upload data.
This receives the multipart/form-data file data.
We ensure that the user's ID (and the exhibit info) is in the session.

If this detects multiple simultaneous attempts
to upload at once by any one user,
or attempts beyond the free space left to the user in the upload area,
this refuses the excess uploads rather bluntly.

This first uses a static variable to limit accesses to one at once.
For multi-VM (distributed) front-ends this should within that use
(for example) a lock in the file area to which the upload is done
to serialise access to the filesystem.

This checks that the file uploaded by the user has a valid magic
number and that it is not too long for the space available for upload
(the upload will be aborted at the point where the file would be too large).

Other validity checks may then be performed on the content.

The uploaded file is then moved into position.

If a description has been supplied it is atomically saved beside the exhibit
a in the usual way, ie as <exhibitname>.txt.

Thanks to <A HREF="mailto:bmarchal@pineapplesoft.com">Benot Marchal</A>
for example code to handle multipart/form-data HTTP POST data.
--%>
<%@ page
    import="java.io.*,
            java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            org.hd.d.pg2k.webSvr.upload.*,
            javax.mail.internet.*,
            javax.activation.*,
            org.hd.d.pg2k.svrCore.datasource.ExhibitDataFileSource,
            javax.mail.MessagingException,
            org.hd.d.pg2k.svrCore.uploader.UploaderConsts,
            org.hd.d.pg2k.svrCore.uploader.UploadInfoBean,
            org.hd.d.pg2k.svrCore.uploader.UploaderUtils,
            org.hd.d.pg2k.svrCore.uploader.UploaderConsts"
    session="true"
%>
<%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up... --%>
<%
    // Get the exhibit properties; never null.
    final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);

    // Make this page uncacheable.
    response.setHeader("Cache-Control", "no-cache");
    response.setHeader("Pragma", "no-cache");
    response.setHeader("Expires", "0");

%>
<html>
<head><title>Exhibit Upload (Stage 2): Receiving the File</title></head>
<body bgcolor="white">
<h1>Exhibit Upload (Stage 2): Receiving the File</h1>
<%
    // This might be used in a denial-of-service attack.
    final String uploadDirS = LocalProps.getUploadDir();
    if((uploadDirS == null) || !(new File(uploadDirS)).isDirectory() ||
        (LocalProps.getUploadMaxBytesTotal() <= 0))
        {
        // If no upload directory or space, uploads are disabled.
        %>
<p>Uploads are disabled for this system, sorry.</p>
</body></html>
        <%
        return;
        }
    if(!Boolean.FALSE.equals(dataSource.isSlave()))
        {
        // If not the master then we can't do an upload.
        %>
<p>You are not logged onto the master system,
   so you cannot do an upload, sorry.</p>
</body></html>
        <%
        return;
        }

    final Object userIDo = session.getAttribute(UploaderConsts.USERID_FIELD_NAME);
    final Object uibo = session.getAttribute(UploaderConsts.attrUIB);

    // The user must be logged in for us to contemplate us doing
    // anything...
    //
    // If no valid user ID is held,
    // Do nothing, and direct the user back to the main upload page.
    //
    // We are initially very wary about what we retrieve from
    // the session in case (for example) it has been persisted
    // and restored and the code requirements have changed.
    if(!(userIDo instanceof String) ||
       !ExhibitName.validAuthorSyntax((String) userIDo))
        {
        %>
<p>Sorry, I don't recognise your ID.
   Do you have cookies turned off?</p>
</body></html>
        <%
        return;
        }
    final String userID = (String) userIDo;

    if(!(uibo instanceof UploadInfoBean))
        {
        %>
<p>Sorry, I could not find any exhibit info in the session.
   Please contact the Gallery maintainer.
</p>
</body></html>
        <%
        return;
        }
    final UploadInfoBean uib = (UploadInfoBean) uibo;

    // Check that there is no upload currently going on by this user
    // to this filesystem, abort if there is.
    // Measure filespace use within this to prevent gross attacks on
    // filespace usage.

    // Unique lock file for this user.
    // The file is (a) clearly a temporary file (b) not a valid exibit file.
    final File lockFileName = new File(uploadDirS,
                                FileTools.F_tmpPrefix + userID + ".uploading");
    if(!lockFileName.createNewFile())
        {
        %>
        <p>Sorry, you already seem to have an upload in progress.</p>
        <%
        }
    else try
        {
        // Find out how much file space is already being used.
        AllExhibitImmutableData aeidUploaded;
        try {
            aeidUploaded = ExhibitDataFileSource._getAllExhibitImmutableData(uploadDirS, -1L, false);
            }
        catch(Exception e)
            {
            %>
<p>Sorry, uploads are not currently possible.
   Please contact the maintainer.</p>
</body></html>
            <%
            e.printStackTrace();
            return;
            }

        // Set enough information to double-check that the name is still valid.
        uib.setAep(aep);
        uib.setUploadAeid(aeidUploaded);

        if(!uib.enoughValidUniqueInfo())
            {
            %>
<p>Sorry, I could not find enough exhibit info in the session.
   Please contact the Gallery maintainer.
</p>
</body></html>
            <%
            return;
            }

        // Compute size consumed in total and by each user.
        final long uploadedTotal = aeidUploaded.computeFileSpaceBytes(null);
        final long uploadedByThisUser = aeidUploaded.computeFileSpaceBytes(userID);
        final int uploadSpaceLeft = Math.max(0, (int)
            Math.min(LocalProps.getUploadMaxBytesPerUser() - uploadedByThisUser,
                     LocalProps.getUploadMaxBytesTotal() - uploadedTotal));

        if(uploadSpaceLeft < UploaderConsts.MIN_UPLOAD_SPACE_BYTES)
            {
            %>
<p>Sorry, you do not have enough space left to do an upload at all.</p>
            <%
            }
        else if(!request.getContentType().startsWith("multipart/form-data"))
            {
            %>
<p>Sorry, the form seems to have been wrongly posted
   with the Content-Type ``<%= request.getContentType() %>''.
   Please contact the Gallery maintainer.
</p>
            <%
            }
        else
            {
            // OK, everything is OK, can proceed!
            %><p>Uploading...<br><%
            %><%= uib.getFullName() %></p><%
            out.flush();

            try
                {
                final File destinationDir = new File(uploadDirS);

                final long uploadStart = System.currentTimeMillis();
                final String category = uib.getCategory();
                if(aep.getCategoryExhibitCounts().get(category) == null)
                    {
                    %><p>Exhibit category unrecognised, sorry.</p><%
                    return;
                    }

                // Make sure the category directory exists
                // else try to create it...
                final File categoryDir = new File(destinationDir, category);

                // Actually do the upload.
                final SimpleLoggerIF logger = new WebUtils.ServletLogger(application);
                HTTPUploaderUtils.doUpload(request,
                                         uib,
                                         uploadSpaceLeft,
                                         destinationDir,
                                         logger);

                // Generate the final target filename.
                // Omit any intermediate path supplied.
                final File targetDescName = new File(categoryDir,
                                                     uib.getName(false) + CoreConsts.DESCRIPTION_FILE_SUFFIX);

                // If there was a description, attempt to save it now.
                final String desc = uib.getDescription();
                if((desc != null) && (desc.length() > 0))
                    {
                    FileTools.replacePublishedFile(targetDescName.getPath(),
                                                   desc.getBytes(CoreConsts.FILE_ENCODING_8859_1),
                                                   false, // Be noisy for now...
                                                   true); // Use pure-Java mechanisms.
                    }

                final long uploadEnd = System.currentTimeMillis();
                final long uploadTime = uploadEnd - uploadStart;
                final File exhibitFile = new File(destinationDir, uib.getFullName());
                final long uploadLength = exhibitFile.length();

                // Compute effective upload bandwidth.
                final long uploadBps = ((uploadLength * 1000) / Math.max(1, uploadTime));

                // If we get here then it's probably all worked!
                %><strong>Upload done!</strong><br><%
                %><%= uploadTime %>ms, <%= uploadLength %> bytes, <%= uploadBps %> bytes/sec.<%
                out.flush();

                // Now auto-increment the number-in-series counter if not zero.
                if(uib.getNumber() > 0)
                    {
                    %><p>Incrementing number-in-series value to <%= uib.incNumberIfNonZero() %>.</p><%
                    session.setAttribute(UploaderConsts.attrUIB, uib);
                    }
                }
            catch(Exception e)
                {
                System.err.println("UPLOAD ERROR: " + uib.getFullName() + ": " + e.getMessage());
                e.printStackTrace();
                %>
<p>Sorry, I encountered an error during upload:
   <strong><%= e.getMessage() %></strong>.</p>
<p>Your upload may or may not have succeeded; please contact the maintainer.</p>
                <%
                // Fall through to normal end of page.
                }
            }

        } // End try from upload.
    finally
        {
        // Remove lock.
        if(!lockFileName.delete())
            {
            System.err.println("Could not remove upload lock file " + lockFileName);
            %>
            <p>Sorry, your next upload may not succeed.
               Contact the site maintainer please.</p>
            <%
            }
        }
%>

<p>Return to the <a href="upload.jsp">main upload page</a>.</p>
</body></html>