<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%><%--

BASE NODE for virtual "by author" set of trees.

This will intercept all the incoming page requests below its nount point,
so uses root-relative URIs to link other places in the site.

--%><%@ page
    import="org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.webSvr.util.*,
            java.util.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.catalogue.*,
            org.hd.d.pg2k.svrCore.props.GenProps"
    session="false"
%><%--

TIMING, LOCALE AND EXHIBIT BEANS...
--%><%@ include file="/_common/_SetPageStartTime.inc.jsp" %><%-- Note start time if not already done for standard footer.
--%><%@ include file="/_common/_localeBean.inc.jsp" %><%-- i18n support...
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...

--%><%!
    /**Map from author initials to TreeFilterBean; we add entries as they are needed.
     * We don't bother to check for authors disappearing,
     * since we assume that it almost never happens.
     * <p>
     * We use a Hashtable because it is thread-safe.
     * <p>
     * We hold a lock on this to make multiple operations atomic.
     */
    private static final Hashtable trees = new Hashtable();
    %><%--

--%><%
    final String nodeTitle = "All Exhibits By Author"; // FIXME: Needs i18n, etc.


    // Make the breadcrumbs.
    // Both for the top of the forest, and for each tree,
    // add a link up to the virtual collections.
    // We may add more crumbs elsewhere, and must post the crumbs to the request state.
    final Breadcrumbs crumbs = new Breadcrumbs();
    crumbs.append(new Breadcrumbs.Breadcrumb(localeBean.getLocalisedMessage("common.main.VirtualCollections"), WebConsts.VIRTUAL_COLLECTIONS_ROOT));

    // Get Generic properties.
    final GenProps gp = dataSource.getGenProps(-1);

    // Get the exhibit properties...
    final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);

    // Get the list of authors.
    final Map authors = aep.getAuthorExhibitCounts();

    // Extract tail of request containing embedded word sequence prefix,
    // ensuring that it is always non-null and
    // starts with a "/" and
    // ends with some file component (to make parsing easier)...
    String tailURI = request.getPathInfo();
    if(tailURI == null) { tailURI = "/"; }
    assert(tailURI.startsWith("/"));

    final int secondSlash = tailURI.indexOf('/', 1); // Position of second '/', else -1.
    final String auth = tailURI.substring(1,
        (secondSlash == -1) ? tailURI.length() : secondSlash);

    // If the tail URL's first component does not contain a dot,
    // then assume it was meant to be interpretted as part of an author tree.
    if((auth.length() > 0) && (auth.indexOf('.') == -1))
        {
        // First component should be a valid author,
        // else we redirect to the "not found" page.

        final Object count = authors.get(auth);
        if(count == null)
            {
            // Whoops, invalid author, so give up.
            request.setAttribute("javax.servlet.error.request_uri", tailURI); // Assume only tail part interesting...
            request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
            return;
            }

        // 1) Atomically ensure right tree filter bean for author exists in Hashtable, else create it.
        TreeFilterBean tfb;
        synchronized(trees)
            {
            tfb = (TreeFilterBean) trees.get(auth);
            if(tfb == null)
                {
                // Create the filter bean.
                tfb = new TreeFilterBean();
                // By default, hold cache of computed data as this is assumed to be expensive.
                tfb.setMemorySensitiveCache(false);
                // Set the filter for the right author.
                tfb.setExpr(new FilterExpr(null, new BuiltInFilters.filtByAuthor(new String[]{auth})));
                // Store the new bean in the Hashtable.
                trees.put(auth, tfb);
                }
            }

        // 2) Create synthetic baseURI and extract List and pass to Tree display.
        final String baseURI = WebConsts.VIRTUAL_COLLECTIONS_BYAUTHOR_ROOT + auth + '/';
        // Convert this into a word-sequence prefix.
        String wsPrefix = "";
        try { wsPrefix = TreeFilterBean.convertTrailingURIToWordSequence(tailURI.substring(1 + auth.length())); }
        catch(IllegalArgumentException e) // User entered malformed input.
            {
            // Mark malformed input as "not found".
            //response.sendError(response.SC_NOT_FOUND);
            request.setAttribute("javax.servlet.error.request_uri", tailURI);
            request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
            return;
            }
        List<String> nodeContent = tfb.selectPrefixesAndShortNames(application, wsPrefix, null);
        boolean[] uninterestingNodes;
        Map<String,Integer> exhibitCounts;
        if(nodeContent == null)
            {
            // Try using the top node of the tree instead.
            // If this is still null, the display page will understand that it has no entries.
            // Note that the exhibit names are in short form to sort properly with the prefixes.
            nodeContent = tfb.selectPrefixesAndShortNames(application, "", null);
            uninterestingNodes = tfb.getUninterestingNodes(application, "", null);
            exhibitCounts = tfb.getExhibitCount(application, "", null);
            // Reset the prefix to match.
            wsPrefix = "";
            }
        else
            {
            uninterestingNodes = tfb.getUninterestingNodes(application, wsPrefix, null);
            exhibitCounts = tfb.getExhibitCount(application, wsPrefix, null);
            }
            
        // 302 TEMP REDIRECTS FOR NODES THAT CURRENTLY DUPLICATE THEIR PARENT NODE.
        // If we can redirect to a parent node (ie shorter URI) than this
        // that (currently) contains the same info, then we do,
        // to show shorter URLs to users and to reduce duplicate pages in SEs/caches.
        // Since the tree structure may change this must only be a temporary redirect.
        final boolean redirected = TreeFilterServletBase.redirectIfCanonicalisationNeeded(tfb, response, baseURI, wsPrefix, application, nodeContent);
        if(redirected) { return; }

        // Append a crumb back to the main "By Author" page and post to the request state for the display page to fetch.
        crumbs.append(new Breadcrumbs.Breadcrumb(nodeTitle, WebConsts.VIRTUAL_COLLECTIONS_BYAUTHOR_ROOT)); // FIXME: needs i18n, etc.
        request.setAttribute(WebConsts.STDHDR_ATTR_NAME_BREADCRUMBS, crumbs);

        // Set up description text from the author database.
        final StringBuilder desc = new StringBuilder(128);
        desc.append("<a href=\"/terms.html\">(c)</a> ").append(auth);
        final GenProps.AuthData ad = gp.getAuthData(auth);
        if(ad != null)
            {
            desc.append(": <strong>").append(ad.name).append("</strong>");

            // Insert home-page link if available.
            if(ad.www != null)
                {
                desc.append(", <a href=\"").
                     append(ad.www).
                     append("\" target=\"_top\">").
                     append(ad.www).
                     append("</a>");
                }

            // Insert email link if available
            // AND the visitor doesn't look to be a SPAMmer/spider...
            // (SPAMmers may hunt for email addresses in SE cached pages...)
            if(ad.email != null)
                {
			    final boolean dontShowMail = WebUtils.requestProbablyFromSpider(request) ||
			        (null != AddrTools.isAddrBlackholed(gp.getDNSRBLs(), InetAddress.getByName(request.getRemoteAddr()), false));
                if(!dontShowMail)
	                {
	                desc.append(", <a href=\"mailto:").
	                     append(ad.email).
	                     append("\">").
	                     append(ad.email).
	                     append("</a>");
                     }
                }

            // End sentence...
            desc.append('.');

            // Insert description sentence if available.
            if(ad.desc != null)
                { desc.append(' ').append(ad.desc); }
            }

        // Redirect to display JSP, having stored the data we need in attributes at request level.
        request.setAttribute(WebConsts.BETDJ_BASE_URI_ATTR_NAME, baseURI);
        request.setAttribute(WebConsts.BETDJ_LIST_ATTR_NAME, nodeContent);
        request.setAttribute(WebConsts.BETDJ_LIST_UNIN_NAME, uninterestingNodes);
        request.setAttribute(WebConsts.BETDJ_LIST_SNEXCOUNT_NAME, exhibitCounts);
        request.setAttribute(WebConsts.BETDJ_PRWSEQ_ATTR_NAME, wsPrefix);
        request.setAttribute(WebConsts.BETDJ_TITLE_ATTR_NAME, auth + " - "  + ad.name + " (" + count + ')');
        request.setAttribute(WebConsts.BETDJ_HTML_GENERAL_DESC_ATTR_NAME, desc.toString());

        // This might be regarded as a "low-content" or "duplicate" page.
        if(WebConsts.USE_NOINDEX_ON_LOW_CONTENT_PAGES) { request.setAttribute(WebConsts.STDHDR_ATTR_NAME_NOINDEX, Boolean.TRUE); }

        request.getRequestDispatcher(WebConsts.BROWSABLE_EXHIBIT_TREE_DISPLAY_JSP).forward(request, response);
        return;
        }



    // FALL THROUGH IN ORDER TO SHOW HTML "BY AUTHOR" INDEX.
    %><%--


HEADER
--%><%
    // This might be regarded as a "low-content" or "duplicate" page.
    if(WebConsts.USE_NOINDEX_ON_LOW_CONTENT_PAGES) { request.setAttribute(WebConsts.STDHDR_ATTR_NAME_NOINDEX, Boolean.TRUE); }

    request.setAttribute(WebConsts.STDHDR_ATTR_NAME_TITLE, nodeTitle); // FIXME: i18n needed for this page.
    request.setAttribute(WebConsts.STDHDR_ATTR_NAME_BREADCRUMBS, crumbs);

%><%-- POST FOR TREE PAGE TO FIND...
--%><%@ include file="/_common/_HTMLStdHd.inc.jsp" %><%--



BODY
--%><p>Browse the exhibits by author/contributor:<ul><%-- // FIXME: needs i18n...
--%><%
    // List and link to authors in alphabetic order (and show number of exhibits by each author).
    final SortedSet ss = new TreeSet(authors.keySet());
    for(final Iterator it = ss.iterator(); it.hasNext(); )
        {
        final String a = (String) it.next();
        %><li><a href="<%= WebConsts.VIRTUAL_COLLECTIONS_BYAUTHOR_ROOT %><%= a %>/"><%= a %></a> <i>(<%= authors.get(a) %>)</i><%
        final GenProps.AuthData ad = gp.getAuthData(a);
        if(ad != null) { %>: <%= ad.name %><% }
        %></li><%-- Insert line-end for clarity. --%>
<%
        }
    %><%--
--%></ul></p><%--



FOOTER
--%><%@ include file="/_common/_HTMLStdFt.inc.jsp" %>
