<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%><%--

BASE NODE for virtual "by category" set of trees.

This will intercept all the incoming page requests below its nount point,
so uses root-relative URIs to link other places in the site.

--%><%@ page
    import="org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.webSvr.util.*,
            java.util.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.catalogue.*,
            org.hd.d.pg2k.svrCore.uploader.*"
    session="false"
%><%--

TIMING, LOCALE AND EXHIBIT BEANS...
--%><%@ include file="/_common/_SetPageStartTime.inc.jsp" %><%-- Note start time if not already done for standard footer.
--%><%@ include file="/_common/_localeBean.inc.jsp" %><%-- i18n support...
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...

--%><%
    final String nodeTitle = localeBean.getLocalisedMessage("common.virtCol.byCategory.title");


    // Make the breadcrumbs.
    // Both for the top of the forest, and for each tree,
    // add a link up to the virtual collections.
    // We may add more crumbs elsewhere, and must post the crumbs to the request state.
    final Breadcrumbs crumbs = new Breadcrumbs();
    crumbs.append(new Breadcrumbs.Breadcrumb(localeBean.getLocalisedMessage("common.main.VirtualCollections"), WebConsts.VIRTUAL_COLLECTIONS_ROOT));


    // Get the exhibit properties...
    final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);

    // Get the list of categories.
    final Map<String,Integer> categories = aep.getCategoryExhibitCounts();

    // Extract tail of request containing embedded word sequence prefix,
    // ensuring that it is always non-null and
    // starts with a "/" and
    // ends with some file component (to make parsing easier)...
    String tailURI = request.getPathInfo();
    if(tailURI == null) { tailURI = "/"; }
    //assert(tailURL.startsWith("/"));

    final int secondSlash = tailURI.indexOf('/', 1); // Position of second '/', else -1.
    final String category = tailURI.substring(1,
        (secondSlash == -1) ? tailURI.length() : secondSlash);

    // If the tail URL's first component does not contain a dot,
    // then assume it was meant to be interpretted as part of a category tree.
    if((category.length() > 0) && (category.indexOf('.') == -1))
        {
        // First component should be a valid category,
        // else we redirect to the "not found" page.

        // Whoops, invalid category, so give up.
        if(categories.get(category) == null)
            {
            request.setAttribute("javax.servlet.error.request_uri", tailURI); // Assume only tail part interesting...
            request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
            return;
            }

        // 1) Get tree filter bean for category (atomically); never null.
        final TreeFilterBean tfb = WebUtils.getCategoryTreeFilterBean(dataSource, category);

        // 2) Create synthetic baseURI and extract List and pass to Tree display.
        final String baseURI = WebConsts.VIRTUAL_COLLECTIONS_BYCATEGORY_ROOT + category + '/';
        // Convert this into a word-sequence prefix.
        String wsPrefix = "";
        try { wsPrefix = TreeFilterBean.convertTrailingURIToWordSequence(tailURI.substring(1 + category.length())); }
        catch(final IllegalArgumentException e) // User entered malformed input.
            {
            // Mark malformed input as "not found".
            //response.sendError(response.SC_NOT_FOUND);
            request.setAttribute("javax.servlet.error.request_uri", tailURI);
            request.getRequestDispatcher(WebConsts.NOT_FOUND_PAGE).forward(request, response);
            return;
            }
        List<String> nodeContent = tfb.selectPrefixesAndShortNames(application, wsPrefix, null);
        boolean[] uninterestingNodes;
        Map<String,Integer> exhibitCounts;
        if(nodeContent == null)
            {
            // Try using the top node of the tree instead.
            // If this is still null, the display page will understand that it has no entries.
            // Note that the exhibit names are in short form to sort properly with the prefixes.
            nodeContent = tfb.selectPrefixesAndShortNames(application, "", null);
            uninterestingNodes = tfb.getUninterestingNodes(application, "", null);
            exhibitCounts = tfb.getExhibitCount(application, "", null);
            // Reset the prefix to match.
            wsPrefix = "";
            }
        else
            {
            uninterestingNodes = tfb.getUninterestingNodes(application, wsPrefix, null);
            exhibitCounts = tfb.getExhibitCount(application, wsPrefix, null);
            }
            
        // 302 TEMP REDIRECTS FOR NODES THAT CURRENTLY DUPLICATE THEIR PARENT NODE.
        // If we can redirect to a parent node (ie shorter URI) than this
        // that (currently) contains the same info, then we do,
        // to show shorter URLs to users and to reduce duplicate pages in SEs/caches.
        // Since the tree structure may change this must only be a temporary redirect.
        final boolean redirected = TreeFilterServletBase.redirectIfCanonicalisationNeeded(tfb, response, baseURI, wsPrefix, application, nodeContent);
        if(redirected) { return; }

        // We'll see if there is descriptive text for this category.
        final String i18nName = UploaderConsts.SECTION_DESC_PROPNAME_PREFIX + category;
        final String desc = localeBean.getLocalisedMessage(i18nName);
        // We show the localised version if it exists and isn't just in a different case.
        final String cDesc = localeBean.getLocalisedMessage("common.cat.sectionDescription");
        // Compute the title...
        final StringBuilder title = new StringBuilder(GenUtils.computeSectionTitle(category, localeBean));
        // Compute the description.
        final StringBuilder descSb = new StringBuilder(desc.length() + cDesc.length() + 9);
        if(!desc.equals(i18nName))
            { descSb.append("<h2>").append(cDesc).append("</h2>").append(desc); }
        // Append tree-based information (localised) if we have any.
        // Cannot do this at top of tree.
        if(wsPrefix.endsWith("-"))
            {
            // Create a syntactically-valid exhibit name
            // to do a lookup of tree description information.
            final String fakeExhibit = category + '/' + wsPrefix + "A.a";
            final String treeDesc = GenUtils.getLocalisedTreeDesc(aep,
                                                                  fakeExhibit,
                                                                  localeBean,
                                                                  true, true, true, false);
            if(treeDesc.length() > 0)
                { descSb.append("<p><ul>").append(treeDesc).append("</ul></p>"); }

            // Compute specific AKAs to add to title.
            final String specificAKA = GenUtils.getLocalisedTreeDesc(aep,
                                                                     fakeExhibit,
                                                                     localeBean,
                                                                     false, true, false, true);
            if(specificAKA.length() > 0)
                {
                title.append(" - ");
                title.append(specificAKA);
                }
            }
        // If we have some real descriptive test,
        // then set the attribute to have it displayed.
        if(descSb.length() > 0)
            { request.setAttribute(WebConsts.BETDJ_HTML_GENERAL_DESC_ATTR_NAME, descSb.toString()); }

        // Append a crumb back to the main "By Category" page and post to the request state for the display page to fetch.
        crumbs.append(new Breadcrumbs.Breadcrumb(nodeTitle, WebConsts.VIRTUAL_COLLECTIONS_BYCATEGORY_ROOT)); // FIXME: needs i18n, etc.
        request.setAttribute(WebConsts.STDHDR_ATTR_NAME_BREADCRUMBS, crumbs);

        // Redirect to display JSP, having stored the data we need in attributes at request level.
        request.setAttribute(WebConsts.BETDJ_BASE_URI_ATTR_NAME, baseURI);
        request.setAttribute(WebConsts.BETDJ_LIST_ATTR_NAME, nodeContent);
        request.setAttribute(WebConsts.BETDJ_LIST_UNIN_NAME, uninterestingNodes);
        request.setAttribute(WebConsts.BETDJ_LIST_SNEXCOUNT_NAME, exhibitCounts);
        request.setAttribute(WebConsts.BETDJ_PRWSEQ_ATTR_NAME, wsPrefix);
        request.setAttribute(WebConsts.BETDJ_TITLE_ATTR_NAME, title.toString());
        request.setAttribute(WebConsts.BETDJ_COMMON_SECTION, category);

        // This might be regarded as a "low-content" or "duplicate" page.
        if(WebConsts.USE_NOINDEX_ON_LOW_CONTENT_PAGES) { request.setAttribute(WebConsts.STDHDR_ATTR_NAME_NOINDEX, Boolean.TRUE); }

        request.getRequestDispatcher(WebConsts.BROWSABLE_EXHIBIT_TREE_DISPLAY_JSP).forward(request, response);
        return;
        }



    // FALL THROUGH IN ORDER TO SHOW HTML "BY CATEGORY" INDEX.
    %><%--


HEADER
--%><%
    // This might be regarded as a "low-content" or "duplicate" page.
    if(WebConsts.USE_NOINDEX_ON_LOW_CONTENT_PAGES) { request.setAttribute(WebConsts.STDHDR_ATTR_NAME_NOINDEX, Boolean.TRUE); }

    request.setAttribute(WebConsts.STDHDR_ATTR_NAME_TITLE, nodeTitle); // FIXME: i18n needed for this page.
    request.setAttribute(WebConsts.STDHDR_ATTR_NAME_BREADCRUMBS, crumbs);
    
%><%-- POST FOR TREE PAGE TO FIND...
--%><%@ include file="/_common/_HTMLStdHd.inc.jsp" %><%--



BODY
--%><p>Browse the exhibits by category:<ul><%-- // FIXME: needs i18n...
--%><%= WebUtils.getCategoryListSortedAsHTML(dataSource, localeBean, true) %><%--
--%></ul></p><%--



FOOTER
--%><%@ include file="/_common/_HTMLStdFt.inc.jsp" %>
