<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%@ page
    import="org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.webSvr.util.*,
            java.util.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.virtualHosts.VirtualHosts,
            org.hd.d.pg2k.svrCore.MIME.ExhibitMIME,
            org.hd.d.pg2k.webSvr.ads.*,
            org.hd.d.pg2k.svrCore.uploader.UploaderUtils"
    session="false"
    buffer="32kb" %><%-- As at 20050124 front page is typically just over 16kB; 32kB buffer allows write in one block.


IF THIS IS THE OPENING PAGE OF A VIRTUAL HOST, REDIRECT IT APPROPRIATELY.
--%><%
    final VirtualHosts.VirtualHost vHostHomePage =
        VirtualHosts.getVirtualHostDetails(request.getServerName(),
                                           null); // Do not attempt URI match.
    if(vHostHomePage != null)
        {
        final String redirectURL = vHostHomePage.virtualRootDir;
        if(redirectURL != null)
            {
            // redirectURL is guaranteed not to point back to this page...
            response.sendRedirect(redirectURL);
            return;
            }
        }
%><%--


FORCE ALIASES OF HOME PAGE TO CANONICAL URI
--%><%
    final String requestURI = request.getRequestURI();
    final String canonicalHomePageURI = "/index.jsp";
    if(!canonicalHomePageURI.equals(requestURI))
        {
        application.log("Redirecting home page from " + requestURI);
        try {
            response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
            response.setHeader("Location", canonicalHomePageURI);
            return;
            }
        catch(final Exception e) { }
        }
%><%--


LOCALE AND EXHIBIT BEANS...
--%><%@ include file="/_common/_localeBean.inc.jsp" %><%-- i18n support...
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...
--%><%@ include file="/_common/_FullSortedExhibitSetBean.inc.jsp" %><%-- Get sorted-exhibits bean...


EXPIRY/CACHEING
Force a relatively short expiry for this potentially-"fresh" page, even for spiders.
We don't publickly cache pages generated within a session
(eg with a non-default locale) to avoid confusion.
--%><%
    response.setHeader("Cache-Control",
            ((request.getSession(false) == null) ? "public," : "private,") +
            "max-age="+(WebConsts.DEFAULT_PAGE_CACHE_MS/2000));
    response.setDateHeader("Expires", System.currentTimeMillis() +
            WebConsts.DEFAULT_PAGE_CACHE_MS/2);
%><%--


HEADER
--%><% request.setAttribute(WebConsts.STDHDR_ATTR_NAME_TITLE, localeBean.getLocalisedMessage("org.hd.pg2k.simpleWelcome")); %><%--
--%><% request.setAttribute(WebConsts.STDHDR_ATTR_NAME_SUBTITLE, localeBean.getLocalisedMessage("org.hd.pg2k.2ndWelcome")); %><%--
--%><%@ include file="/_common/_HTMLStdHd.inc.jsp" %><%--


BODY
--%><%
    // Get the exhibit properties...
    final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1L);

    // Count the exhibits,
    // in particular in case there are temporarily zero during start-up.
    // (In which case we omit the intro para entirely.)
    final int nExhibits = aep.aeid.size();
    final List items = FullSortedExhibitSet.select(application);
    if((nExhibits > 1) && (items.size() > 0)) // Ultra-cautious!
        {
        // Insert links to catalogue pages to give spiders a good entree, as well as humans!
        final String first = (String) items.get(0);
        final String last = (String) items.get(items.size() - 1);
        final SortedSet<String> attrWords = ExhibitAttrUtils.getAttrWords().getAttrWordsSortedSet();
        %><p><%= localeBean.getLocalisedMessage("common.main.intro",
            nExhibits,
            "<a href=\"" + WebUtils.makeCatPageRRURL(first, WebConsts.F_secondary_generated_HTML_suffix) +"\">" + WebUtils.hyphenateHTMLText(ExhibitName.getMainWordsComponent(first, attrWords)) + "</a>",
            "<a href=\"" + WebUtils.makeCatPageRRURL(last, WebConsts.F_secondary_generated_HTML_suffix) +"\">" + WebUtils.hyphenateHTMLText(ExhibitName.getMainWordsComponent(last, attrWords)) + "</a>")
            %></p><%
        }
%>

<p><%= localeBean.getLocalisedMessage("common.main.followon") %></p>

<!--[if IE]> <%-- IE-only add-to-favo[u]rites link... --%>
<p><a href="?action=bm" onClick="window.external.AddFavorite('http://<%= CoreConsts.MAIN_DATA_HOST %>/','<%= localeBean.getLocalisedMessage("common.addToFavourites.IE.desc") %>');"><strong><span class="favorites"><%= localeBean.getLocalisedMessage("common.addToFavourites.IE") %></span></strong></a></p>
<![endif]-->

<p align="center"><%= localeBean.getLocalisedMessage("common.main.quote") %></p>

<%
    final boolean finishASAP = siteIsBusy || (WebConsts.FIRST_PAGE_AFAP && (WebUtils.requestProbablyReferredFromExternalSite(request) != null));

    // Find a "popular" (JPEG) exhibit to show...
    // (If we're not massively busy.)
    String popularExhibitName = null;
    if(!siteIsBusy)
        {
        final String exhibitName = WebUtils.getPopularExhibit(dataSource,
                              ExhibitMIME.getParamsByType(ExhibitMIME.ET_JPEG));
        popularExhibitName = exhibitName;
        if(exhibitName != null)
            {
            // Try and get a thumbnail, which must be an inlineable image.
            final java.awt.Dimension thumbnailXyDim = new java.awt.Dimension();
            String thumbnailRRURL = WebUtils.makeHTMLInlineImageThumbnailURL(
                dataSource, exhibitName,
                true, // Make a standard thumbnail.
                true, // Make relative URL.
                thumbnailXyDim,
                finishASAP); // Force thumbnail generation (if site not busy)...
            // Fallback to small thumbnail if we have to...
            if(thumbnailRRURL == null) // If that fails, try for the small thumbnail...
                {
                thumbnailRRURL = WebUtils.makeHTMLInlineImageThumbnailURL(
                    dataSource, exhibitName,
                    false, // Make a small thumbnail.
                    true, // Make relative URL.
                    thumbnailXyDim,
                    finishASAP); // Force thumbnail generation (if site not busy)...
                }
            if(thumbnailRRURL != null)
                {
                // Compute specific AKA, with links if available!
                final String specificAKA = GenUtils.getLocalisedTreeDesc(aep,
                                                                         exhibitName,
                                                                         localeBean,
                                                     false, true, true, true);
// Avoid the rotating "eye-candy" content confusing Google's ad targetting.
if(AdConsts.AD_SUPPORT) { %>
<!-- google_ad_section_start(weight=ignore) -->
<% } %>
<center>
<h2><%= localeBean.getLocalisedMessage("common.main.PopJPEG") %></h2>
<p>
<table border=<%= PageSkinUtils.GENERIC_TN_BORDER_WIDTH_PIXELS %>><tr><td><a href="<%= WebUtils.makeCatPageRRURL(exhibitName, WebConsts.F_secondary_generated_HTML_suffix) %>"><img src="<%= thumbnailRRURL %>" width=<%= thumbnailXyDim.width %> height=<%= thumbnailXyDim.height %> alt=<%= UploaderUtils.quoteHTMLArg(localeBean.getLocalisedMessage("common.thumbnail") ) %><%= WebConsts.TN_IMG_TAG_EXTRA_ATTR %>></a></td></tr></table><br />
<a href="<%= WebUtils.makeCatPageRRURL(exhibitName, WebConsts.F_secondary_generated_HTML_suffix) %>"><%= WebUtils.hyphenateHTMLText(ExhibitName.getFileComponent(exhibitName)) %></a><br />
<% if(specificAKA.length() > 0) { %><i>(<%= specificAKA %>)</i><% } %>
</p>
</center>
<% if(AdConsts.AD_SUPPORT) { %>
<!-- google_ad_section_end -->
<% } %>
<%
                }
            }
        }
%><%--


PICASA TEXT LINK
DISABLED: DELAYS LOADING OF FRONT PAGE TOO MUCH
<br /><br /><center>
<%= localeBean.getLocalisedMessage("common.ad.textlink.FrontPagePicasaTextLink") %>
<br /></center>



FLUSH ANY CONTENT SO FAR TO THE USER...
--%><% if(finishASAP) { out.flush(); } %><%--


COLLECTIONS TABLE
--%><center><h2><%= localeBean.getLocalisedMessage("common.main.Collections") %></h2></center>
<p><table width="100%" border=1 cols=2 align=center>
<tr>
<th width=<%= PageSkinUtils.GENERIC_WIDTH_PX/2 - 2 %>><%= localeBean.getLocalisedMessage("common.main.Categories") %></th>
<th width=<%= PageSkinUtils.GENERIC_WIDTH_PX/4 - 2 %>><a href="<%= WebConsts.VIRTUAL_COLLECTIONS_NEW_PAGE %>"><%= localeBean.getLocalisedMessage("common.selection.newExhibits.title") %></a></th>
<th width=<%= PageSkinUtils.GENERIC_WIDTH_PX/4 - 2 %>><a href="<%= WebConsts.VIRTUAL_COLLECTIONS_BEST_PAGE %>"><%= localeBean.getLocalisedMessage("common.selection.bestExhibits.title") %></a></th>
</tr>
<tr valign=top>
<td><p><%= WebUtils.getCategoryListSortedAsHTML(dataSource, localeBean, false) %></p>
<p><%= localeBean.getLocalisedMessage("common.main.categories.followon", WebConsts.VIRTUAL_COLLECTIONS_ROOT, localeBean.getLocalisedMessage("common.main.VirtualCollections"), "/_cat/doHTMLSearch.jsp", localeBean.getLocalisedMessage("common.cat.search")) %></p>
</td>
<td align=center><%-- Get up to 5 new-ish exhibits to show. --%>
<%= HTMLThumbnailInsertGenerators.makeHTMLExhibitColumnDisplay(HTMLThumbnailInsertGenerators.getNewExhibitSelection(
    application,
    5,
    (Rnd.fastRnd.nextBoolean() ? null : Rnd.fastRnd)), // Alternate between the very newest, and a "new-ish" selection...
  PageSkinUtils.GENERIC_WIDTH_PX/4 - 4,
  PageSkinUtils.GENERIC_WIDTH_PX,
  dataSource,
  localeBean,
  finishASAP) %>
</td>
<td align=center><%-- Get up to 5 "good"/"best" exhibits to show. --%>
<%= HTMLThumbnailInsertGenerators.makeHTMLExhibitColumnDisplay(HTMLThumbnailInsertGenerators.getBestExhibitSelection(
    application,
    5,
    (Rnd.fastRnd.nextBoolean() ? null : Rnd.fastRnd)), // Alternate between the very best, and a "good" selection...
  PageSkinUtils.GENERIC_WIDTH_PX/4 - 4,
  PageSkinUtils.GENERIC_WIDTH_PX,
  dataSource,
  localeBean,
  finishASAP) %>
</td>
</table></p>

<%
// Try and show a link to another popular exhibit, of any type,
// but not the same as the popular JPEG already shown (if any).
// Skip this on a busy server, but do show on the first page a visitor sees.
if(!siteIsBusy)
    {
    final String exhibitName = WebUtils.getPopularExhibit(dataSource, null,
        (popularExhibitName == null) ? null : Collections.singleton(popularExhibitName));
    if(exhibitName != null)
        {
        %><p><strong><%= localeBean.getLocalisedMessage("common.tower.Popular") %></strong>: <%
        %><a href="<%= WebUtils.makeCatPageRRURL(exhibitName, WebConsts.F_secondary_generated_HTML_suffix) %>"><%= WebUtils.hyphenateHTMLText(ExhibitName.getFileComponent(exhibitName)) %></a><%
        %></p><%
        }
    }
%>

<center><h2><%= localeBean.getLocalisedMessage("common.main.news.heading") %></h2></center>

<%  // Insert emphasised news-flash HTML text (wrapped in a div) if available.  No i18n.
    final String newsflash = WebUtils.getNewsflashHTML(dataSource.getGenProps(-1));
    if(newsflash.length() > 0)
        { %><div class="newsflash"><p><strong><%= newsflash %></strong></p></div><% }
%>

<%= localeBean.getLocalisedMessage("common.main.news.body") %>

<center><h2><%= localeBean.getLocalisedMessage("common.main.background.heading") %></h2></center>
<%= localeBean.getLocalisedMessage("common.main.background.body") %>

<center><h2><%= localeBean.getLocalisedMessage("common.main.coda.heading") %></h2></center>
<%= localeBean.getLocalisedMessage("common.main.coda.body") %>

<%--


FOOTER
--%><%@ include file="/_common/_HTMLStdFt.inc.jsp" %>