<%--
Copyright (c) 1996-2007, Damon Hart-Davis
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--%>
<%--`Not found' page in HTML.

    We also use this to intercept data-dependent and legacy locations
    for old gallery.hd.org and www.hd.org/Damon/photos eg flowers/index.html
    and exhibits available directly under the root by their full name.

    Note that in all cases to handle these legacy paths
    we look for a URI whose first component is a valid section name/directory.

    The exhibit access is redirected to the new location,
    preferably with the client being given a permanent redirection status.

--%><%@ page
    contentType="text/html;charset=UTF-8"
    import="java.io.*,
            java.net.*,
            java.text.*,
            java.util.*,
            org.hd.d.pg2k.svrCore.*,
            org.hd.d.pg2k.svrCore.props.*,
            org.hd.d.pg2k.webSvr.exhibit.*,
            org.hd.d.pg2k.webSvr.util.*,
            ORG.hd.d.IsDebug"
    session="false"
    buffer="16kb" %><%--
--%><%@ include file="/_common/_localeBean.inc.jsp" %><%-- Get locale bean set up...
--%><%@ include file="/_common/_exhibitBean.inc.jsp" %><%-- Get exhibit data bean set up...

--%><%
if(IsDebug.isDebug) { System.err.println("NOT FOUND: starting... "); }

    // CHECK FOR AND HANDLE LEGACY URIs.

    // We have to be especially careful when redirecting to
    // external display JSPs in case they need to generate a not-found response.
    // To avoid indefinite recursion (and an ugly error)
    // we set a flag to ourself that we have already visited this page once
    // and drop straight through if we find it to be set.
    final String recursionStopAttrName = "org.hd.pg2k.notFound.stoprecursion";
    final Object errURI = request.getAttribute("javax.servlet.error.request_uri");
    if((request.getAttribute(recursionStopAttrName) == null) && // Make sure recursion flag not set...
        (errURI instanceof String)) // ...and error URI is present for us to examine.
        {
        // Prevent recursion...
        request.setAttribute(recursionStopAttrName, Boolean.TRUE);

if(IsDebug.isDebug) { System.err.println("NOT FOUND: checking for legacy URI: " + errURI); }

        // ROBOTS PAGE: LET SPIDERS IN QUICKLY!
        // Very quick check if this is the "robot-exclusion" page for a spider.
        // By default, and to save CPU, immediately generate error 404
        // so as to allow full site spidering.
        // If an explicit 404 page is provided this won't get called.
        // However, if the host name requested is a deprecated alias,
        // and iff we are a not a mirror or this IS NOT a mirror name (eg an old mirror alias),
        // or if this is a "bad" client (eg using a blacklisted IP),
        // then instead we serve a "go away" robots.txt forbidding all spidering.
        //
        // However, server a GO AWAY! page to bad clients!
        if(WebConsts.ROBOTS_PAGE.equals(errURI))
            {
            final GenProps gp = dataSource.getGenProps(-1);
            // Check for "bad" client.
            final String addrBlackholed = AddrTools.isAddrBlackholed(gp.getDNSRBLs(), InetAddress.getByName(request.getRemoteAddr()), false);
            final boolean badClient = (addrBlackholed != null);
            // Check if deprecated/non-standard name has been used.
            final String serverName = request.getServerName();
            // Our preferred form of the hostname is all-lower-case.
            final String serverNameLC = serverName.toLowerCase();
            final String normalisedServerName =
                HostUtils.normaliseVirtualHostName(serverNameLC);
            final VirtualHosts.VirtualHost vHost =
                VirtualHosts.getVirtualHostDetails(serverNameLC, null);
            final boolean badHostname = (vHost == null);
            if(badClient || badHostname ||
                (!serverNameLC.equals(vHost.normalisedName) /* Not a primary/preferred name. */ && 
                	((LocalProps.getMirrorTag() == null) || !HostUtils.isMirrorName(serverNameLC)) /* Not a mirror name, or we're not a mirror... */ ))
                {
                getServletContext().log("WARNING: serving 'GO AWAY' robots.txt to client="+request.getRemoteAddr()+" for serverName="+serverName);
                // Allow this robots.txt to be cached for a short while.
                // This may partially protect us from broken bots behind less-broken proxies.
                // Cache for a few hours (but much less than a day).
                final long rtCacheTime_ms = 6 * 3600 * 1000L; // 6 hours.
                response.setStatus(HttpServletResponse.SC_OK); // Mark this as OK; no longer an error.
                response.setHeader("Cache-Control", "max-age="+(rtCacheTime_ms/1000));
                response.setDateHeader("Expires", System.currentTimeMillis() + rtCacheTime_ms);

                // Set the correct content type (7-bit ASCII plain text).
                response.setContentType("text/plain;charset=US-ASCII");

                // By default all robots are forbidden everything.
                %>
# We don't think that you should be spidering this URL at all.
User-agent: *
Disallow: /
                <%

                // Always allow the Google AdSense/AW bots in if ads are enabled!
                if(AdConsts.AD_SUPPORT)
                    {
                %>

User-agent: Mediapartners-Google
Disallow:
User-agent: Adsbot-Google
Disallow:
                <%
                    }

                return;
                }

if(IsDebug.isDebug) { System.err.println("NOT FOUND: robots page answered with simple 404."); }
            response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            return;
            }

        // Intercept the "failing" URI and check for this being a "legacy" path,
        // whose first component is valid section name/directory.
        // We assume that this path will start with a '/'.
        final String reqURI = (String) errURI;

        // We may yet drop through to the standard not-found section
        // if an invalid URL is presented.

        // Compute first component of path quickly and safely.
        final int secondSlash = reqURI.indexOf('/', 1); // Position of second '/', else -1.
        final String category = reqURI.substring(1,
            (secondSlash == -1) ? reqURI.length() : secondSlash);
//System.err.println("NOT FOUND: first path component: " + firstComponent);

        final AllExhibitProperties aep = dataSource.getAllExhibitProperties(-1);

        if((reqURI.length() > 1) &&             // Must be at least "/a".
            (reqURI.charAt(0) == '/') &&        // Must start with "/".
            (category.indexOf('.') == -1) &&    // No dot in first component, if any.

            // Must start with valid section dir/name to be considered further, else fall through.
            (null != aep.getCategoryExhibitCounts().get(category)))
            {
if(IsDebug.isDebug) { System.err.println("NOT FOUND: possible legacy section or exhibit..."); }

            // If URI stripped of first slash is an exact extant exhibit name,
            // then do a (permanent) redirect to the new served location.
            // Hopefully spiders will remember the new location for efficiency.
            final String putativeExhibitName = reqURI.substring(1);
            if(null != aep.aeid.getStaticAttr(putativeExhibitName))
                {
if(IsDebug.isDebug) { System.err.println("NOT FOUND: redirecting exhibit from legacy location: " + reqURI); }
                response.setStatus(HttpServletResponse.SC_MOVED_PERMANENTLY);
                response.setHeader("Location", "/" + WebConsts.BASE_PATH_EXHIBITS + reqURI);
                return;
                }

            // Assume anything else is a "by category" URI.
            // Use a temporary redirect to keep this canonical form alive.
            response.sendRedirect(WebConsts.VIRTUAL_COLLECTIONS_BYCATEGORY_ROOT + reqURI);
            return;
            }

        // Fall through for unrecognised URI...
if(IsDebug.isDebug) { System.err.println("NOT FOUND: falling through for: " + reqURI); }
        }

    %><%--
--%><jsp:useBean id="catPageBean"
             class="org.hd.d.pg2k.webSvr.catalogue.SearchPageJavaBean"
             scope="page"/><%-- Create the backing JavaBean
--%><% catPageBean.setAep(dataSource.getAllExhibitProperties(-1L)); %><%-- Get the exhibit properties ASAP...
--%><%
// No cacheing!
response.setHeader("Cache-Control", "no-cache");
response.setHeader("Pragma", "no-cache");
response.setHeader("Expires", "0");
response.setStatus(HttpServletResponse.SC_NOT_FOUND);
%><%--

HEADER
--%><% request.setAttribute(WebConsts.STDHDR_ATTR_NAME_TITLE, localeBean.getLocalisedMessage("common.notFoundTitle")); %><%--
--%><%@ include file="/_common/_HTMLStdHd.inc.jsp" %><%--

BODY: "Sorry, not found..."
--%><center><%--
--%><font color=red size="+2"><%= localeBean.getLocalisedMessage("common.notFound") %></font><%---

Automatically do a catalogue seach based on:
  * The values of any GET/POST parameters (not the parameter names)
  * The non-query part of the URI
In all cases applying normal tokenisation and sanitising to the putative terms
having possibly performed URL decoding first.


SEARCH BOX
--%><% pageContext.setAttribute("searchBoxWidth", "64"); %><%--
--%><p><font size="-1"><%@ include file="/_cat/HTMLSearchBox.inc.jsp" %></font></p><%--
--%><% pageContext.removeAttribute("searchBoxWidth"); %><%--


--%><%
    if(!siteIsBusy) // Do site search ONLY IF NOT BUSY...
        { // Open scope for local variables.
        // Collect raw potential query terms, for sanitisation later.
        final StringBuilder rawQuery = new StringBuilder();

        // Get the URI for the URI that didn't work,
        // which is probably stored one of several places.
        if(errURI instanceof String) { rawQuery.append(errURI).append(' '); }
        // Look for exhibitName attr set by catalogue-handler JSP.
        final Object no = request.getAttribute("exhibitName");
        if(no instanceof String) { rawQuery.append(no).append(' '); }
        // TODO: extend for other cases...

    //    final Enumeration an = request.getAttributeNames();
    //    while(an.hasMoreElements())
    //        { System.err.println("ATTR NAME: " + an.nextElement()); }

        // ... If nothing else has been found try this request's URI
        // (if it isn't just this JSP's own URI).
        final String reqURI = request.getRequestURI();
        if(!WebConsts.NOT_FOUND_PAGE.equals(reqURI))
            { rawQuery.append(reqURI).append(' '); }

        // Add in all parameter values we can find...
        final Enumeration enNames = request.getParameterNames();
        if(enNames != null)
            {
            while(enNames.hasMoreElements())
                {
                final String values[] = request.getParameterValues((String) enNames.nextElement());
                if(values != null)
                    {
                    for(int i = 0; i < values.length; ++i)
                        {
                        rawQuery.append(values[i]).append(' ');
                        }
                    }
                }
            }

        // Now sanitise the query...
        final String saneQuery = DataSourceBean.canonicaliseSimpleByWordQuery(rawQuery.toString(),
                WebConsts.MAX_SIMPLE_QUERY_LEN);

        final int MAX_TO_SHOW = 10;

        // Raw search results...
        catPageBean.setQ(saneQuery);
        final String searchResults[] = catPageBean.doLookup(dataSource, MAX_TO_SHOW);

        // Record the search time for later display.
        pageContext.setAttribute(WebConsts.SEARCH_TIME_MS_ATTR_NAME, new Long(catPageBean.getLastSearchTimeMs()));

        // Display results...
        final int firstPos = 0;
        // Build the search-result HTML.
        %><p><%@ include file="/_cat/HTMLSearchResults.inc.jsp" %></p><%
        } // Close scope for local variables.
    %><%--


FOOTER
--%></center><%--
--%><%@ include file="/_common/_HTMLStdFt.inc.jsp" %>
